/*
 *            Copyright (c) 1998-2000 by NETsilicon Inc.
 *
 *  This software is copyrighted by and is the sole property of 
 *  NETsilicon.  All rights, title, ownership, or other interests
 *  in the software remain the property of NETsilicon.  This
 *  software may only be used in accordance with the corresponding
 *  license agreement.  Any unauthorized use, duplication, transmission,  
 *  distribution, or disclosure of this software is expressly forbidden.
 *
 *  This Copyright notice may not be removed or modified without prior
 *  written consent of NETsilicon.
 *
 *  NETsilicon, reserves the right to modify this software
 *  without notice.
 *
 *  NETsilicon
 *  411 Waverley Oaks Road                  USA 781.647.1234
 *  Suite 227                               http://www.netsilicon.com
 *  Waltham, MA 02452                       AmericaSales@netsilicon.com
 *
 *************************************************************************
 *
 *     Module Name: dialog.c
 *         Version: 1.00
 *   Original Date: 10/06/99    
 *          Author: Li Zhen
 *   Original Date: 10-Feb-2000
 *          Author: 
 *        Language: Ansi C
 * Compile Options: 
 * Compile defines: 
 *       Libraries: 
 *    Link Options: 
 *
 *    Entry Points: customizeDialog()
 *
 * Description.
 * =======================================================================
 *  This file contains startup dialog routines. THE CODE IN THIS FILE MUST 
 *  BE CUSTOMIZED TO SUPPORT YOUR APPLICATION HARDWARE.  The default 
 *  implementation supports the NET+OS development boards
 *                                                                        
 *                                                                        
 * Edit Date/Ver   Edit Description
 * ==============  ===================================================
 * LZ   02/08/00   added "printf("APPLICATION: %s\r\n", APP_DIALOG_APP_NAME)"
 * LZ   02/16/00   changed "Copyright (c) 1999" to "Copyright (c) 2000"
 * SW   03/06/00   cast return value of getchar() to char avoid a run time error
 * LZ   04/05/00   Added support for little endian.
 * LZ   06/06/00   Added more comments for NVM.
 *                 Added code to handle no NVM case.
 * DXL  07/06/00   netosConfigDialog () return -1 if APP_DIALOG_PORT is not defined 
 * LZ   07/12/00   if !nvParams.useDhcp, then print IP addresses.
 * PMJ  10/18/00   Updated to support dynamic PPP NVRAM parameters.
 * PMJ  10/19/00   Corrected problems with mislabeling of COM1 ports to COM2
 * PMJ  10/26/00   Removed private function NAResolve_MAC in place of public API.
 * NGUR 08/13/01   Add useAppConfh.
 * PMJ  03/20/02   Added root password protection to the system dialog on modification.
 * PMJ  03/22/02   Included dialog capabilities for root password, administrators 
 *                 password, Fixed Key setting and Kerberos parameters.
 * PMJ  03/29/02   Included dialog capabilities for SNTP Servers. 
 * PMJ  04/11/02   Included dialog capabilities for the Kerberos parameters: KDC address,
 *                 username, password, and realmname.   Cleaned up some application 
 *                 issues with the dialog.
 * PMJ  04/23/02   Corrected compiler Warnings.
 * SW   04/30/02   Added GNU support for redirecting std I/O
 * DXL  08/21/002  Use API define for application configuration; 
 *                  Removed unneccessary layeers if conditions
 *
 */

#include "bsp.h"
#include "npttypes.h"
#include <bsp_api.h>

#if BSP_ENABLE_DIALOG
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "tx_api.h"
#include "appconf_api.h"
#include "boardParams.h"
#include "netosIo.h"
#include "netoserr.h"
#include "sysAccess.h"
#include "dialog.h"
#include "NASecurity.h"
#include "bsp_local.h"
#include "termios.h"
#include "narmapi.h"
#include "iam_params.h"

#if BSP_WANT_WPA
#include "wpa_api.h"
#endif

/*  For now -- a way to check that our platforms are done properly  */
#ifndef BSP_WANT_ETHERNET
#error BSP_WANT_ETHERNET not defined in bsp.h
#endif
#ifndef BSP_WANT_PPP
#error BSP_WANT_PPP not defined in bsp.h
#endif
#ifndef BSP_WANT_WIRELESS
#error BSP_WANT_WIRELESS not defined in bsp.h
#endif
#ifndef BSP_WANT_WPA
#error BSP_WANT_WPA not defined in bsp.h
#endif

#define SERIALNUMBER_LENGTH     8
#define MAX_IPADDRESS_LENGTH    15
#define NADIALOG_VERIFY         0x01
#define NADIALOG_CUSTOM_REPLY   0x02
#define NADIALOG_CUSTOM_DEF_YES 0x04
#define DIALOGUPDATED 0xFF
#define LAST_RESORT_BAUD_RATE   9600

static int fd;
static unsigned inch, outch, errch;

#ifdef NETOS_GNU_TOOLS
#include <stdio.h>
#include <unistd.h>
#include <sys/fcntl.h>  /* for open */
typedef _ssize_t (*readtype) (void *, char *, int);
typedef _ssize_t (*writetype) (void *, const char *, int);
extern void   _EXFUN(__sinit,(struct _reent *));
#define CHECK_INIT(fp) \
  do                                    \
    {                                   \
      if ((fp)->_data == 0)             \
        (fp)->_data = _REENT;           \
      if (!(fp)->_data->__sdidinit)     \
        __sinit ((fp)->_data);          \
    }                                   \
  while (0)
/* need to save cookies, and read/write only */
static void *readFn, *writeFn;
static int _infile, _outfile, _errfile;

#define SAVE_IO_FOR_DIALOG     CHECK_INIT(stdin); CHECK_INIT(stdout); CHECK_INIT(stderr);\
                readFn = stdin->_read; writeFn= stdout->_write;\
        inch = (int)stdin->_cookie; outch=(int)stdout->_cookie;errch=(int)stderr->_cookie;\
        _infile=stdin->_file; _outfile=stdout->_file; _errfile=stderr->_file;

#define RESTORE_IO               stdin->_read = (void*)readFn;stdout->_write= (void*)writeFn;\
        stdin->_cookie=(_PTR)inch; stdout->_cookie=(_PTR)outch;stderr->_cookie=(_PTR)errch;\
        stdin->_file=_infile; stdout->_file=_outfile; stderr->_file=_errfile;
#else
typedef int (*readtype) (void *, char *, int);
typedef int (*writetype) (void *, const char *, int);
#endif

#if BSP_WANT_ETHERNET
static bpDuplexLevelType kbGetDuplex(char * desc, bpDuplexLevelType setting);
#endif

#if BSP_WANT_ETHERNET || BSP_WANT_WIRELESS
static int promptForMacAddress (char *description, unsigned char *macAddress);
#endif
static int checkSerialNumber(char * buffer);

#if BSP_WANT_WIRELESS
static wln_country_code_t wln_country_code = COUNTRY_UNITED_STATES;
#endif

/*ad*
 * Determine if a baud rate is valid.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @param baudrate  baudrate to validate
 *
 * @return TRUE     baud rate is valid for dialog
 * @return FALSE    baud rate is not valid for dialog
*ad*/
static BOOLEAN isBaudRateValid(unsigned long baudrate)
{
    return ((baudrate == 1200)||(baudrate == 2400)||(baudrate == 4800)||(baudrate == 9600)||
           (baudrate == 19200)||(baudrate == 38400)||(baudrate == 57600)||(baudrate == 115200));
}

/*ad*
 * Get a baudrate that can be used for the dialog.  The selection is as follows.
 *
 * 1.  Use the baudrate passed to us if it is valid.
 * 2.  Next try the value returned by NAGetAppBspBaudRate().
 * 3.  If all else fails, use LAST_RESORT_BAUD_RATE.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @param tryThisOneFirst   suggested baud rate
 *
 * @return  baud rate to use for dialog
*ad*/
static unsigned long getUsableBaudrate(unsigned long const tryThisOneFirst)
{
    unsigned long result = LAST_RESORT_BAUD_RATE;
    
    if (isBaudRateValid(tryThisOneFirst))
    {
        result = tryThisOneFirst;
    }
    else if (isBaudRateValid(NAGetAppBspBaudRate()))
    {
        result = NAGetAppBspBaudRate();
    }
    
    return result;
}

#if BSP_WANT_ETHERNET 
/*ad*
 * Determine if the duplex setting is within range.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @param   setting     duplex setting to validate
 *
 * @return TRUE         is in range
 * @return FALSE        out of range
*ad*/
static BOOLEAN isDuplexValid(bpDuplexLevelType setting)
{
    return ((setting == bpHalfDuplex) || (setting == bpFullDuplex) || (setting == bpDefaultDuplex));
}

        

/*ad*
 * Return a text description of the duplex setting.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @param setting       duplex setting to translate to a text string
 *
 * @return "Half Duplex"    half duplex setting
 * @return "Full Duplex"    full duplex setting
 * @return "Default"        use default duplex setting of hub
 * @return "Invalid Duplex Setting" duplex setting is out of range
*ad*/
static char * getDuplexString(bpDuplexLevelType setting)
{
    char *result = "Invalid Duplex Setting";
    static char * const duplexString[] = {"half", "full", "default"};
    
    if (isDuplexValid(setting))
    {
        result = duplexString[setting];
    }
    
    return result;
}
#endif /* BSP_WANT_ETHERNET */

/*ad*
 *
 * Sets up stdin, stdout, and stderr to be redirected
 * out the APP_DIALOG_PORT if APP_DIALOG_PORT is defined.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @param     flag              0: NONBLOCK I/O
 *                              1: BLOCK I/O
 * @param     nvParamsPtr       Pointer to structure with development board parameters
 *
 * @return    >0                channel ID
 *
*ad*/
static int NAOpenDialog(int flag, devBoardParamsType * nvParamsPtr)
{
    int rc;
    unsigned long baudrate;
    struct termios tios;

#ifdef NETOS_GNU_TOOLS
    SAVE_IO_FOR_DIALOG
#elif defined(__CC_ARM)
	/* Save IO */
    inch = stdin->handle;
    outch = stdout->handle;
    errch = stderr->handle;
#else
    /* Save IO */
    inch = stdin->io_channel;
    outch = stdout->io_channel;
    errch = stderr->io_channel;
#endif

    /* open device */
    if (flag == 0)
    {
        fd = open (NAGetAppDialogPort (), (O_RDWR | O_NONBLOCK) );
    }
    else
    {
       fd = open (NAGetAppDialogPort (), O_RDWR );
    }
        
    if (fd <0)
    {
        netosFatalError("APP_DIALOG_PORT improperly defined", ERROR_CLASS_BSP, ERROR_BAD_DIALOG_PORT);
        return -1;
    }

#ifdef NETOS_GNU_TOOLS
    CHECK_INIT(stdin);
    CHECK_INIT(stdout);
    CHECK_INIT(stderr);
    
    /* need to override the following so that
       std I/O will call into DDI */
    stdout->_cookie = (_PTR)fd;
    stdout->_file =fd;
    stdin->_cookie = (_PTR)fd;
    stdin->_file = fd;
    stderr->_cookie = (_PTR)fd;
    stderr->_file = fd;
    stdin->_read = (readtype)read;
    stdout->_write = (writetype)write;
    stderr->_write = (writetype)write;
#elif defined(__CC_ARM)
    stdin->handle = fd;
    stdout->handle = fd;
    stderr->handle = fd;
#else
    stdin->io_channel = fd;
    stdout->io_channel = fd;
    stderr->io_channel = fd;
#endif
    
    baudrate = getUsableBaudrate(nvParamsPtr->baudrate);
    
        tcgetattr(fd, &tios);
        cfsetospeed(&tios, baudrate);
        rc = tcsetattr(fd, TCSANOW, &tios);
     
    if (rc < 0) 
    {
        netosFatalError("error when change baud rate in Dialog.c", ERROR_CLASS_BSP, ERROR_INVALID_DIALOG_BAUDRATE);
    }

    /* give serial driver a chance to fully initialize and stabilize */
    tx_thread_sleep(BSP_TICKS_PER_SECOND);

    return fd;
}

/*ad*
 *
 * Print out the board's IP address with descriptions.
 *
 * @param   buffer    points to the string which contains some descriptions.
 * @param   ip        ip address
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD SHOULD YOU USE DIFFERENT 
 *          DISPLAY FORMAT. The implementation below is for NET+OS development 
 *          boards only.
 *
 * @category Board_Params  
 * @since 6.0
 *
 *
*ad*/ 
static void printIpAddress(char * buffer, unsigned long ip)
{
#ifndef _LITTLE_ENDIAN
    printf(buffer, (ip >> 24) & 0xFF,
                   (ip >> 16) & 0xFF,
                   (ip >> 8)  & 0xFF,
                   (ip) & 0xFF);
#else
    printf(buffer, (ip) & 0xFF,
                   (ip >> 8)  & 0xFF,
                   (ip >> 16) & 0xFF,
                   (ip >> 24) & 0xFF);
#endif
}
#if BSP_WANT_ETHERNET || BSP_WANT_WIRELESS
/*ad*
 *
 * Print out the board's MAC address with descriptions.
 *
 * @param   buffer    points to the string which contains some descriptions.
 * @param   mac       points to the array which contains the MAC address
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD SHOULD YOU USE DIFFERENT 
 *          DISPLAY FORMAT. The implementation below is for NET+OS development 
 *          boards only.
 *
*ad*/ 
static void printMacAddress(char * buffer, BYTE * mac)
{
    printf(buffer, mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
}
#endif

/*ad*
 *
 * Gets one line of the information input from keyboard.
 * It prints out the information as well.
 *
 * @param   buf     points to an array to hold the information
 *
 * @category Board_Params  
 * @since 6.0
 *
*ad*/ 
static void readLine(char * buf)
{
  char character;
  int i = 0;
  do
  {
    character = getchar();
    if (character == '\b') 
    {
       if (i > 0)
       {
          putchar(character);
          putchar(0x20);   /*space*/
          putchar(character);
          --i;
       }
    }
    
    else if (character != '\n')
    {
       buf[i] = character;
       putchar(character);
       ++i;
    }
    fflush(stdout);
  }
  while (character != '\r');
  putchar('\n');
  
  buf[i-1] = '\0';
}  

/*ad*
 *
 * Gets new password input from keyboard.
 * Prints out "***" while getting information.
 *
 * @param   buf         Points to an array that holds the new password.
 * @param   buffersize  Size of the bufffer used to hold the password string.
 *
 * @category Board_Params  
 * @since 6.0
 *
*ad*/ 
static void readPassword(char * buf, unsigned int buffersize)
{
    char character;
    unsigned int i = 0;

    do
    {
        character = getchar();
        if (character == '\b') 
        {
            if (i > 0)
            {
                putchar(character);
                putchar(0x20);   /*space*/
                putchar(character);
                --i;
            }
        }
        else if ((character != '\r') && (character != '\n'))
        {
            buf[i] = character;
            putchar('*');

            /*  We need to reserve one character for the NUL at the end  */
            if (i < buffersize - 1)
            {
                ++i;
            }
        }
        fflush(stdout);
    }
    while ((character != '\r') && (character != '\n'));
    putchar('\r');
    putchar('\n');
  
    buf[i] = '\0';
}

/*ad*
 *
 * Displays items asking if you would like to update,
 * and waits for yes or no
 *
 * @param   itemDesc    Points to the string which contains description of the item.
 * @param   flag        Indicates how the item description is going to be displayed.
 *
 * @return  0           No update, keep the old setting.
 * @return  1           Yes, we need to change the setting.
 *
 * @category Board_Params  
 * @since 6.0
 *
*ad*/  
static int kbGetReply(char * itemDesc, int flag)
{
    int reply;
    char buffer[128];

    while (1)
    {
        fflush(stdout);
        if (flag & NADIALOG_CUSTOM_REPLY)
        {
            printf ("%s ", itemDesc);
            fflush(stdout);

        }
        else
        {
            printf ("Would you like to update the %s ", itemDesc);
            fflush(stdout);
        }
        if (flag & NADIALOG_CUSTOM_DEF_YES)
        {
            printf ("[Y]? ");
            fflush(stdout);
        }
        else
        {
            printf ("[N]? ");
            fflush(stdout);
        }

        readLine (buffer);
        if (buffer[0] == 'y' || buffer[0] =='Y')
        {
            reply = 1;
            break;
        }
        if (buffer[0] == 'n' || buffer[0] =='N')
        {
            reply = 0;
            break;
        }
        if (buffer[0] == '\0')
        {
            reply = ((flag & NADIALOG_CUSTOM_DEF_YES) != 0);
            break;
        }
    }
    return (reply);
}

#if BSP_WANT_WIRELESS

/* Description of each wireless protocol mode */
static const char *wln_modes[] = {
    "any available wireless network", /* was "Infrastructure", */
    "access point (infrastructure) networks only", /* was Ad hoc (join or create)", */
    "peer-to-peer (ad hoc) networks only)", /* was "Ad hoc (join only)" */
    "do not join a BSS", /* added as monitor mode */
};

/* Description of each wireless authorization mode */
static const char *wln_auth_modes[] = {
    "Any available authentication", 
    "Shared Key", 
    "WEP (802.1x)",
    "WPA Personal (WPA-PSK)",
    "WPA Enterprise (802.1x)",
    "Cisco LEAP",
    "Open System",
};

/* Description of each wireless transmit rate */
static const char *wln_tx_rate[] = { 
    "1", 
    "2",
    "5.5",
#ifdef BSP_ARM9			
    "6",
    "9",
#endif
    "11",
#ifdef BSP_ARM9
    "12",
    "18",
    "24",
    "36",
    "48",
    "54",
#endif
};

static int wln_txrate_list[] = 
{
    10,   /* 1 mbps */
    20,   /* 2 mbps */
    55,   /* 5.5 mbps */	
#ifdef BSP_ARM9			
    60,   /* 6 mbps */		
    90,   /* 9 mbps */	
#endif	
    110,  /* 11 mbps */
#ifdef BSP_ARM9		
    120,  /* 12 mbps */	
    180,  /* 18 mbps */
    240,  /* 24 mbps */	
    360,  /* 36 mbps */	
    480,  /* 48 mbps */	
    540   /* 54 mbps */
#endif
};

static int wln_txrate_num = (sizeof wln_txrate_list)/(sizeof wln_txrate_list[0]); 

/* Description of each wireless transmit power */
static const char *wln_tx_power[] = {
    "15dbm", 
    "14dbm", 
    "12dbm",
    "10dbm",
    "8dbm",
    "6dbm"
};

static int wln_txpwr_list[] = 
{
    15,
    14,
    12,
    10,
    8,
    6
};

static int wln_txpwr_num = (sizeof wln_txpwr_list)/(sizeof wln_txpwr_list[0]); 

#define MAP_INDEX_TO_TXRATE_BIT   0
#define MAP_TXRATE_BIT_TO_INDEX   1

#define MAP_INDEX_TO_TXPWR_VALUE   0
#define MAP_TXPWR_VALUE_TO_INDEX   1

static int getWlnTxRateMappedIdx(int idx, int isTxRate)
{
    int retVal=0, i = 0;

    if (isTxRate == MAP_INDEX_TO_TXRATE_BIT)
    {
        if (idx < wln_txrate_num)
            retVal = wln_txrate_list[idx];
    }
    else
    {
        /* Find display value. */
        while (i < wln_txrate_num)
        {
            if (wln_txrate_list[i] == idx)
                break; 
            i++;
        }

        retVal = (i < wln_txrate_num) ? i:0;
    }

    return retVal;              
}

static int kbGetTxRate(char *descr, int *value)
{
    char buffer[128];
    int n;

    while (1)
    {
        printf(descr, *value);
        printf("    ");
        fflush(stdout);
        for (n=0; n < wln_txrate_num; n++)
        {
            printf("%s", wln_tx_rate[n]);  
            fflush(stdout);
            if (n < wln_txrate_num-1)
                printf(", ");
            else
                printf(" Mbps: ");
            fflush(stdout);
        }

        readLine(buffer);  

        if (buffer[0] == '\0')
            return 0;
            
        for (n=0; n < wln_txrate_num; n++)
        {
            if (!strcmp(buffer, wln_tx_rate[n]))
            {
                *value = n;
                return 1;
            }
        }
    }  
}

static int getWlnTxPwrMappedIdx(int idx, int isTxPwr)
{
    int retVal = 0, i=0;

    if (isTxPwr == MAP_INDEX_TO_TXPWR_VALUE)
    {
        if (idx < wln_txpwr_num)
            retVal = wln_txpwr_list[idx];
        else
            retVal = WLN_TXPOWER_DEFAULT;         
    }
    else
    {
        /* Find display value. */
        while (i < wln_txpwr_num)
        {
            if (wln_txpwr_list[i] == idx)
                break; 
            i++;
        }

        retVal = (i < wln_txpwr_num) ? i:0;
    }

    return retVal;              
}

#if BSP_WANT_WPA
/* Description of each wireless encryption mode */
static const char *wln_encrypt_modes[] = {
    "Any available encryption", 
    "WEP", 
    "TKIP",
    "CCMP (AES)",
    "Open System (no encryption)",
};
#ifdef EAP_SUPPORT_TYPES
/* Description of each wireless outer EAP mode */
static const char *wln_outer_eap_modes[] = {
    "Any", 
    "PEAP", 
    "TLS",
    "TTLS",
};

/* Description of each wireless inner EAP mode */
static const char *wln_inner_eap_modes[] = {
    "Any", 
    "GTC", 
    "MD5",
    "MSCHAPV2",
    "OTP",
    "CHAP",         
    "MSCHAP",
    "TTLS_MSCHAPV2",
    "PAP",          
    "NONE", 
};
#endif
#endif

/*
 *
 *  Function: static void printWirelessParams (devBoardParamsType *nvParamsPtr)
 *
 *  Description:
 *
 *      This routine prints all wireless parameters.
 *
 *  Parameters:
 *
 *      nvParamsPtr     pointer to structure with development board parameters
 *
 */ 
static void printWirelessParams(devBoardParamsType * nvParamsPtr)
{
    int oldAuth = 0;
    int idx;
    int current_bss_type;
    
    NaIamParams_t *interface = customizeIamFindInterfaceConfig(BP_WLN_INTERFACE, &nvParamsPtr->iamParamsInfo);
    if (interface == NULL)
    {
        /* Find an unused config record, but don't overwrite ethernet */
        interface = customizeIamFindUnusedInterfaceConfig(&nvParamsPtr->iamParamsInfo, FALSE);
        if (interface)
            customizeIamGetDefaultInterfaceConfig(BP_WLN_INTERFACE, interface);
    }

    if (interface)
    {
        if (interface->staticParams.isEnabled)
        {
            printIpAddress("  Wireless IP address is %d.%d.%d.%d\r\n", interface->staticParams.IPV4_ADDR(ipAddress));
            printIpAddress("  Wireless subnet mask is %d.%d.%d.%d\r\n", interface->staticParams.subnetMask);
            printIpAddress("  Wireless IP address of default gateway to other networks is %d.%d.%d.%d\r\n", interface->staticParams.IPV4_ADDR(gateway));
        }
        else
        {
            printf("  The board will obtain wireless IP configuration parameters from the network.\r\n");
        }
    }

    if (wln_is_country_valid((wln_country_code_t)nvParamsPtr->wlnCountryCode))    
        printf("  Wireless country name configured is %s\r\n", wln_get_country_string_from_code((wln_country_code_t)nvParamsPtr->wlnCountryCode));
    else
        printf("  Wireless country name is not configured\r\n");

    fflush(stdout);

    switch(nvParamsPtr->wlnParams.bss_type) {
        case 0:
            current_bss_type = 1;
            break;
        case 1:
            current_bss_type = 2;
            break;
        case 3:
            current_bss_type = 0;
            break;
        case 4:
            current_bss_type = 3;
            break;
        default:
            current_bss_type = 0;
            break;
    }
    if (nvParamsPtr->wlnParams.bss_type <= WLN_BSS_NONE) 
        printf("  Wireless protocol mode is %s\r\n", wln_modes[current_bss_type]);
        
    if (nvParamsPtr->wlnParams.ssid_len == 0)
        printf("  Wireless SSID is (search)\r\n");
    else
        printf("  Wireless SSID is %.*s\r\n", nvParamsPtr->wlnParams.ssid_len, nvParamsPtr->wlnParams.ssid);
    
    if(nvParamsPtr->wlnParams.channel == WLN_CHAN_SCAN)
        printf("  Wireless channel is (search)\r\n");
    else
        printf("  Wireless channel is %d\r\n", nvParamsPtr->wlnParams.channel);

    fflush(stdout);
    idx = getWlnTxRateMappedIdx(nvParamsPtr->wlnParams.tx_rate, MAP_TXRATE_BIT_TO_INDEX);       
    printf("  Wireless transmit rate is %s\r\n", wln_tx_rate[idx]);
    idx = getWlnTxPwrMappedIdx(nvParamsPtr->wlnParams.tx_power, MAP_TXPWR_VALUE_TO_INDEX);
    printf("  Wireless transmit power is %s\r\n", wln_tx_power[idx]);
        
    /* multi domain enabled? */
    if (nvParamsPtr->wlnParams.options & WLN_OPT_MULTI_DOMAIN)
        printf("  Wireless multi domain capability (802.11d) enabled\r\n");
    else
        printf("  Wireless multi domain capability (802.11d) disabled\r\n");

    /* Find old auth for display */
    if (nvParamsPtr->wlnParams.authen == WLN_AUTH_OPEN) {
        oldAuth = 6;
    } else if (nvParamsPtr->wlnParams.authen & WLN_AUTH_SHAREDKEY) {
        oldAuth = 1;                
    } 
#if BSP_WANT_WPA
    else if (nvParamsPtr->wlnParams.authen == WLN_AUTH_ANY) {
        oldAuth = 0;
    } 
    else if (nvParamsPtr->wlnParams.authen & WLN_AUTH_WEP_8021X) {
        oldAuth = 2;                
    } else if (nvParamsPtr->wlnParams.authen & WLN_AUTH_WPA_PSK) {
        oldAuth = 3;                
    } else if (nvParamsPtr->wlnParams.authen & WLN_AUTH_WPA_8021X) {
        oldAuth = 4;                
    } else if (nvParamsPtr->wlnParams.authen & WLN_AUTH_LEAP) {
        oldAuth = 5;                
    }               
                                        
    /* Authentication type */
    printf("  Authentication type = %s\r\n", wln_auth_modes[oldAuth]);    
#else
    /* Authentication type */
    printf("  Authentication type = %s\r\n", wln_auth_modes[oldAuth]);    
    
    /* encryption type */
    if (nvParamsPtr->wlnParams.encrypt & WLN_ENCR_WEP)
		printf("  Encryption type = WEP\r\n");
	else
	    printf("  Encryption type = Open System (no encryption)\r\n");
#endif    
}

/*
 *
 *  Function: static int kbGetNumber (char *descr, int *value, int min, int max)
 *
 *  Description:
 *
 *      This routine gets a number from the keyboard.
 *
 *  Parameters:
 *
 *      descr       description string displayed to user
 *      value       default value and returned result
 *      min         minimum allowed value
 *      max         maximum allowed value
 *
 *  Return Values:
 *
 *      0             no update, value unchanged
 *      1             updated     
 *
 */ 
static int kbGetNumber(char * descr, int * value, int min, int max)
{
    char buffer[128];
    int n, result;

    while (1)
    {
        fflush(stdout);
        printf(descr, *value);
        fflush(stdout);
        readLine(buffer);  
        if (buffer[0] == '\0')
            return 0;
            
        n = sscanf(buffer, "%d", &result);
        if (n == 1 && result >= min && result <= max)
        {
            *value = result;
            return 1;
        }
        
        printf("Please enter a number between %d and %d\r\n", min, max);
    }  
}

/*
 *
 *  Function: static int kbGetChannel (char *descr, int *value, int search)
 *
 *  Description:
 *
 *      This routine gets a number from the keyboard.
 *
 *  Parameters:
 *
 *      descr       description string displayed to user
 *      value       default value and returned result
 *      search      if set user will be allowed to enter 0
 *
 *  Return Values:
 *
 *      0             no update, value unchanged
 *      1             updated     
 *
 */ 
static int kbGetChannel (char *descr, int *value, int search)
{
    char buffer[128];
    int n, result, max, min;
    unsigned short mask;

    mask = wln_get_channel_mask(wln_country_code); 
    min = (int)wln_get_min_channel(mask);
    max = (int)wln_get_max_channel(mask);

    while (1)
    {
        fflush(stdout);
        printf(descr, min, max, *value);
        fflush(stdout);
        readLine(buffer);  
        if (buffer[0] == '\0')
        {
            n = 1;
            result = *value;
        }
        else
            n = sscanf(buffer, "%d", &result);

        if (n == 1 && result >= min && result <= max)
        {
            *value = result;
            return 1;
        }
        else if (n == 1 && search && result == 0) /* allow channel 0 */
        {
            *value = result;
            return 1;
        }

        if (search)
        {
            printf("Please enter 0 for search or a number between %d and %d\r\n", min, max);
        }
        else
        {        
            printf("Please enter a number between %d and %d\r\n", min, max);
        }
    }  
}

/*
 *
 *  Function: static int kbGetStringLimit (char *descr, char *value, int min, int max)
 *
 *  Description:
 *
 *      This routine gets a number from the keyboard.
 *
 *  Parameters:
 *
 *      descr       description string displayed to user
 *      value       default value and returned result
 *      min         minimum allowed characters
 *      max         maximum allowed characters
 *
 *  Return Values:
 *
 *      0             no update, value unchanged
 *      1             updated     
 *
 */ 
static int kbGetStringLimit(char * descr, char * value, int min, int max)
{
    char buffer[128];
    int n;

    while (1)
    {
        printf(descr, value);
        fflush(stdout);
        readLine(buffer);  
        n = strlen(buffer);
        if (n == 0)
            return 0;
        if (n >= min && n <= max)
        {
            strcpy(value, buffer);
            return 1;
        }
        
        printf("Please enter a string of %d to %d characters\r\n", min, max);
    }  
}

/*
 *
 *  Function: static int kbGetCountryCode (char *descr, char *value)
 *
 *  Description:
 *
 *      This routine gets a country code from the user.
 *
 *  Parameters:
 *
 *      descr       description string displayed to user
 *      value       default value and returned result
 *
 *  Return Values:
 *
 *      0             no update, value unchanged
 *      1             updated     
 *
 */ 
static int kbGetCountryCode(char * descr, unsigned long * value)
{
    char buffer[128];
    int n;
    wln_country_code_t code;

    while (1)
    {
        printf(descr, wln_get_country_string_from_code((wln_country_code_t)*value));
        fflush(stdout);
        readLine(buffer);  
        n = strlen(buffer);
        if (n == 0)
            return 0;

        code = wln_get_country_code_from_string(buffer);
        if (code != COUNTRY_NOTFOUND)
        {
            if (wln_is_country_valid(code))
            {
                *value = (unsigned long)code;
                return 1;
            }
        }

        printf("Got the approval from following countries... \r\n");
        wln_print_supported_countries();
    }  
}

/*
 *
 *  Function: static int kbGetWEPKey (int idx, wln_params *params)
 *
 *  Description:
 *
 *      This routine gets a new WEP key from the keyboard.
 *
 *  Parameters:
 *
 *      idx       index of key (0-3)
 *      params      points to wireless parameters
 *
 *  Return Values:
 *
 *      0             no update, keep the old setting
 *      1             updated     
 *
 */ 
static int kbGetWEPKey(int idx, wln_params * params)
{
    char buffer[128];
    unsigned char key[WLN_WEP104_SIZE];
    int i, n;
    
    while (1)
    {
        printf("WEP key %d [no change]? ", idx);
        fflush(stdout);
        readLine(buffer);  
        if (buffer[0] == '\0')
            return 0;
            
        for (i = 0; i < WLN_WEP104_SIZE; i++)
        {
            if (isxdigit(buffer[2*i]) && isxdigit(buffer[2*i+1]) &&
                sscanf(&buffer[2*i], "%2x", &n) == 1)
                    key[i] = n;
            else
                break;
        }
        
        if (i == WLN_WEP40_SIZE || i == WLN_WEP104_SIZE)
        {
            params->wep_key_len = i;
            memcpy(params->wep_key[idx], key, i);
            return 1;
        }

        printf("Please enter a 10 or 26 digit hex number\r\n");
    }
}

/*
 *
 *  Function: static int NAWirelessDialog (wln_params *wp)
 *
 *  Description:
 *
 *      This routine gets the new wireless parameters.
 *
 *  Parameters:
 *
 *      wp     pointer to structure with wireless parameters
 *
 *  Return Values:
 *
 *      0:     no updates
 *      1:     some or all network information are updated
 *
 */ 
static int NAWirelessDialog(wln_params * wp) 
{
    char buffer[256];
    int i, flag, result;
#if BSP_WANT_WPA    
    char buffer2[128];
    int encrypt = 0;
    int authen = 0; 
    int wpaFlag = 0;
    uint32 current_encrypt;
#endif  
    uint32 old_auth = wp->authen;
    uint32 old_encrypt = wp->encrypt;
    int oldAuth = 0;
    int oldEncrypt = 0;
    int txRate = 0;
    int txPower = 0;
    
#ifdef EAP_SUPPORT_TYPES    
    uint32 old_eap_methods = wp->eap_methods;
    uint32 current_eap_methods;
#endif
    uint32 current_bss_type; 
    wp->authen = WLN_AUTH_OPEN;
    wp->encrypt = WLN_ENCR_OPEN;
    wp->eap_methods = NAGetAppWlnEapMethods();
    
    /* BSS type */
    switch(wp->bss_type) {
        case 0:
            current_bss_type = 1;
            break;
        case 1:
            current_bss_type = 2;
            break;
        case 3:
            current_bss_type = 0;
            break;
        case 4:
            current_bss_type = 3;
            break;
        default:
            current_bss_type = 0;
            break;
    }
    sprintf(buffer, "\r\nNetwork type [%d]?\r\n"
        "    0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n",
        current_bss_type, wln_modes[0], wln_modes[1], wln_modes[2], wln_modes[3]);
    result = -1;    
    kbGetNumber(buffer, (int *) &result, WLN_BSS_ESS, WLN_BSS_NONE - 1);
    if (result == -1)
        wp->bss_type = (wln_bss)current_bss_type;
    else
    {
        if (result == 3)
            wp->bss_type = (wln_bss)4;
        else
            wp->bss_type = (wln_bss)result;
        }
    
    switch(wp->bss_type) {
        case 0:
            wp->bss_type = (wln_bss)3;
            break;
        case 1:
            wp->bss_type = (wln_bss)0;
            break;
        case 2:
            wp->bss_type = (wln_bss)1;
            break;
        case 4:
            wp->bss_type = (wln_bss)WLN_BSS_NONE;
            break;
        default:
            wp->bss_type = (wln_bss)3;
            break;
    }
        
        
        
    /* SSID */
    memcpy(buffer, wp->ssid, wp->ssid_len);
    buffer[wp->ssid_len] = '\0';
    if (kbGetStringLimit("Network name (SSID) (enter ANY to search) [%s]? ",
        buffer, 0, WLN_SSID_SIZE)) {
        int n = strlen(buffer);
        if (strcmp(buffer, "ANY") == 0)
            n = 0;
        
        memcpy (wp->ssid, buffer, n);
        wp->ssid_len = n;
    }
    
    /* 802.11d */                                                        
    flag = NADIALOG_CUSTOM_REPLY;
    if ((wp->options & WLN_OPT_MULTI_DOMAIN)) {
        flag |= NADIALOG_CUSTOM_DEF_YES;
    }
    printf("\r\nTo use the multi domain capability (802.11d), your access point MUST\r\n");
    if (kbGetReply ("have 802.11d support.  Enable multi domain capability (802.11d)", flag))
        wp->options |= WLN_OPT_MULTI_DOMAIN;
    else
        wp->options &= ~WLN_OPT_MULTI_DOMAIN;

    /* Channel */
    if (wp->bss_type == APP_WLN_BSS_NONE)
    {
        do {
        kbGetChannel("\r\nChannel (enter %d-%d) [%d]? ", 
            &wp->channel, 0);
        } while (wp->channel == 0);
    }
    else
        kbGetChannel("\r\nChannel (enter %d-%d or 0 to search) [%d]? ", 
            &wp->channel, 1);
        
    /* Find display value. */
    txRate = getWlnTxRateMappedIdx(wp->tx_rate, MAP_TXRATE_BIT_TO_INDEX);
    /* Max transmit rate */
    sprintf(buffer, "\r\nMaximum transmit rate [%s]?\r\n", wln_tx_rate[txRate]);
    kbGetTxRate(buffer, &txRate);
    wp->tx_rate = getWlnTxRateMappedIdx(txRate, MAP_INDEX_TO_TXRATE_BIT);
    /* Find display value. */
    txPower = getWlnTxPwrMappedIdx(wp->tx_power, MAP_TXPWR_VALUE_TO_INDEX);
    /* Max transmit power */
    sprintf(buffer, "\r\nMaximum transmit power [%s]?\r\n"
        "    0 for 15dbm\r\n    1 for 14dbm\r\n    2 for 12dbm\r\n    3 for 10dbm\r\n    4 for 8dbm\r\n    5 for 6dbm\r\n", wln_tx_power[txPower]);
    kbGetNumber(buffer, &txPower, 0, 5);
    wp->tx_power = getWlnTxPwrMappedIdx(txPower, MAP_INDEX_TO_TXPWR_VALUE);
    
    printf("WIRELESS AUTHENTICATION SETTINGS:\r\n");    
    
#if !BSP_WANT_WPA
    if (wp->bss_type == WLN_BSS_IBSS || wp->bss_type == WLN_BSS_IBSS_JOIN) 
    {
        /*there is no authentication in ad hoc mode*/
        printf("\r\nAuthentication type is %s\r\n", wln_auth_modes[6]);
        wp->authen = WLN_AUTH_OPEN; 
    
        printf("\r\nWIRELESS ENCRYPTION SETTINGS:\r\n");
        /* Security type */
        oldEncrypt = 0;
    	if (old_encrypt & WLN_ENCR_WEP)
    		oldEncrypt = 1; 
    		
    	sprintf(buffer, "\r\nWireless security type [%d]?\r\n"
                "    0 for %s\r\n    1 for %s\r\n",
                oldEncrypt, "no encryption", "WEP");
          
        i = -1;      
    	kbGetNumber(buffer, &i, 0, 1);  
        if (i == -1)
            i = oldEncrypt;
            
    	if (i)
    	    wp->encrypt = WLN_ENCR_WEP;
    	else
    	    wp->encrypt = WLN_ENCR_OPEN;
	}
	else
	{
        /* Find old auth for display */
         if (old_auth & WLN_AUTH_SHAREDKEY) {
            oldAuth = 1;                
         } else {
            oldAuth = 0;                
         }            
                                                
        /* Authentication type */
        sprintf(buffer, "\r\nAuthentication type [%d]?\r\n"
                "    0 for %s\r\n    1 for %s\r\n",
                oldAuth, wln_auth_modes[6], wln_auth_modes[1]);
            
        i = -1;    
        kbGetNumber(buffer, &i, 0, 1);  
        if (i == -1)
            i = oldAuth;
               
        if (i == 1) 
            wp->authen = WLN_AUTH_SHAREDKEY;
        else
            wp->authen = WLN_AUTH_OPEN;
                 
                
        printf("\r\nWIRELESS ENCRYPTION SETTINGS:\r\n");
        /* Security type */
        oldEncrypt = 0;
    	if (old_encrypt & WLN_ENCR_WEP)
    		oldEncrypt = 1; 
    		
    	sprintf(buffer, "\r\nWireless security type [%d]?\r\n"
                "    0 for %s\r\n    1 for %s\r\n",
                oldEncrypt, "no encryption", "WEP");
          
        i = -1;      
    	kbGetNumber(buffer, &i, 0, 1);  
        if (i == -1)
            i = oldEncrypt;
            
    	if (i)
    	    wp->encrypt = WLN_ENCR_WEP;
    	else
    	    wp->encrypt = WLN_ENCR_OPEN;
    	
	}
#endif         
#if BSP_WANT_WPA
    if (wp->bss_type == WLN_BSS_IBSS || wp->bss_type == WLN_BSS_IBSS_JOIN) 
    {
        /*there is no authentication in ad hoc mode*/
        printf("\r\nAuthentication type is %s\r\n", wln_auth_modes[6]);
        wp->authen = WLN_AUTH_OPEN; 
    
        printf("\r\nWIRELESS ENCRYPTION SETTINGS:\r\n");
        /* Security type */
        oldEncrypt = 0;
    	if (old_encrypt & WLN_ENCR_WEP)
    		oldEncrypt = 1; 
    		
    	sprintf(buffer, "\r\nWireless security type [%d]?\r\n"
                "    0 for %s\r\n    1 for %s\r\n",
                oldEncrypt, "no encryption", "WEP");
          
        i = -1;      
    	kbGetNumber(buffer, &i, 0, 1);  
        if (i == -1)
            i = oldEncrypt;
            
    	if (i)
    	    wp->encrypt = WLN_ENCR_WEP;
    	else
    	    wp->encrypt = WLN_ENCR_OPEN;
    } 
    else {
        #ifdef READ_WPA_FLAG                    
        /* Enable WPA */
        flag = NADIALOG_CUSTOM_REPLY;
        if (old_auth & (WLN_AUTH_WPA_PSK|WLN_AUTH_WPA_8021X)) {
            flag |= NADIALOG_CUSTOM_DEF_YES;
        }
            
        wpaFlag = kbGetReply ("Enable WPA", flag);
        #else
        wpaFlag = 1;
        #endif
        if (wpaFlag) {          
            
            /* Find old auth for display */
            if (old_auth == WLN_AUTH_ANY) {
                oldAuth = 0;
            } else if (old_auth == WLN_AUTH_OPEN) {
                oldAuth = 6;
            } else if (old_auth & WLN_AUTH_SHAREDKEY) {
                oldAuth = 1;                
            } else if (old_auth & WLN_AUTH_WEP_8021X) {
                oldAuth = 2;                
            } else if (old_auth & WLN_AUTH_WPA_PSK) {
                oldAuth = 3;                
            } else if (old_auth & WLN_AUTH_WPA_8021X) {
                oldAuth = 4;                
            } else if (old_auth & WLN_AUTH_LEAP) {
                oldAuth = 5;                
            }               
                                                
            /* Authentication type */
            sprintf(buffer, "\r\nAuthentication type [%d]?\r\n"
                "    0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n    4 for %s\r\n    5 for %s\r\n    6 for %s\r\n",
                oldAuth, wln_auth_modes[0], wln_auth_modes[1], 
                wln_auth_modes[2], wln_auth_modes[3], wln_auth_modes[4], wln_auth_modes[5], wln_auth_modes[6]);
            
            authen = -1;    
            kbGetNumber(buffer, &authen, 0, 6);
            if (authen == -1)
                authen = oldAuth;    
            
            switch (authen) {
                case 0: 
                    wp->authen = WLN_AUTH_ANY;
                        
                    if (wp->ssid_len == 0) {
                        printf("Must specify an SSID to use WPA-PSK\r\n");
                    } else if (kbGetStringLimit("WPA Personal (WPA-PSK) passphrase [no change]?\r\n", buffer, 8, 63)) {
                        printf("Calculating key (about 30 seconds)...\r\n\r\n");
                        wpa_passphrase_to_psk (buffer, (char*)wp->ssid, wp->ssid_len, wp->wpa_psk);
                    }
        
                    printf("For WEP (802.1x), WPA (802.1x) and Cisco LEAP authentication:\r\n");
                    /* Identity */
                    kbGetStringLimit("Username [no change]? ", wp->identity, 0, WLN_IDENT_SIZE);

                    /* Password */
                    while (1) {
                        printf("Password [no change]? ");
                        fflush(stdout);
                        readPassword (buffer, sizeof buffer);
                        if (buffer[0] == '\0') {
                            break;
                        }

                        printf("Re-enter password: ");
                        fflush(stdout);
                        readPassword (buffer2, sizeof buffer2);
                        if (strcmp(buffer, buffer2) == 0) {
                                strncpy(wp->password, buffer, WLN_PWD_SIZE);
                                break;
                        } else
                            printf("Passwords do not match\r\n");
                    }
                                      
                    /* Encryption type */
                    if (old_encrypt == WLN_ENCR_ANY)
                    {
                        current_encrypt = 0;
                    }
                    else if (old_encrypt == WLN_ENCR_WEP)
                    {
                        current_encrypt = 1;
                    }
                    else if (old_encrypt == WLN_ENCR_TKIP)
                    {
                        current_encrypt = 2;
                    }
                    else if (old_encrypt == WLN_ENCR_CCMP)
                    {
                        current_encrypt = 3;
                    }
                    else if (old_encrypt == WLN_ENCR_OPEN)
                    {
                        current_encrypt = 4;
                    }
                    else
                    {
                        current_encrypt = 0;
                    }
                
                    printf("\r\nWIRELESS ENCRYPTION SETTINGS:\r\n");
            
                    sprintf(buffer, "\r\nEncryption type [%d]?\r\n"
                        "    0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n    4 for %s\r\n",
                        current_encrypt, wln_encrypt_modes[0], wln_encrypt_modes[1], 
                        wln_encrypt_modes[2], wln_encrypt_modes[3], wln_encrypt_modes[4]);
                    encrypt = -1;    
                    kbGetNumber(buffer, &encrypt, 0, 4);
                    if (encrypt == -1)
                        encrypt = current_encrypt;
                    switch (encrypt) {
                        case 0: 
                            wp->encrypt = WLN_ENCR_ANY;
                            break;
                        case 1: 
                            wp->encrypt = WLN_ENCR_WEP;
                            break;
                        case 2: 
                            wp->encrypt = WLN_ENCR_TKIP;
                            break;
                        case 3: 
                            wp->encrypt = WLN_ENCR_CCMP;
                            break;
                        case 4: 
                            wp->encrypt = WLN_ENCR_OPEN;
                            break;              
                    }
                    
                    break;
                case 1: 
                    wp->authen = WLN_AUTH_SHAREDKEY;
                                      
                    /* Encryption type */
                    if (old_encrypt == WLN_ENCR_WEP)
                    {
                        current_encrypt = 0;
                    }
                    else if (old_encrypt == WLN_ENCR_OPEN)
                    {
                        current_encrypt = 1;
                    }
                    else
                    {
                        current_encrypt = 1;
                    }
                
                    printf("WIRELESS ENCRYPTION SETTINGS:\r\n");
            
                    sprintf(buffer, "\r\nEncryption type [%d]?\r\n"
                        "    0 for %s\r\n    1 for %s\r\n",
                        current_encrypt, wln_encrypt_modes[1], wln_encrypt_modes[4]);
                    encrypt = -1;    
                    kbGetNumber(buffer, &encrypt, 0, 1);
                    if (encrypt == -1)
                        encrypt = current_encrypt;
                    switch (encrypt) {
                        case 0: 
                            wp->encrypt = WLN_ENCR_WEP;
                            break;
                        case 1: 
                            wp->encrypt = WLN_ENCR_OPEN;
                            break;              
                    }                 
                    break;                                      
                case 2: 
                    wp->authen = WLN_AUTH_WEP_8021X;
                    wp->encrypt = WLN_ENCR_WEP;
        
                    printf("\r\nFor WEP (802.1x), WPA (802.1x) and Cisco LEAP authentication:\r\n");
                    /* Identity */
                    kbGetStringLimit("Username [no change]? ", wp->identity, 0, WLN_IDENT_SIZE);

                    /* Password */
                    while (1) {
                        printf("Password [no change]? ");
                        fflush(stdout);
                        readPassword (buffer, sizeof buffer);
                        if (buffer[0] == '\0') {
                            break;
                        }

                        printf("Re-enter password: ");
                        fflush(stdout);
                        readPassword (buffer2, sizeof buffer2);
                        if (strcmp(buffer, buffer2) == 0) {
                                strncpy(wp->password, buffer, WLN_PWD_SIZE);
                                break;
                        } else
                            printf("Passwords do not match\r\n");
                    }
                
                    break;
                case 3: 
                    wp->authen = WLN_AUTH_WPA_PSK;
                        
                    if (wp->ssid_len == 0) {
                        printf("Must specify an SSID to use WPA-PSK\r\n");
                    } else if (kbGetStringLimit("WPA Personal (WPA-PSK) passphrase [no change]?\r\n", buffer, 8, 63)) {
                        printf("Calculating key (about 30 seconds)...\r\n\r\n");
                        wpa_passphrase_to_psk (buffer, (char*)wp->ssid, wp->ssid_len, wp->wpa_psk);
                    }
                                      
                    /* Encryption type */
                    if (old_encrypt == WLN_ENCR_ANY)
                    {
                        current_encrypt = 0;
                    }
                    else if (old_encrypt == WLN_ENCR_WEP)
                    {
                        current_encrypt = 1;
                    }
                    else if (old_encrypt == WLN_ENCR_TKIP)
                    {
                        current_encrypt = 2;
                    }
                    else if (old_encrypt == WLN_ENCR_CCMP)
                    {
                        current_encrypt = 3;
                    }
                    else if (old_encrypt == WLN_ENCR_OPEN)
                    {
                        current_encrypt = 4;
                    }
                    else
                    {
                        current_encrypt = 0;
                    }
                
                    printf("WIRELESS ENCRYPTION SETTINGS:\r\n");
            
                    sprintf(buffer, "\r\nEncryption type [%d]?\r\n"
                        "    0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n    4 for %s\r\n",
                        current_encrypt, wln_encrypt_modes[0], wln_encrypt_modes[1], 
                        wln_encrypt_modes[2], wln_encrypt_modes[3], wln_encrypt_modes[4]);
                    encrypt = -1;    
                    kbGetNumber(buffer, &encrypt, 0, 4);
                    if (encrypt == -1)
                        encrypt = current_encrypt;
                    switch (encrypt) {
                        case 0: 
                            wp->encrypt = WLN_ENCR_ANY;
                            break;
                        case 1: 
                            wp->encrypt = WLN_ENCR_WEP;
                            break;
                        case 2: 
                            wp->encrypt = WLN_ENCR_TKIP;
                            break;
                        case 3: 
                            wp->encrypt = WLN_ENCR_CCMP;
                            break;
                        case 4: 
                            wp->encrypt = WLN_ENCR_OPEN;
                            break;              
                    }
                                       
                    break;
                case 4: 
                    wp->authen = WLN_AUTH_WPA_8021X;
        
                    printf("\r\nFor WEP (802.1x), WPA (802.1x) and Cisco LEAP authentication:\r\n");
                    /* Identity */
                    kbGetStringLimit("Username [no change]? ", wp->identity, 0, WLN_IDENT_SIZE);

                    /* Password */
                    while (1) {
                        printf("Password [no change]? ");
                        fflush(stdout);
                        readPassword (buffer, sizeof buffer);
                        if (buffer[0] == '\0') {
                            break;
                        }

                        printf("Re-enter password: ");
                        fflush(stdout);
                        readPassword (buffer2, sizeof buffer2);
                        if (strcmp(buffer, buffer2) == 0) {
                                strncpy(wp->password, buffer, WLN_PWD_SIZE);
                                break;
                        } else
                            printf("Passwords do not match\r\n");
                    }
                                      
                    /* Encryption type */
                    if (old_encrypt == WLN_ENCR_ANY)
                    {
                        current_encrypt = 0;
                    }
                    else if (old_encrypt == WLN_ENCR_WEP)
                    {
                        current_encrypt = 1;
                    }
                    else if (old_encrypt == WLN_ENCR_TKIP)
                    {
                        current_encrypt = 2;
                    }
                    else if (old_encrypt == WLN_ENCR_CCMP)
                    {
                        current_encrypt = 3;
                    }
                    else if (old_encrypt == WLN_ENCR_OPEN)
                    {
                        current_encrypt = 4;
                    }
                    else
                    {
                        current_encrypt = 0;
                    }
                
                    printf("\r\nWIRELESS ENCRYPTION SETTINGS:\r\n");
            
                    sprintf(buffer, "\r\nEncryption type [%d]?\r\n"
                        "    0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n    4 for %s\r\n",
                        current_encrypt, wln_encrypt_modes[0], wln_encrypt_modes[1], 
                        wln_encrypt_modes[2], wln_encrypt_modes[3], wln_encrypt_modes[4]);
                    encrypt = -1;    
                    kbGetNumber(buffer, &encrypt, 0, 4);
                    if (encrypt == -1)
                        encrypt = current_encrypt;
                    switch (encrypt) {
                        case 0: 
                            wp->encrypt = WLN_ENCR_ANY;
                            break;
                        case 1: 
                            wp->encrypt = WLN_ENCR_WEP;
                            break;
                        case 2: 
                            wp->encrypt = WLN_ENCR_TKIP;
                            break;
                        case 3: 
                            wp->encrypt = WLN_ENCR_CCMP;
                            break;
                        case 4: 
                            wp->encrypt = WLN_ENCR_OPEN;
                            break;              
                    }
                                                           
                    break;              
                case 5: 
                    wp->authen = WLN_AUTH_LEAP;
                    wp->encrypt = WLN_ENCR_WEP;
                    wp->eap_methods = WLN_EAP_LEAP;
        
                    printf("\r\nFor WEP (802.1x), WPA (802.1x) and Cisco LEAP authentication:\r\n");
                    /* Identity */
                    kbGetStringLimit("Username [no change]? ", wp->identity, 0, WLN_IDENT_SIZE);

                    /* Password */
                    while (1) {
                        printf("Password [no change]? ");
                        fflush(stdout);
                        readPassword (buffer, sizeof buffer);
                        if (buffer[0] == '\0') {
                            break;
                        }

                        printf("Re-enter password: ");
                        fflush(stdout);
                        readPassword (buffer2, sizeof buffer);
                        if (strcmp(buffer, buffer2) == 0) {
                                strncpy(wp->password, buffer, WLN_PWD_SIZE);
                                break;
                        } else
                            printf("Passwords do not match\r\n");
                    }
                    break;              
                case 6: 
                    wp->authen = WLN_AUTH_OPEN;
                                      
                    /* Encryption type */
                    if (old_encrypt == WLN_ENCR_ANY)
                    {
                        current_encrypt = 0;
                    }
                    else if (old_encrypt == WLN_ENCR_WEP)
                    {
                        current_encrypt = 1;
                    }
                    else if (old_encrypt == WLN_ENCR_TKIP)
                    {
                        current_encrypt = 2;
                    }
                    else if (old_encrypt == WLN_ENCR_CCMP)
                    {
                        current_encrypt = 3;
                    }
                    else if (old_encrypt == WLN_ENCR_OPEN)
                    {
                        current_encrypt = 4;
                    }
                    else
                    {
                        current_encrypt = 0;
                    }
                
                    printf("WIRELESS ENCRYPTION SETTINGS:\r\n");
            
                    sprintf(buffer, "\r\nEncryption type [%d]?\r\n"
                        "    0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n    4 for %s\r\n",
                        current_encrypt, wln_encrypt_modes[0], wln_encrypt_modes[1], 
                        wln_encrypt_modes[2], wln_encrypt_modes[3], wln_encrypt_modes[4]);
                    encrypt = -1;    
                    kbGetNumber(buffer, &encrypt, 0, 4);
                    if (encrypt == -1)
                        encrypt = current_encrypt;
                    switch (encrypt) {
                        case 0: 
                            wp->encrypt = WLN_ENCR_ANY;
                            break;
                        case 1: 
                            wp->encrypt = WLN_ENCR_WEP;
                            break;
                        case 2: 
                            wp->encrypt = WLN_ENCR_TKIP;
                            break;
                        case 3: 
                            wp->encrypt = WLN_ENCR_CCMP;
                            break;
                        case 4: 
                            wp->encrypt = WLN_ENCR_OPEN;
                            break;              
                    }                                       

                    break;              
            }

#ifdef EAP_SUPPORT_TYPES
            /* Select EAP type if needed */
            if (wp->authen != WLN_AUTH_LEAP) {
                                            
                /* Outer EAP type */
                if (old_eap_methods == WLN_EAP_ANY)
                {
                    current_eap_methods = 0;
                }
                else if ((old_eap_methods & 0x0f) == WLN_EAP_PEAP)
                {
                    current_eap_methods = 1;
                }
                else if ((old_eap_methods & 0x0f) == WLN_EAP_TLS)
                {
                    current_eap_methods = 2;
                }
                else if ((old_eap_methods & 0x0f) == WLN_EAP_TTLS)
                {
                    current_eap_methods = 3;
                }
                else
                {
                    current_eap_methods = 0;
                }
                sprintf(buffer, "\r\nOuter EAP type [%d]?\r\n"
                    "Enter 0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n",
                    current_eap_methods, wln_outer_eap_modes[0], wln_outer_eap_modes[1], wln_outer_eap_modes[2], 
                    wln_outer_eap_modes[3]);
                
                encrypt = -1;    
                kbGetNumber(buffer, &encrypt, 0, 3);
                if (encrypt == -1)
                    encrypt = current_eap_methods;
                switch (encrypt) {
                    case 0: 
                        wp->eap_methods = WLN_EAP_ANY;
                        break;
                    case 1: 
                        wp->eap_methods = WLN_EAP_PEAP;
                        break;
                    case 2: 
                        wp->eap_methods = WLN_EAP_TLS;
                        break;
                    case 3: 
                        wp->eap_methods = WLN_EAP_TTLS;
                        break;                      
                }
                
                if (wp->eap_methods == WLN_EAP_TTLS) {
                    /* Inner EAP type */
                    if ((old_eap_methods & WLN_EAP_INNER_ANY) == WLN_EAP_INNER_ANY)
                    {
                        current_eap_methods = 0;
                    }
                    else if ((old_eap_methods & WLN_EAP_GTC) == WLN_EAP_GTC)
                    {
                        current_eap_methods = 1;
                    }
                    else if ((old_eap_methods & WLN_EAP_MD5) == WLN_EAP_MD5)
                    {
                        current_eap_methods = 2;
                    }
                    else if ((old_eap_methods & WLN_EAP_MSCHAPV2) == WLN_EAP_MSCHAPV2)
                    {
                        current_eap_methods = 3;
                    }
                    else if ((old_eap_methods & WLN_EAP_OTP) == WLN_EAP_OTP)
                    {
                        current_eap_methods = 4;
                    }
                    else if ((old_eap_methods & WLN_EAP_TTLS_CHAP) == WLN_EAP_TTLS_CHAP)
                    {
                        current_eap_methods = 5;
                    }
                    else if ((old_eap_methods & WLN_EAP_TTLS_MSCHAP) == WLN_EAP_TTLS_MSCHAP)
                    {
                        current_eap_methods = 6;
                    }
                    else if ((old_eap_methods & WLN_EAP_TTLS_MSCHAPV2) == WLN_EAP_TTLS_MSCHAPV2)
                    {
                        current_eap_methods = 7;
                    }
                    else if ((old_eap_methods & WLN_EAP_TTLS_PAP) == WLN_EAP_TTLS_PAP)
                    {
                        current_eap_methods = 8;
                    }
                    else
                    {
                        current_eap_methods = 0;
                    }
                    sprintf(buffer, "\r\nInner EAP type [%d]?\r\n"
                        "Enter 0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n    4 for %s\r\n    5 for %s\r\n    6 for %s\r\n    7 for %s\r\n    8 for %s\r\n",
                        current_eap_methods, wln_inner_eap_modes[0], wln_inner_eap_modes[1],wln_inner_eap_modes[2],wln_inner_eap_modes[3],
                        wln_inner_eap_modes[4],wln_inner_eap_modes[5], wln_inner_eap_modes[6], wln_inner_eap_modes[7], 
                        wln_inner_eap_modes[8]);
                    
                    encrypt = -1;    
                    kbGetNumber(buffer, &encrypt, 0, 8);
                    if (encrypt == -1)
                        encrypt = current_eap_methods;
                    switch (encrypt) {
                        case 0: 
                            wp->eap_methods |= WLN_EAP_INNER_ANY;
                            break;
                        case 1: 
                            wp->eap_methods |= WLN_EAP_GTC;
                            break;
                        case 2: 
                            wp->eap_methods |= WLN_EAP_MD5;
                            break;
                        case 3: 
                            wp->eap_methods |= WLN_EAP_MSCHAPV2;
                            break;
                        case 4: 
                            wp->eap_methods |= WLN_EAP_OTP;
                            break;                                                  
                        case 5: 
                            wp->eap_methods |= WLN_EAP_TTLS_CHAP;
                            break;
                        case 6: 
                            wp->eap_methods |= WLN_EAP_TTLS_MSCHAP;
                            break;
                        case 7: 
                            wp->eap_methods |= WLN_EAP_TTLS_MSCHAPV2;
                            break;
                        case 8: 
                            wp->eap_methods |= WLN_EAP_TTLS_PAP;
                            break;                      
                    }                   
                } else {                                
                    /* Inner EAP type */
                    if ((old_eap_methods & (WLN_EAP_GTC | WLN_EAP_MD5 | WLN_EAP_MSCHAPV2 | WLN_EAP_OTP)) == 
                        (WLN_EAP_GTC | WLN_EAP_MD5 | WLN_EAP_MSCHAPV2 | WLN_EAP_OTP))
                    {
                        current_eap_methods = 0;
                    }
                    else if ((old_eap_methods & WLN_EAP_GTC) == WLN_EAP_GTC)
                    {
                        current_eap_methods = 1;
                    }
                    else if ((old_eap_methods & WLN_EAP_MD5) == WLN_EAP_MD5)
                    {
                        current_eap_methods = 2;
                    }
                    else if ((old_eap_methods & WLN_EAP_MSCHAPV2) == WLN_EAP_MSCHAPV2)
                    {
                        current_eap_methods = 3;
                    }
                    else if ((old_eap_methods & WLN_EAP_OTP) == WLN_EAP_OTP)
                    {
                        current_eap_methods = 4;
                    }
                    else
                    {
                        current_eap_methods = 0;
                    }
                    sprintf(buffer, "\r\nInner EAP type [%d]?\r\n"
                        "Enter 0 for %s\r\n    1 for %s\r\n    2 for %s\r\n    3 for %s\r\n    4 for %s\r\n",                        
                        current_eap_methods, wln_inner_eap_modes[0], wln_inner_eap_modes[1], wln_inner_eap_modes[2], wln_inner_eap_modes[3], 
                        wln_inner_eap_modes[4]);
                    
                    encrypt = -1;    
                    kbGetNumber(buffer, &encrypt, 0, 4);
                    if (encrypt == -1)
                        encrypt = current_eap_methods;
                    switch (encrypt) {
                        case 0: 
                            wp->eap_methods |= WLN_EAP_GTC | WLN_EAP_MD5 | WLN_EAP_MSCHAPV2 | WLN_EAP_OTP;
                            break;
                        case 1: 
                            wp->eap_methods |= WLN_EAP_GTC;
                            break;
                        case 2: 
                            wp->eap_methods |= WLN_EAP_MD5;
                            break;
                        case 3: 
                            wp->eap_methods |= WLN_EAP_MSCHAPV2;
                            break;
                        case 4: 
                            wp->eap_methods |= WLN_EAP_OTP;
                            break;                      
                    }
                }                                   
            }
#endif /* #ifdef EAP_SUPPORT_TYPES */            

        }
    }
#endif // BSP_WANT_WPA
        
        if (wp->authen == WLN_AUTH_ANY || wp->authen == WLN_AUTH_SHAREDKEY ||
           ((wp->authen == WLN_AUTH_OPEN) && (wp->encrypt & WLN_ENCR_WEP))) {
            /* Default key */
            kbGetNumber("\r\nWEP default key number (0-3) [%d]? ", 
                &wp->wep_key_id, 0, 3);                 

            /* WEP keys */
            for (i = 0; i < 4; i++) {
                kbGetWEPKey(i, wp);
            }        
#if 0            
            if (wp->authen == WLN_AUTH_OPEN) {  
                wp->authen |= WLN_AUTH_SHAREDKEY|WLN_AUTH_WEP_8021X;
            }
            
            wp->encrypt |= WLN_ENCR_WEP;            
#endif
        }

    return DIALOGUPDATED;
}
#endif // BSP_WANT_WIRELESS

/*ad*
 *
 * Displays the target configuration and prompts the user
 * for modifications.
 *
 * @param   nvParamsPtr     pointer to structure with development board parameters
 *
 * @return  0               success
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD SHOULD YOU USE DIFFERENT 
 *          DISPLAY FORMAT. The implementation below is for NET+OS development 
 *          boards only.
 *
*ad*/ 
static int NADisplayDialog(devBoardParamsType * nvParamsPtr)
{   
    char s[20];  
#if BSP_WANT_ETHERNET || BSP_WANT_WIRELESS 
    BYTE mac_addrp[6];
#endif
    NaIamParams_t *interface = NULL;

    NA_UNUSED_PARAM(interface);   /* To remove GNU Compiler warning. */

    /*
     * We use to have code here that corrected invalid baudrates and waittimes before 
     * they were displayed.  I removed it since we should display invalid settings 
     * so that users will correct them.
     */
         
    printf("\r\n");
    printf("\r\n");
    printf("\r\n");
    printf("NET+OS Version %s\r\n", NETOS_VERSION_NUMBER);
    printf("Copyright (c) 1997-2007, Digi International, Inc.\r\n");
    printf("\r\n");
    printf("PLATFORM: %s\r\n", NABspPlatform);
    printf("APPLICATION: %s\r\n", NAGetAppDialogAppName ());
    printf("-----------------------------------------------------------------------------\r\n");
    printf("NETWORK INTERFACE PARAMETERS:\r\n");
    
#if BSP_WANT_ETHERNET
    interface = customizeIamFindInterfaceConfig(BP_ETH_INTERFACE, &nvParamsPtr->iamParamsInfo);

    if (interface == NULL)
    {
        interface = customizeIamFindUnusedInterfaceConfig(&nvParamsPtr->iamParamsInfo, TRUE);
        customizeIamGetDefaultEthernetConfig(interface, 0);
    } 
    if (interface->staticParams.isEnabled)
    {
        printIpAddress("  IP address on LAN is %d.%d.%d.%d\r\n", interface->staticParams.IPV4_ADDR(ipAddress));
        printIpAddress("  LAN interface's subnet mask is %d.%d.%d.%d\r\n", interface->staticParams.subnetMask);
        printIpAddress("  IP address of default gateway to other networks is %d.%d.%d.%d\r\n", interface->staticParams.IPV4_ADDR(gateway));
    }
    else
    {
        printf("  The board will obtain IP configuration parameters from the network.\r\n");
    }
#endif

#if BSP_WANT_WIRELESS
    printWirelessParams(nvParamsPtr);
#endif

/*  New PPP parameters allows backward compatibility with older applications    */

    if (NAGetAppEnablePPP1 ())
    {
        printIpAddress("  IP address on PPP interface 1 is %d.%d.%d.%d\r\n", nvParamsPtr->ipAddrPpp1);

        if (nvParamsPtr->ipAddrPpp1peer)
        {
            printIpAddress("  Peer IP address connected to PPP interface 1 is %d.%d.%d.%d\r\n", nvParamsPtr->ipAddrPpp1peer);
        }
    }

    if (NAGetAppEnablePPP2 ())
    {
        printIpAddress("  IP address on PPP interface 2 is %d.%d.%d.%d\r\n", nvParamsPtr->ipAddrPpp2);

        if (nvParamsPtr->ipAddrPpp2peer)
        {
            printIpAddress("  Peer IP address connected to PPP interface 2 is %d.%d.%d.%d\r\n", nvParamsPtr->ipAddrPpp2peer);
        }
    }

    if (NAGetAppUseNetworkTimeProtocol ())
    {
        if (nvParamsPtr->ipAddrSntp1 != 0)
        {
            printIpAddress("  Primary SNTP Server IP address is %d.%d.%d.%d\r\n", nvParamsPtr->ipAddrSntp1);
            if (nvParamsPtr->ipAddrSntp2 != 0)
            {
                printIpAddress("  Secondary SNTP Server IP address is %d.%d.%d.%d\r\n", nvParamsPtr->ipAddrSntp2);
            }
        }
    }


    printf("HARDWARE PARAMETERS:\r\n");   
    printf("  Serial channels will use a baud rate of %lu\r\n",nvParamsPtr->baudrate);

    memcpy(s,  nvParamsPtr->serialNumber,sizeof(nvParamsPtr->serialNumber));
    s[BP_SERIAL_NUMBER_LENGTH]=0;  /*for correct printf %s format*/
    printf("  This board's serial number is %s\r\n", s);

#if BSP_WANT_ETHERNET
    customizeGetInterfaceMACAddress(BP_ETH_INTERFACE, mac_addrp);
    printMacAddress("  This board's Ethernet MAC Address is %2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X\r\n",mac_addrp);
#endif
#if BSP_WANT_WIRELESS
    customizeGetInterfaceMACAddress(BP_WLN_INTERFACE, mac_addrp);
    printMacAddress("  This board's Wireless MAC Address is %2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X\r\n",mac_addrp);
    fflush(stdout);
#endif                  
    printf("  After board is reset, start-up code will wait %lu seconds\r\n", nvParamsPtr->waitTime);

#if BSP_WANT_ETHERNET
    printf("  Default duplex setting for Ethernet connection: %s\r\n", getDuplexString(nvParamsPtr->defaultDuplexLevel));
#endif

    printf("-----------------------------------------------------------------------------\r\n");

    return (0);
}

/*ad*
 *
 * Closes APP_DIALOG_PORT if APP_DIALOG_PORT is defined, and
 * restore stdin, stdout, and stderr.
 *
 * @return  0   success
 *
 * @category Board_Params  
 * @since 6.0
 *
*ad*/
static int NACloseDialog(void)
{
    #ifdef NETOS_GNU_TOOLS
        close (fd);

        RESTORE_IO
    #elif defined(__CC_ARM)
    	close(stdin->handle);

		/* Restore IO */
        stdin->handle = inch;
        stdout->handle = outch;
        stderr->handle = errch;
    #else
        close(stdin->io_channel);

        /* Restore IO */
        stdin->io_channel = inch;
        stdout->io_channel = outch;
        stderr->io_channel = errch;
    #endif

    return (0);
}

/*ad*
 *
 * Checks for keyboard entry before timeout
 *
 * @param   timeout     Time in seconds before this routine returns
 *                      if no key is pressed.
 *
 * @return  >0          The value of the pressed key.
 * @return  0xff        Timed out.
 *
 * @category Board_Params  
 * @since 6.0
 *
*ad*/ 
static char checkKeyboardInput(unsigned long timeout)
{
    unsigned int i;
    char rc=0xff;
    
      
    for (i=0; i < timeout; i++)
      {
        tx_thread_sleep(BSP_TICKS_PER_SECOND);
        rc=0;
        rc= (char) getchar();
        if ((rc != 0xff) && (rc != '\n'))
           return (rc);
      }
    return (rc);  
}


/*ad*
 *
 * Checks if the user intends to make changes, if yes, it will close
 * the dialog port and reopen it with block mode to wait for input. It also
 * authenticates thw user for the right to change. If the password is incorrect,
 * it allows the user to reenter 3 times before giving up.
 *
 * @param   nvParamsPtr Pointer to structure with development board parameters
 *
 * @return  0   Continue without any updates.
 * @return  1   Need to update.
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *          below is for NET+OS development boards only.
 *
 * @category Board_Params  
 * @since 6.0
 *
*ad*/ 
static int NAConfigureDialog(devBoardParamsType * nvParamsPtr)
{
    int rc;
    char temp;
    char buffer[128];
    unsigned int  notAuthenticated = 1;
    char password[NASYSACC_STRLEN_PASSWORD];

    if (NAGetAppUseNvram () != APP_DO_NOT_USE_NVRAM)
    {
        printf("Press any key in %lu seconds to change these settings.\r\n", nvParamsPtr->waitTime);
    }
    printf("\r\n");
   
    /*any key is pressed?*/
    temp = checkKeyboardInput(nvParamsPtr->waitTime);
    if ((temp != 0xff) && (NAGetAppUseNvram () != APP_DO_NOT_USE_NVRAM))
    {
        printf("Press A to Accept the settings, or M to Modify?");
        fflush(stdout);
    }
    else
    {
        return 0;          /*time out spared, no modification needed, continue...*/
    }
    
    /*let's close nonblock i/o, use block i/o*/
    NACloseDialog ();
        
    /* Open for Blocking Mode */
    NAOpenDialog (1, nvParamsPtr); 

     
    /*"A" or "a", "M" or "m"* is pressed?*/
    while (1)
    {
        readLine(buffer);
        fflush(stdout);
        if (buffer[0] == 'a' || buffer[0] =='A')
        {
            printf("continue...\r\n");
            return 0;         /*no modification needed, user needs to continue...*/
        }

        if (buffer[0] == 'm' || buffer[0] =='M')
        {
            break;
        }

        printf("Press A to Accept the settings, or M to Modify?"); 
        fflush(stdout);
    }  

    /*  "M" or "m" is pressed   */
    printf("\r\n");
    rc = NAgetSysAccess ("root", &password[0], NULL);
    if ((rc == (int) NASYSACC_LEVEL_ROOT) && (notAuthenticated))
    {
        for (rc = 0; ; rc ++)
        {
            printf("Enter the root password: ");
            fflush(stdout);
            readPassword(buffer, sizeof buffer);
            fflush(stdout);
            if (strcmp(buffer, password) == 0)
            {
                /*  OK - this is an authentic administrator */
                notAuthenticated = 0;
                break;
            }

            if (rc < 3)
            {
                printf("FAILURE: root password incorrect\r\n");
            }
            else
            {
                /*  no modification allowed, continue...*/
                printf("FAILURE: root password incorrect\r\ncontinue...\r\n");
                return 0;
            }
        }
    } 
    
    if (kbGetReply ("\r\nReset configuration to default values", NADIALOG_CUSTOM_REPLY))
    {
        customizeUseDefaultParams(nvParamsPtr);
    }

    return (1);
}

/*ad*
 *
 * Validates an IP address.
 *
 * @param   buffer  Points to the string which contains an IP address in "a.b.c.d" format.
 *
 * @return  0   invalid
 * @return  1   valid
 *
 * @category Board_Params  
 * @since 6.0
 *
*ad*/ 
static int  checkIpAddress(char * buffer)
{
   unsigned int length,i;
   unsigned int j =0;
   unsigned int val;
      
   
   length = strlen(buffer);
   
   if ( (length > MAX_IPADDRESS_LENGTH))
      return 0;
   else
   {
     for (i=0; i<length; i++)
     {
       
       if (buffer[i] =='.')
           {
             j++;
             if (j > 3)   /* only three '.' allowed */
                return 0;
           }
       else if (!(isdigit(buffer[i]))) /* all input char must be numeric */
           return 0;
     } 
     
     for (i = 0; i < 4; i++)
      {
           if (*buffer != '\0') 
           {
               val = 0;
               while (*buffer != '\0' && *buffer != '.') 
               {  
                    val *= 10;
                    val += *buffer - '0';
                    buffer++;
               }
               if (val > 255)
                  return 0;       /* any field should be less than 256 */
                  
               if (*buffer != '\0') 
                   buffer++;
           } 
       }   
   }
   return 1; 
} 
 
/*ad*
 *
 * Prints out what kind of IP address it needs and gets it from keyboard.
 *
 * @param   ipAddrDesc  Points to string which contains some description.
 * @param   ipAddress   Points to an unsigned long which holds the old IP address.
 *                      Upon return, it will hold the new IP address if any.
 *
 * @return  0   No update, keep the old IP address.
 * @return  1   Got a new IP address.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note        THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *              below is for NET+OS development boards only.
 *
*ad*/
static int kbGetIpAddress(char * ipAddrDesc, unsigned long * ipAddress)
{
    int rc;

    char buffer[128] = {0};

    while (1)
    {
        printf ("%s ", ipAddrDesc);
        fflush(stdout);
        printIpAddress(" [%d.%d.%d.%d]? ", *ipAddress);
        fflush(stdout);
        readLine (buffer);
        if (buffer[0] == '\0')
        {
            rc = 0;
            break;
        }

        if(checkIpAddress(buffer))
        {
            *ipAddress = NAInet_addr(buffer);
            printf ("  %s ", ipAddrDesc);
            printIpAddress("IP address is %d.%d.%d.%d\r\n", *ipAddress);
            rc = 1;
            break;
        }

        printf("Invalid IP address.\r\n");
    }

    return (rc);
}

#if BSP_WANT_ETHERNET || BSP_WANT_WIRELESS 
/*
 *
 *  Function: static int NAIPParamDialog (devBoardParamsType *nvParamsPtr, char *ifname, char *prompt)
 *
 *  Description:
 *
 *      This routine gets a new IP parameters.
 *
 *  Parameters:
 *
 *      nvParamsPtr     pointer to structure with development board parameters
 *      ifname        name of interface (BP_ETH_INTERFACE of BP_WLN_INTERFACE)
 *      prompt        string displayed to user
 *
 *  Return Values:
 *
 *      0             no update, keep the old setting
 *      1             updated     
 *
 */ 
static int NAIPParamDialog(devBoardParamsType * nvParamsPtr, char * ifname, char * prompt)
{
    int callStatus = 0;
    unsigned long temp;

    /*
     * Extract the configuration structure from the NVRAM structure.  There should be a configuration
     * settings for the interface.  If not, then reinitialize the structure.
     */
    NaIamParams_t *config = customizeIamFindInterfaceConfig(ifname, &nvParamsPtr->iamParamsInfo);

    if (config == NULL)
    {
        customizeIamGetDefaultConfig(&nvParamsPtr->iamParamsInfo);
        config = customizeIamFindInterfaceConfig(ifname, &nvParamsPtr->iamParamsInfo);
    }

    if (!config->isEnabled)
    {
        customizeIamGetDefaultInterfaceConfig(ifname, config);
    }

    if (config->dhcpParams.isEnabled)
    {
        temp = kbGetReply (prompt, NADIALOG_CUSTOM_REPLY | NADIALOG_CUSTOM_DEF_YES);
    }
    else
    {
        temp = kbGetReply (prompt, NADIALOG_CUSTOM_REPLY);
    }
    
    if (temp)
    {
        /*
         * Disable the static configuration to make ACE get an address from the network.
         * The protocols actually used by ACE are determined by the settings stored
         * in NVRAM.
         */
        config->staticParams.isEnabled = FALSE;
        config->dhcpParams.isEnabled = TRUE;
    }
    else
    {
        /* ip address */
        kbGetIpAddress ("IP address", &config->staticParams.IPV4_ADDR(ipAddress));
        
        /* subnet mask */
        kbGetIpAddress ("Subnet mask", &config->staticParams.subnetMask);
        
        /* gateway mask */
        kbGetIpAddress ("Gateway address", &config->staticParams.IPV4_ADDR(gateway));
        
        /* validate ip information. If invalid, return an error */
        callStatus = naValidateNetworkInformation(config->staticParams.IPV4_ADDR(ipAddress), 
                                                config->staticParams.subnetMask,
                                                config->staticParams.IPV4_ADDR(gateway));

        /* if error - return error code, then print message and loop */
        if(callStatus)
        {
            return callStatus;
        }

        /*
         * Enabling the static configuration will cause ACE to ignore the settings used
         * for DHCP, BOOTP, et al on the same interface.
         */
        config->staticParams.isEnabled = TRUE;
        config->dhcpParams.isEnabled = FALSE;
    }

    return DIALOGUPDATED;
}
#endif

/*ad*
 *
 * Gets new network information.
 *
 * @param       nvParamsPtr         Pointer to structure with development board parameters
 *
 * @return     NOSUBNETTING         IP Address can not be subnetted.
 * @return     ZEROIPNOTALLOWED     IP Address 0.0.0.0 is not allowed.
 * @return     IPANDGWMATCH         IP Address and Gateway Address are the same.
 * @return     IPORGWISMULTICAST    IP Address or Gatweay Address is > 223.
 * @return     IPORGWISLOCALLOOP    IP Address or Gatweay Address is 127.
 * @return     ZEROOCTETNOTALLOWED  Zero is not allowed in leftmost octet of IP Address.
 * @return     ZEROMASKNOTALLOWED   Subnet mask 0.0.0.0 not allowed.
 * @return     MAXSUBNETEXCEEDED    Subnet mask can not exceed 255.255.255.252
 * @return     IPISNETWORKID        IP Address is network ID.
 * @return     IPISBROADCAST        IP Address is network broadcast address.
 * @return     GWISNETWORKID        Gateway Address is network ID.
 * @return     GWISBROADCAST        Gateway Address is network broadcast address.
 * @return     GWNOTONSUBNET        Gateway Address is not on subnet.
 * @return     DIALOGUPDAYED        Changes have been made. UpdateDialog will need to be called.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *       below is for NET+OS development boards only.
 *
*ad*/ 
static int NANetworkDialog(devBoardParamsType * nvParamsPtr)
{
#if BSP_WANT_ETHERNET || BSP_WANT_WIRELESS
    int callStatus = 0, updated;
#endif

#if BSP_WANT_ETHERNET     
    bpDuplexLevelType newDuplex = bpDefaultDuplex;
#endif

    /*  Operator has been authenticated */
    printf("\r\n");
    printf("For each of the following questions, you can press <Return> to ");
    printf("select the value shown in braces, or you can enter a new value.\r\n"); 
    printf("\r\n");
    
#if BSP_WANT_ETHERNET
    printf("ETHERNET INTERFACE SETTINGS:\r\n\r\n");
    
    callStatus = NAIPParamDialog(nvParamsPtr, BP_ETH_INTERFACE, 
                    "Obtain IP settings automatically using DHCP for this interface");

    if (callStatus != DIALOGUPDATED)
    {
        return callStatus;
    }
    
    printf ("\r\n");
    
    printf("This interface must have a unique MAC address.\r\n");
    updated = promptForMacAddress("Ethernet MAC Address ", nvParamsPtr->ethernetMac);
     do
     {
        newDuplex = kbGetDuplex ("Ethernet duplex setting (default, full, half)? " , nvParamsPtr->defaultDuplexLevel);
        if (isDuplexValid(newDuplex) == FALSE)
        {
            printf("You must select Full duplex, Half duplex, or the PHY's Default setting.\n\n");
        }
     } while (isDuplexValid(newDuplex) == FALSE);

     updated |= (newDuplex != nvParamsPtr->defaultDuplexLevel);
     nvParamsPtr->defaultDuplexLevel = newDuplex;
#endif

#if BSP_WANT_WIRELESS
    printf ("\r\n");
    printf("WIRELESS INTERFACE SETTINGS:\r\n\r\n");
    
    callStatus = NAIPParamDialog(nvParamsPtr, BP_WLN_INTERFACE, 
                    "Obtain IP settings automatically using DHCP for this interface");
   
    if (callStatus != DIALOGUPDATED)
    {
        return callStatus;
    }
    
    printf ("\r\n");
    printf("This interface must have a unique MAC address.\r\n");
#if BSP_WANT_ETHERNET
    updated |= promptForMacAddress("Wireless MAC Address ", nvParamsPtr->wlnParams.sta_addr);
#else
    updated = promptForMacAddress("Wireless MAC Address ", nvParamsPtr->wlnParams.sta_addr);
    if (updated)
    {
        #ifdef DEBUG_WIRELESS_MACADDRESS
        printf("NANetworkDialog: new wireless mac address);
        #endif
    }
#endif
    /* country code */    
    fflush(stdout);
    updated |= kbGetCountryCode("Country name [%s]? ", &nvParamsPtr->wlnCountryCode);
    wln_country_code = (wln_country_code_t)nvParamsPtr->wlnCountryCode;
    nvParamsPtr->wlnParams.chan_mask = wln_get_channel_mask(wln_country_code);
    fflush(stdout);
    NAWirelessDialog(&nvParamsPtr->wlnParams);
#endif

    if (NAGetAppEnablePPP1 ())
    {
        /* ppp com1 address */
        kbGetIpAddress ("PPP interface 1", &nvParamsPtr->ipAddrPpp1);

        /* ppp com1 peer address */
        kbGetIpAddress ("PPP interface 1 Peer", &nvParamsPtr->ipAddrPpp1peer);
    }

    if (NAGetAppEnablePPP2 ())
    {
        /* ppp com2 address */
        kbGetIpAddress ("PPP interface 2", &nvParamsPtr->ipAddrPpp2);

        /* ppp com2 peer address */
        kbGetIpAddress ("PPP interface 2 Peer", &nvParamsPtr->ipAddrPpp2peer);
    }

    if (NAGetAppUseNetworkTimeProtocol ())
    {
        /*  Primary SNTP Server */
        kbGetIpAddress ("Primary SNTP Server", &nvParamsPtr->ipAddrSntp1);

        /*  Secondary SNTP Server */
        if (nvParamsPtr->ipAddrSntp1 != 0)
        {
            kbGetIpAddress ("Secondary SNTP Server", &nvParamsPtr->ipAddrSntp2);
        }
    }

    return DIALOGUPDATED;
}

/*ad*
 *
 * Displays those items you would like to update,
 * and gets the update information from keyboard.
 *
 * @param   itemDesc    Points to the string which contains descriptions of the item.
 * @param   destBuffer  points to an array to hold the new information.
 * @param   length      length boundary of new information
 * @param   flag        to indicate if we need the new information to be entered again.
 *
 * @return  0           no update, keep the old setting
 * @return  1           updated     
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD SHOULD YOU USE DIFFERENT 
 *          DISPLAY FORMAT. The implementation below is for NET+OS development 
 *          boards only.
 *
*ad*/ 
static int kbGetData(char * itemDesc, char * destBuffer, unsigned int length, int flag)
{
    int rc;

    char srcBuffer_1[128];
    char srcBuffer_2[128];

    while (1)
    {
        printf("Enter the new %s:    ", itemDesc);
        fflush(stdout);
        readPassword(srcBuffer_1, sizeof srcBuffer_1);
        if (srcBuffer_1[0] == '\0')
        {
            rc = 0;
            break;
        }

        if (strlen(srcBuffer_1) < length)
        {
            if (flag & NADIALOG_VERIFY)
            {
                /* verify input */
                printf("Re-enter the new %s: ", itemDesc);
                fflush(stdout);
                readPassword(srcBuffer_2, sizeof srcBuffer_2);
            }

            if (!(flag & NADIALOG_VERIFY)  || strcmp (srcBuffer_1, srcBuffer_2) == 0)
            {
                /*  update destination buffer  */
                memset (destBuffer, 0, length);
                strcpy (destBuffer, srcBuffer_1);
                rc = 1;
                break;
            }
            
            printf("ERROR: %s entries mismatched.\r\n", itemDesc);
        }
        else
        {
            printf("ERROR: Invalid %s.  Length must be less than %d characters.\r\n", itemDesc, length);
        }
    }

    return (rc);
}

/*ad*
 *
 * Displays those items you would like to update,
 * and gets the update information from keyboard.
 *
 * @param     itemDesc    points to the string which contains descriptions of the item.
 * @param     destBuffer  points to an array to hold the new information.
 * @param     length      length boundary of new information
 * @param     flag        to indicate if we need the answer to be yes or no,
 *                        or indicate if we need the new information to be entered again.
 *
 * @return    0           no update, keep the old setting
 * @return    1           updated     
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD SHOULD YOU USE DIFFERENT 
 *          DISPLAY FORMAT. The implementation below is for NET+OS development 
 *          boards only.
 *
*ad*/ 
static int kbGetString(char * itemDesc, char * destBuffer, int length, int flag)
{
    int rc;

    while (1)
    {
        fflush(stdout);
        if (kbGetReply (itemDesc, flag))
        {
            if (kbGetData (itemDesc, destBuffer, length, flag))
            {
                rc = 1;
                break;
            }
        }
        else
        {
            /* No update */
            rc = 0;
            break;
        }
    }
    
    return (rc);
}

/*ad*
 *
 * Gets new security information.
 *
 * @param   nvParamsPtr     pointer to structure with development board parameters
 *
 * @return  0       no updates
 * @return  1       Some or all security information was updated
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *          below is for NET+OS development boards only.
 *
*ad*/ 
static int NASecurityDialog(devBoardParamsType * nvParamsPtr)
{
    char s[20];
    char buffer[128];
    int updated = 0;
    char *valueStr;
    char bufDesc[32];
    char password[NASYSACC_STRLEN_PASSWORD];

    if (NAGetAppRootPassword () || NAGetAppAdministratorsAccount () 
        || NAGetAppFixedGatewayKey ())
    {
        printf("\r\nSECURITY SETTINGS:\r\n\r\n");
    }

    /*  root password   */
    if (NAGetAppRootPassword ())
    {
        if (kbGetString ("Root Password", password, NASYSACC_STRLEN_PASSWORD, NADIALOG_VERIFY)) 
        {
            if (nvParamsPtr->rootPassword[0] == 0)
            {
                NAsetSysAccess (NASYSACC_DEL, "root", NAGetAppRootPassword (), NASYSACC_LEVEL_ROOT, 0);
            }
            else
            {
                NAsetSysAccess (NASYSACC_DEL, "root", nvParamsPtr->rootPassword, NASYSACC_LEVEL_ROOT, 0);
            }

            NAsetSysAccess (NASYSACC_ADD, "root", password, NASYSACC_LEVEL_ROOT, 0);

            /*  update the nvParams record  */
            memset (nvParamsPtr->rootPassword, 0, sizeof(nvParamsPtr->rootPassword));
            strcpy (nvParamsPtr->rootPassword, password);
            updated = 1;
        }
    }

    /*  administrator account password  */
    if ((valueStr = NAGetAppAdministratorsAccount ()) != NULL)
    {
        sprintf (bufDesc, "Administrator '%s' Password", valueStr);
        updated |= kbGetString (bufDesc, nvParamsPtr->adminPassword, NASYSACC_STRLEN_PASSWORD, NADIALOG_VERIFY);
    }

    /*  fixed Key */
    if (NAGetAppFixedGatewayKey ())
    {
        updated |= kbGetString ("Fixed Key", nvParamsPtr->FixedKey, NASYSACC_STRLEN_PASSWORD, NADIALOG_VERIFY);
    }    
     
    printf("\r\n");
    printf("MISCELLANEOUS SETTINGS:\r\n");
    printf("\r\n");

    /*set the board's serial number?*/
    printf("Each development board must have a unique serial number\r\n");
    memcpy(s,  nvParamsPtr->serialNumber,sizeof(nvParamsPtr->serialNumber));
    s[BP_SERIAL_NUMBER_LENGTH]=0;  /*for correct printf %s format*/
    while (1)
    {
        printf("Set the board's serial number[%s]? ", s);
        fflush(stdout);
        readLine(buffer);  
        if (buffer[0] == '\0')
        {
            break;
        }

        if ( checkSerialNumber(buffer))
        {
                memcpy(nvParamsPtr->serialNumber, buffer ,sizeof(nvParamsPtr->serialNumber));
                memcpy(s,  nvParamsPtr->serialNumber,sizeof(nvParamsPtr->serialNumber));
                s[BP_SERIAL_NUMBER_LENGTH]=0;  /*for correct printf %s format*/
                printf("  The board's new serial number is %s\r\n", s);
                updated = 1;
                break;
        }

        printf("The board's serial number has to be A00000000-Z99999999.\r\n");
        printf("1 alpha character followed by 8 digits.\r\n");
    }
   
    printf("\r\n");
    
    /*set wait time?*/
    printf("How long (in seconds) should CPU delay before starting up [%lu]? ", nvParamsPtr->waitTime);
    fflush(stdout);
    readLine(buffer);  
    if (buffer[0] != '\0')
    {
        if(buffer[1] == '\0')
        {
            if (buffer[0] < 0x31)
            {
                nvParamsPtr->waitTime = 1;
                printf("The smallest wait time is 1 second.\r\n");
            }
            else
            {
                nvParamsPtr->waitTime = buffer[0]-0x30;        
            }
        }
        else
        {
            nvParamsPtr->waitTime = (unsigned long)((buffer[0]-0x30)*10) + (unsigned long)(buffer[1]-0x30);
        }
         
        printf("  After board is reset, start-up code will wait %lu seconds.\r\n", nvParamsPtr->waitTime);   
        updated = 1;
     }

    return (updated);
}

/*ad*
 *
 * Validates the board's serial number.
 *
 * @param      buffer    points to the string which contains serial number.
 *
 * @return     0         invalid
 * @return     1         valid
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *          below is for NET+OS development boards only.
 *
*ad*/ 
static int checkSerialNumber(char * buffer)
{
    int i;
    int result = 0;
  
    if (((BP_SERIAL_NUMBER_LENGTH + 1) == strlen(buffer)) && (toupper(buffer[0]) == 'S'))
    {
        /*
         * Digi Manufacturing process prefixes serial numbers with a leading
         * S.  Ingore it.
         */
        memmove(buffer, &buffer[1], BP_SERIAL_NUMBER_LENGTH + 1);
    }

    if (BP_SERIAL_NUMBER_LENGTH == strlen(buffer))
    {
        /*
         * Digi serial number must consist of a letter followed by 8 digits.
         */
        if (!isalpha(buffer[0]))
        {
            goto done;
        }
        for (i = 1; i < BP_SERIAL_NUMBER_LENGTH; i++)
        {
            if (!isdigit(buffer[i]))
            {
                goto done;
            }
        }
        result = 1;
    }
    done:
    
    return result;    
} 


#if BSP_WANT_ETHERNET
/*ad*
 *
 * Prompts for a duplex level.
 *
 * @param     desc        points to the string which contains description of the item.
 * @param     setting     indicates current duplex setting
 *
 *  Return Values:
 *
 * @return     bpHalfDuplex      half duplex
 * @return     bpFullDuplex      full duplex  
 * @return     bpDefaultDuplex   automatically negotiated duplex level
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD SHOULD YOU USE DIFFERENT 
 *          DISPLAY FORMAT. The implementation below is for NET+OS development 
 *          boards only.
 *
*ad*/ 
static bpDuplexLevelType kbGetDuplex(char * desc, bpDuplexLevelType setting)
{
    bpDuplexLevelType   reply;
    char buffer[128];



    while (1)
    {
        printf ("%s[%s] ", desc, getDuplexString(setting));
        fflush(stdout);

        readLine (buffer);
        if (buffer[0] == 'f' || buffer[0] =='F')
        {
            reply = bpFullDuplex;
            break;
        }

        if (buffer[0] == 'h' || buffer[0] =='H') 
        {
            reply = bpHalfDuplex;
            break;
        }
        
        if (buffer[0] == 'd' || buffer[0] =='D') 
        {
            reply = bpDefaultDuplex;
            break;
        }
        
       if (buffer[0] == 0)
        {
            reply = setting;
            break;
        }
    }

    return (reply);
}
#endif /* BSP_WANT_ETHERNET */

#define ETHERNET_MAC_ADDRESS_LENGTH         (6)


#if BSP_WANT_ETHERNET || BSP_WANT_WIRELESS

static int promptForMacAddress (char *description, unsigned char *macAddress)
{
    char addressDescription[20];
    char buffer[128];
    int updated = 0;
    
    sprintf(addressDescription, "%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X", macAddress[0], macAddress[1], macAddress[2], macAddress[3], macAddress[4], macAddress[5]);
    while (1)
    {
        BYTE tempMacAddress[ETHERNET_MAC_ADDRESS_LENGTH];
        
        printf("%s[%s]? ", description, addressDescription);
        fflush(stdout);
        readLine(buffer);  
        if (buffer[0] == '\0')
        {
            goto done;
        }

        if ( NAConvertMacAddressString(tempMacAddress, buffer) == BP_SUCCESS)
        {
            memcpy(macAddress, tempMacAddress, ETHERNET_MAC_ADDRESS_LENGTH);
            printf("  This board's new %s is %2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X\r\n", description,macAddress[0], macAddress[1], macAddress[2], macAddress[3], macAddress[4], macAddress[5]);                  
            updated = 1;
            goto done;
        }

        printf("The %s must consist of 6 2-hexdigit \r\n", description);
        printf("values seperated by colons.\r\n");
    }
done:
    return updated;
}
#endif

/*ad*
 *
 * Gets new hardware related information.
 *
 * @param     nvParamsPtr     pointer to structure with development board parameters
 *
 * @return    0     no updates
 * @return    1     some or all hardware related information was updated
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *          below is for NET+OS development boards only.
 *
*ad*/    
static int NAHardwareDialog(devBoardParamsType * nvParamsPtr)
{
    char buffer[128];
    int updated = 0;
    unsigned long baudrate;

    printf("\r\n");
    printf("SERIAL CONSOLE PORT SETTINGS:\r\n");  
    
    /*set the baudrate*/
    printf("\r\n");
    
    while (1)
    {
        printf("Set the baud rate of the serial console port [%lu]?\r\n    9600, 19200, 38400, 57600, 115200\r\n",nvParamsPtr->baudrate);
        readLine(buffer);  
        if (buffer[0] == '\0')
        {
            baudrate = nvParamsPtr->baudrate;
        }
        else
        {

            baudrate = atol(buffer);
        }
        if (isBaudRateValid(baudrate))    
        {
            if (nvParamsPtr->baudrate != baudrate)
            {
                printf("  The new baud rate is %lu\r\n", baudrate);
                printf("  The baud rate will be changed on next power up\r\n");
                printf("  Please set the baud rate for your terminal accordingly\r\n");
            }
            nvParamsPtr->baudrate = baudrate;
            updated = 1;
            break;
        }

        printf("The baud rate has to be 1200, 2400, 4800, 9600, 19200, 38400, 57600, or 115200\r\n");
    }

    return (updated);
}

/*ad*
 *
 * Update board parameters if there is any change.
 *
 * @param     nvParamsPtr     pointer to structure with development board parameters
 *
 * @return    0               board parameters are updated
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *          below is for NET+OS development boards only.
 *
*ad*/         
static int NAUpdateDialog(devBoardParamsType * nvParamsPtr)     
{
   /* 
    * save all the changes in NVMEM
    * if the APP_USE_NVRAM flag is set to 0, just update default parameters.
    * 
    */
    printf("\r\n");
    
    if (NAGetAppUseNvram () != APP_DO_NOT_USE_NVRAM)
    {
        printf("Saving the changes in NV memory...");
        fflush(stdout);
        if (customizeWriteDevBoardParams (nvParamsPtr) == 0)
        {
            printf ("Done.\r\n");
        }
        else
        {
            printf ("\r\nError writing to NVRAM.\r\n");
            tx_thread_sleep(NABspTicksPerSecond);
        }
    }        
    else
    {
        printf("APP_USE_NVRAM is set to 0 in appconf.h.  Your changes will not be saved.\r\n");
        tx_thread_sleep(NABspTicksPerSecond);
    }
         
    tx_thread_sleep(NS_MILLISECONDS_TO_TICKS(250)); /*to see better*/
    return (0);
}

/*ad*
 *
 * Sets up configuration dialog if APP_DIALOG_PORT is defined.
 *
 *  Parameters:
 *
 *      none
 *
 *  Return Values:
 *
 *      none
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @note    THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *          below is for NET+OS development boards only.
 *
*ad*/ 
void customizeDialog(void)
{
    devBoardParamsType *nvParams;
    unsigned int updated = 0;
    char myBuffer[128] = {0};

    /* array of error messages used for IP validation */
    char *validationfailure[14] =
    {    
        "No Error. Valid IP Address.",
        "IP Address is not subnetable. Subnet Mask and Gateway must be 0.0.0.0.",
        "IP Address 0.0.0.0 is not allowed.",
        "IP Address and Gateway Address match.",
        "IP Address or Gateway Address is a Multicast Address.",
        "IP Address or Gateway Address is a Local Loopback Address.",
        "IP Address or Gateway Address can not begin with 0.",
        "Subnet Mask can not be 0.0.0.0.",
        "Subnet Mask can not exceed 255.255.255.252.",
        "Invalid IP Address. It is the Network ID of the subnet.",
        "Invalid IP Address. It is the subnet Broadcast Address.",
        "Invalid Gateway Address. It is the Network ID of the subnet.",
        "Invalid Gateway Address. It is the subnet Broadcast Address.",
        "Invalid Gateway Address. It is not on the subnet."
    };

    if (NAGetAppDialogPort () == 0)
    {
        /* No DIALOG defined */ 
        return;
    }

    nvParams = (devBoardParamsType *)malloc(sizeof(devBoardParamsType));
    if (nvParams == NULL)
    {
        return;
    }

        /*  
         *  customizeReadDevBoardParams() reads the contents of the section of NVRAM that contains
         *  the development board's startup parameters if APP_USE_NVRAM flag is set to a nonzero value. 
         *  If APP_USE_NVRAM is 0, customizeReadDevBoardParams() reads the default parameters as defined
         *  in appconfig.h. customizeReadDevBoardParams() and default data struct are in 
         *  customize\boardParams.c.
         *
     */
    do
    {
        updated = 0;
        customizeReadDevBoardParams (nvParams);

        /* Open Dialog for Non-Blocking Mode */
        NAOpenDialog (0, nvParams);

        /* Display Configuration */
        NADisplayDialog (nvParams);

        /* Changing configuration? */
        if (NAConfigureDialog (nvParams) == 1)
        {

         /* 
          * we want to loop until the guy provides us with a valid set of 
          * ip address, gateway and subnet mask
          * by looping here we keep the guy from having to reenter the password
          */
          do
          {
              updated = NANetworkDialog (nvParams);
              if(updated != DIALOGUPDATED)    /*there is an error */
              {
                  printf("\r\n");
                  printf("\r\n");
                  printf("-----------------------------------------------------------------------------\r\n");
                  printf("Invalid network information entered. Please review and reenter.\r\n");
                  printf("\r\nERROR %d: %s\r\n", updated, validationfailure[updated]);
                  printf("\r\nHit return to continue.\r\n");
                  printf("-----------------------------------------------------------------------------\r\n");
                  readLine(myBuffer);
                  printf("\r\n");
              }
          } while (updated != DIALOGUPDATED);
          
          updated |= NAHardwareDialog (nvParams);
          updated |= NASecurityDialog (nvParams);
          if (updated)
          {
            /* Update Dialog */
             NAUpdateDialog (nvParams);
          }
        }
        /* Close Dialog */
        NACloseDialog ();
    
    }while (updated != 0);   
    
    free(nvParams);

}
#endif /*#if BSP_ENABLE_DIALOG*/





