/*
 *  Copyright (c) 1996-2007 Digi International Inc., All Rights Reserved
 *
 *  This software contains proprietary and confidential information of Digi
 *  International Inc.  By accepting transfer of this copy, Recipient agrees
 *  to retain this software in confidence, to prevent disclosure to others,
 *  and to make no use of this software other than that for which it was
 *  delivered.  This is an unpublished copyrighted work of Digi International
 *  Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
 *  prohibited.
 *
 *  Restricted Rights Legend
 *
 *  Use, duplication, or disclosure by the Government is subject to
 *  restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
 *  Technical Data and Computer Software clause at DFARS 252.227-7031 or
 *  subparagraphs (c)(1) and (2) of the Commercial Computer Software -
 *  Restricted Rights at 48 CFR 52.227-19, as applicable.
 *
 *  Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
 *
 * =======================================================================
 * This header file contains declarations and constants used by the
 * NET+OS serial driver.
 *
 *
 * Edit Date/Ver   Edit Description
 * ==============  ===================================================
 * 10/12/99        Merge support for SIO_TERM_SEND_CR
 * 11/17/99        Moved definition of XTAL_FREQ to bsp.h.
 * 5/5/00           JFU: added support for BAUD_115200_DIV
 * 6/26/00		   jzw: the algorithms used for calculating the N register
 *					value from the baud rate, is wrong. It happens to work because
 *					an integer is used so the remainder is discarded
 *					this refers to SCR-869
 * 					the problem was probably introduced because  the /16 for the
 *					N register value was factored in as an afterthought. This causes
 * 					the remainder. The key is to perform the division by 16 PRIOR
 *					to the subtraction by -1
 *
 * 10/10/00  JFU    Updated non parity setting for the serial port.  Used AND operation.
 *
 * 11/20/00  JFU    Took out some #define statement related to baud rate calculation.
 * 04/11/02  PMJ    Added support to sense the DSR state.  See SIO_GET_STATUS_DSR.
 * 04/12/02  PMJ    Extracted the public portion of the API and moved to serParams.h
 * 08/02/02  PMJ    New and improved serial API with select functionality and better
 *                  modem signals control.  Introducing the RTS Transmission encapsulation
 *                  which can be used for RS-485 half-duplex style communucation.
 * 09/24/02  LZ     Added 20UM support.
 * 11/11/02  JW     Changed MAX_BAUD_RATE to 115200.
 */

#include "Npttypes.h"
#include "fwtypes.h"
#include "serlDev.h"
#include "bsp_api.h"
#include "reg_def.h"


/* netos_serl.h */

/*
 * Header file for netos serial driver.
 */


#ifndef _NETOS_SERL_H_
#define _NETOS_SERL_H_

#ifdef __cplusplus
extern "C"
{
#endif

/* GENERAL DEFINES FOR Digi International SERIAL DEVICES */
#ifndef UINT32
#define UINT32 unsigned int
#endif
#ifndef UINT8
#define UINT8 unsigned char
#endif
#ifndef UINT16
#define UINT16 unsigned short
#endif

#define NUM_NCC_SER_CHAN      2         /* currently the NET+ARM chip support 2 serial channels */

#define SERL_POLL_RATE  4


#define XON 0x11                        /* Values for xon and xoff */
#define XOFF 0x13                       /* xoff to stop xon to send */



/* initial settings in lieu of NV storage, CONSOLE_BAUDRATE defined in configAll.h */
#define CONSOLE_OPTIONS         CREAD | CS8 | CLOCAL  /* init value = enable rx,8,n,1 */

/* REGISTER DESCRIPTION OF NCC DUART */
/***
 *** Register offsets from base address. The NCC has two physical serial
 *** ports (channels). The registers for each port are identical to each other.
 *** Physical port 1 starts at NCC_SER_CH1_BASE and physical port 2 starts at
 *** NCC_SER_CH1_BASE + SER_CH2_OFFSET.
 *** Note: Basic registers are 32 bits in size (for an old hack like me that's
 *** huge over-kill)!
 ***/

#define NCC_SER_BASE        (0xFFD00000U)
#define CH2_OFFSET             (0x00000040U)
#define SER_REG(base_addr,reg)      *((volatile UINT32 *)((base_addr) + (reg)))
#define SER_REG_BYTE(base_addr,reg) *((volatile UINT8 *)((base_addr) + (reg)))
#define SER_REG_SHORT(base_addr,reg) *((volatile UINT16 *)((base_addr) + (reg)))

/* Add these offsets to the port base addresses to access a particular register */
#define CTRL_REGA              (0x00000000U)
#define CTRL_REGB              (0x00000004U)
#define STAT_REG               (0x00000008U)
#define BAUD_REG               (0x0000000CU)
#define DATA_REG               (0x00000010U)
#define RECV_BUF_TIMER_REG     (0x00000014U)
#define RECV_CHAR_TIMER_REG    (0x00000018U)
#define RECV_MATCH_REG         (0x0000001CU)
#define RECV_MATCH_MASK_REG    (0x00000020U)
/*** Control Register A defined bits ***/
/* Start Register at 0 and OR in what you want */
/* Control section ... */
#define SER_CHANNEL_ENABLE     0x80000000U  /* Enable Serial channel xmit & recv  */
#define SEND_BREAK             0x40000000U  /* Send Break characters              */
#define CTRL_EVEN_PARITY       0x18000000U
#define CTRL_ODD_PARITY        0x08000000U
#define CTRL_MARK_PARITY       0x38000000U
#define CTRL_SPACE_PARITY      0x28000000U

#define CTRL_NO_PARITY         0xC7FFFFFFU
#define CTRL_ONE_STOP_BIT      0x00000000U
#define CTRL_TWO_STOP_BITS     0x04000000U
#define CTRL_FIVE_BITS         0x00000000U
#define CTRL_SIX_BITS          0x01000000U
#define CTRL_SEVEN_BITS        0x02000000U
#define CTRL_EIGHT_BITS        0x03000000U
#define CTS_XMIT               0x00800000U  /* Transmitter on when CTS active     */
#define RTS_RECV               0x00400000U  /* RTS active only when receiver ~full */
#define REMOTE_LOOP            0x00200000U  /* Enable Remote loopback testing     */
#define LOCAL_LOOP             0x00100000U  /* Enable Local loopback testing      */
#define OUT_BIT2               0x00080000U  /* Mask for output bit 2              */
#define OUT_BIT1               0x00040000U  /* Mask for output bit 1              */
#define DTR_ON                 0x00020000U  /* Enable DTR hardware handshaking    */
#define RTS_ON                 0x00010000U  /* Enable RTS hardware handshaking    */
#define REGA_OPTIONS_MASK      0x403FFFFFU  /* Mask off hardware options bits     */
#define REGA_INTFLAGS_MASK     0xFFFF0000U  /* Mask off interrupt condition bits  */

/* Receiver Interrupt section ... */
/* Setting these bits will enable interrrupt for that bit's condition */
#define RECV_BREAK_IRQ         0x00008000U
#define RECV_FRAME_ERR_IRQ     0x00004000U
#define RECV_PARITY_ERR_IRQ    0x00002000U
#define RECV_OVRRUN_ERR_IRQ    0x00001000U
#define RECV_RDY_IRQ           0x00000800U
#define RECV_HALF_FULL_IRQ     0x00000400U  /* Receiver FIFO is half full         */
#define RECV_CLOSED_IRQ        0x00000200U  /* Receiver FIFO is closed            */
#define RECV_DMA_ON            0x00000100U  /* Turn on receiver DMA requests?     */
#define RECV_DCD_IRQ           0x00000080U  /* Change in state of DCD             */
#define RECV_RI_IRQ            0x00000040U  /* Change in state of RI              */
#define RECV_DSR_IRQ           0x00000020U  /* Change in state of DSR             */
#define RECV_CTS_IRQ           0x00000010U  /* Change in state of CTS             */

#define RECV_IRQ_MASK	       (RECV_RDY_IRQ | RECV_HALF_FULL_IRQ | RECV_CLOSED_IRQ | RECV_BREAK_IRQ)

/* Transmitter Interrupt section ... */
/* Setting these bits will enable interrrupt for that bit's condition */
#define XMITR_RDY_IRQ          0x00000008U
#define XMITR_HALF_FULL_IRQ    0x00000004U  /* Transmitter FIFO is half full      */
#define XMITR_CLOSED_IRQ       0x00000002U  /* Transmitter FIFO is closed         */
#define XMITR_DMA_ON           0x00000001U  /* Turn on Transmitter DMA requests?  */
/* End Control register A */

/*** Control Register B defined bits.
 *** Yes, a control register spans over 32 bits.
 ***/
/* Start Register at 0 and OR in what you want */
#define RECV_DATA_MATCH1_ON    0x80000000U  /* Looking for this data pattern      */
#define RECV_DATA_MATCH2_ON    0x40000000U  /* Looking for this data pattern      */
#define RECV_DATA_MATCH3_ON    0x20000000U  /* Looking for this data pattern      */
#define RECV_DATA_MATCH4_ON    0x10000000U  /* Looking for this data pattern      */
#define RECV_BUF_GAP_TIMER_ON  0x08000000U
#define RECV_CHAR_GAP_TIMER_ON 0x04000000U

#define RTSTX_ENCAPSULATION    0x00008000U
/* End Control register B */

/*** Serial Channel Status Register ***/
/* A one in the following bit positions means the condition is true */
/* Receiver section */
#define MATCH1_TRUE            0x80000000U  /* Recvr buf closed - got char MATCH1 */
#define MATCH2_TRUE            0x40000000U  /* Recvr buf closed - got char MATCH2 */
#define MATCH3_TRUE            0x20000000U  /* Recvr buf closed - got char MATCH3 */
#define MATCH4_TRUE            0x10000000U  /* Recvr buf closed - got char MATCH4 */
#define BUF_GAP_TIMEOUT        0x08000000U  /* Recvr buf closed - buf GAP timeout */
#define CHAR_GAP_TIMEOUT       0x04000000U  /* Recvr buf closed -char GAP timeout */
/* Bits 25 to 22 reserved for NCC */
#define NUM_DATA_BYTES         0x00300000U  /* Recv data available when RECV_RDY
                                             * is true. Sort of equals the number
                                             * of data bytes:
                                             * 01 = one byte, 02 = two bytes, 03 =
                                             * three bytes and 00 = four bytes.    */
#define DCD_STATE              0x00080000U  /* State of DCD                       */
#define RI_STATE               0x00040000U  /* State of RI                        */
#define DSR_STATE              0x00020000U  /* State of DSR                       */
#define CTS_STATE              0x00010000U  /* State of CTS                       */
#define RECV_BREAK             0x00008000U
#define RECV_FRAME_ERR         0x00004000U
#define RECV_PARITY_ERR        0x00002000U
#define RECV_OVRRUN_ERR        0x00001000U
#define RECV_RDY               0x00000800U
#define RECV_HALF_FULL         0x00000400U  /* Receiver FIFO is half full         */
#define RECV_CLOSED            0x00000200U  /* Receiver FIFO is closed            */
#define RECV_FULL              0x00000100U  /* Receiver FIFO is full              */
#define RECV_DCD               0x00000080U  /* Change in state of DCD             */
#define RECV_RI                0x00000040U  /* Change in state of RI              */
#define RECV_DSR               0x00000020U  /* Change in state of DSR             */
#define RECV_CTS               0x00000010U  /* Change in state of CTS             */
#define RX_ERR_CONDITION       (RECV_FRAME_ERR | RECV_PARITY_ERR | RECV_OVRRUN_ERR)
#define CLR_ALL_RX_CONDITIONS  (RECV_BREAK_IRQ | RECV_FRAME_ERR_IRQ | RECV_PARITY_ERR | \
                               RECV_OVRRUN_ERR | RECV_HALF_FULL | RECV_FULL | \
                               RECV_DCD | RECV_RI | RECV_DSR | RECV_CTS)
/*  IRQs used for read active select    */
#define RECV_SELECT_IRQS       (RECV_DCD_IRQ | RECV_RI_IRQ | RECV_DSR_IRQ)

/* Transmitter section */
#define XMITR_EMPTY            0x00000001U  /* Transmitter FIFO is empty          */
#define CLR_ALL_TX_CONDITIONS  (XMITR_RDY_IRQ | XMITR_HALF_FULL_IRQ | XMITR_CLOSED_IRQ | XMITR_EMPTY)
/*** End Serial Channel Status Register ***/

/*** Serial Channel Bit-Rate Register ***/
/* Control section ... */
#define BIT_GEN_ON             0x80000000U  /* Enable Bit-Rate Generator          */
#define BIT_RATE_X1            0x40000000U  /* Timing mode: 0 = X16 1 = X1        */
#define BIT_RATE_X16           0x00000000U  /* Timing mode: 0 = X16 1 = X1        */
#define BIT_TDCR_RDCR_X1       0x00000000U  /* for 20UM only. Transmit/Receive Divide Clock Rate, D20:19/D17:16=00/00 1X clock mode*/
#define BIT_TDCR_RDCR_X8	   0x00090000U  /* for 20UM only. Transmit/Receive Divide Clock Rate, D20:19/D17:16=01/01 8X clock mode*/
#define BIT_TDCR_RDCR_X16      0x00120000U  /* for 20UM only. Transmit/Receive Divide Clock Rate, D20:19/D17:16=10/10 16X clock mode*/
#define BIT_TDCR_RDCR_X32      0x001b0000U  /* for 20UM only. Transmit/Receive Divide Clock Rate, D20:19/D17:16=11/11 32X clock mode*/
#define RECV_CLK_SRC_EXT       0x20000000U  /* Receive clock source is External   */
#define RECV_CLK_SRC_INT       0x00000000U  /* Receive clock source is Internal   */
#define XMIT_CLK_SRC_EXT       0x10000000U  /* Transmit clock source is External  */
#define XMIT_CLK_SRC_INT       0x00000000U  /* Transmit clock source is Internal  */
#define RECV_CLK_OUT_ON        0x08000000U  /* Drive RXCLK out OUT1 of Port A/B   */
#define RECV_CLK_OUT_OFF       0x00000000U  /* Do not drive RXCLK to OUT1         */
#define XMIT_CLK_OUT_ON        0x04000000U  /* Drive TXCLK out OUT2 of Port C     */
#define XMIT_CLK_OUT_OFF       0x00000000U  /* Do not drive TXCLK to OUT2         */
#define BIT_RATE_CLK_EXT       0x00000000U  /* Use the external Clk source        */
#define BIT_RATE_CLK_SYS       0x01000000U  /* Use System Clk as source Bit-Rate  */
#define BIT_RATE_CLK_OUT1A     0x02000000U  /* Use OUT1A as source Bit-Rate Clk   */
#define BIT_RATE_CLK_OUT2A     0x03000000U  /* Use OUT2A as source Bit-Rate Clk   */
#define BIT_RATE_TXCINV        0x00800000U  /* TXD driven on rising edge of TX clock  */
#define BIT_RATE_RXCINV        0x00400000U  /* RRXD sampled on falling edge of RX clock */
#define BIT_RATE_CLKINV        0x00200000U  /* Receive Clock Invert               */

#define BIT_RATE_RX_DPLL       0x00001000U /* Recover clock from receive data */
/* Bits 23 to 11 are reserved */
/* Divide Rate section */
/* algorithms changed 6/26/00 multiply the baud rate * 10 by 16. Divide the XTAL_FREQ
 * by this product and then as the LAST step, subtract one from this quotient.
 */
#define BIT_RATE_DIV_MASK      0x000007FFU  /* Bit-Rate (baud) divisor count mask */
#define MIN_BAUD_RATE           75
#define MAX_BAUD_RATE           115200
#define DEFAULT_BAUD_RATE       9600  /* default used in case of error */
#define DEFAULT_BAUD_RATE_ERROR 5       /* 1 percent error */

/*** End Serial Channel Bit-Rate Register ***/

/*** Receive Character Gap Timer Register ***/
#define ENABLE_TIMER           0x80000000U  /* Enable Rx Char Gap Timer          */
#define MIN_GAP                0x00000000U  /* Minimum gap count                 */

/*** Serial Channel FIFO Register ***/
/* Writing data to this register loads data into the transmitter FIFO and reading
 * data from this register retreives data from the receiver FIFO.
 */
/*** End Serial Channel FIFO Register ***/


#define IVEC_TO_INUM(intVec)    ((int) (intVec))
#define INUM_TO_IVEC(intNum)    ((VOIDFUNCPTR *) (intNum))

#define SER1RX_INT_LVL      15    /* serial port interrupt level for each serial interrupt */
#define SER1TX_INT_LVL      14
#define SER2RX_INT_LVL      13
#define SER2TX_INT_LVL      12

#define INT_VEC_SERIAL1_RX        IVEC_TO_INUM(SER1RX_INT_LVL)  /* serial 1 receive interrupt */
#define INT_VEC_SERIAL1_TX        IVEC_TO_INUM(SER1TX_INT_LVL)  /* serial 1 transmit interrupt */
#define INT_VEC_SERIAL2_RX        IVEC_TO_INUM(SER2RX_INT_LVL)  /* serial 2 receive interrupt */
#define INT_VEC_SERIAL2_TX        IVEC_TO_INUM(SER2TX_INT_LVL)  /* serial 2 transmit interrupt */

#define SER1RX_DMA_LVL      25    /* serial port interrupt level for each serial interrupt */
#define SER1TX_DMA_LVL      24
#define SER2RX_DMA_LVL      23
#define SER2TX_DMA_LVL      22

#define DMA_VEC_SERIAL1_RX        IVEC_TO_INUM(SER1RX_DMA_LVL)  /* dma 7 serial 1 receive interrupt */
#define DMA_VEC_SERIAL1_TX        IVEC_TO_INUM(SER1TX_DMA_LVL)  /* dma 8 serial 1 transmit interrupt */
#define DMA_VEC_SERIAL2_RX        IVEC_TO_INUM(SER2RX_DMA_LVL)  /* dma 9 serial 2 receive interrupt */
#define DMA_VEC_SERIAL2_TX        IVEC_TO_INUM(SER2TX_DMA_LVL)  /* dma 10 serial 2 transmit interrupt */

			       
#define SIO_AVAIL_MODES_GET 0x1009

#define SIO_GET_REG         (0x100A)	/* Get EEEPROM'S register */
#define SIO_SET_REG         (0x100B)	/* Set EEEPROM'S register */

#define SIO_SLAVE_GET       (0X100C)   /* Get master / slave mode of SPI driver */

#define SIO_RD_ADDRESS_SET (0X100D)    /* Set read address for EEPROM  */
#define SIO_RD_ADDRESS_GET (0X100E)    /* Get read address for EEPROM  */
#define SIO_WR_ADDRESS_SET (0X100F)    /* Set write address for EEPROM */
#define SIO_WR_ADDRESS_GET (0X1010)    /* Get write address for EEPROM */

#define SIO_SET_LSB        (0X1011)    /* Set "Less significant bit" first */
#define SIO_GET_LSB        (0X1012)    /* Set "Less significant bit" first */

#define SIO_SPI_HW_SET     (0x1013)	/* Set hardware option for the SPI operation */
#define SIO_SPI_HW_GET     (0x1014)	/* Get hardware option for the SPI operation */
			       /* Currently includes direct connect and XICOR X25020 */

#define SIO_SPI_SS_SET     (0x1015)    /* Set port for generating "Slave Select" signal */
#define SIO_SPI_USE_SEL_ISR (0x1016)

#define SIO_SET_SLAVE_TIMEOUT (0x1017)  /* Set time out value for the SPI slave mode */
#define SIO_GET_SLAVE_TIMEOUT (0x1018)  /* Get time out value for the SPI slave mode */
#define SIO_SET_SA_SIGNAL     (0X1019)  /* Generate an additional
					   signal when slave is ready to accept CLK signals. */

#define SIO_SET_TXCINV        (0x101A) /* set TXD to be driven on rising edge of TX clock  */
#define SIO_SET_RXCINV        (0x101B)  /* set RRXD to be sampled on falling edge of RX clock */
#define SIO_SET_SPI_CLOCK_HI  (0x101C)  /* Set initial value of the clock signal High */
 

/* callback types */

#define SIO_CALLBACK_GET_TX_CHAR    1
#define SIO_CALLBACK_PUT_RCV_CHAR   2

#if 0 /* new definitions in termios.h */
#define CLOCAL      0x1 /* ignore modem status lines */
#define CREAD       0x2 /* enable device reciever-- NO longer used because we need a bit for sw handshake*/
#endif

#define inCSIZE       0xc /* bits 3 and 4 encode the character size */
#define inCS5     0x0 /* 5 bits */
#define inCS6     0x4 /* 6 bits */
#define inCS7     0x8 /* 7 bits */
#define inCS8     0xc /* 8 bits */

#define inHUPCL       0x10    /* hang up on last close */
#define inSTOPB       0x20    /* send two stop bits (else one) */
#define inPARENB      0x40    /* parity detection enabled (else disabled) */
#define inPARODD      0x80    /* odd parity  (else even) */
#define inXPAR	      0x100   /* Mark/space selector PARODD(space) else mark */

/* definitions for SIO_SPI_HW_SET */

#define SIO_SPI_DIRECT (0X1)  /* Direct connections */
#define SIO_SPI_X25020 (0X2)  /* XICOR X250200 EEPROM chip */
#define SIO_SPI_X25C640U (0X3)  /* FAIRCHILD FM25C640U EEPROM chip */

#define STATUS     int

/* serial device data structures */

typedef struct sio_drv_funcs SIO_DRV_FUNCS;

typedef struct sio_chan             /* a serial channel */
{
    SIO_DRV_FUNCS * pDrvFuncs;
}
 SIO_CHAN;

struct sio_drv_funcs                        /* driver functions */
{
    /*  This routine handles the IOCTL messages from the user.  */
    int (*ioctl)(SIO_CHAN *pSioChan, int cmd, void *arg);

    /*  Call interrupt level character output routine and enable interrupt  */
    int (*txStartup)(SIO_CHAN *pSioChan);

    /*  This routine installs interrupt callbacks for transmitting and receiving characters.    */
    int (*callbackInstall)(SIO_CHAN *pSioChan, int callbackType, STATUS (*callback)(void), void *callbackArg);

    /*  driver function for polling serial input.  Its defined in netos_serl.c  */
    int (*pollInput)(SIO_CHAN *pSioChan, char *inChar);

    /*  driver function for polling serial output.  Its defined in netos_serl.c */
    int (*pollOutput)(SIO_CHAN *  pSioChan, char outChar);
};

/* device initialization structure, required for interrupt operation */
typedef struct
{
    UINT32 rxvector;
    UINT32 rxintLevel;
    UINT32 txvector;
    UINT32 txintLevel;
}
ONCC_SIO_CHAN_PARAS;

enum brk_states {
    BRK_IDLE,
    BRK_REQUESTED,
    BRK_ACTIVE,
};

typedef struct           /* ONCC_CHAN */
{
    /* must be first */
    SIO_CHAN    sio;                /*  standard SIO_CHAN element */
    STATUS      (*getTxChar) (void);    /*  installed Tx callback routine */
    STATUS      (*putRcvChar) (void);   /*  installed Rx callback routine */
    void        *getTxArg;          /*  argument to Tx callback routine */
    void        *putRcvArg;         /*  argument to Rx callback routine */
    UINT8       rxlevel;            /*  Rx interrupt level for this device */
    UINT8       txlevel;            /*  Tx interrupt level for this device */
    UINT8       rxDmalevel;
    UINT8       txDmalevel;
    UINT32      channelMode;        /*  such as INT, POLL modes */
    int         baudRate;           /*  the current baud rate */
    int         baudRateError;      /*  the error in percentage that the user can accept  */
    int         port;               /*  port number for this channel */
    int         options;            /*  hw options for SIO_HW_OPTS_SET */
    int         opened;             /*  whether the serial port is already in use */
    unsigned    openmode;           /*  RD, WR, BLOCKING? */
    int         echo;
#define OP_MODE_INSERT_CR               0x01
#define OP_MODE_NO_HANDSHAKE            0x02
#define OP_MODE_HW_HANDSHAKE            0x04
#define OP_MODE_SW_HANDSHAKE            0x08
#define OP_MODE_HANDSHAKE_MASK		(OP_MODE_NO_HANDSHAKE \
					 | OP_MODE_HW_HANDSHAKE \
					 | OP_MODE_SW_HANDSHAKE)
#define OP_MODE_RTS_TX_ENCAPSULATION    0x10
#define OP_MODE_RTS_TX_ACTIVE           0x20
    int         opMode;

    /* the register address this serial channel uses */
    #ifdef NET_OS_SPECIFIC
    volatile UINT8 *base_address;
    #else
    UINT8 *base_address;
    #endif

    /*  The RTS encapsulation is a bit of a hack.  The driver will increment the count of
        characters to transmit when the caller writes and insert a loopback path to the 
        receiver, which decrements the count.  When the receiver detects the count of zero, 
        it should de-activate the RTS signal.  */
    UINT32      chars2Encapsuate;

    int         (*cbSelect)(int);   /*  callback used for select */
    unsigned    priorState;         /*  prior interrupt enable state -- used for select  */
    int         serChannel;         /*  netarm channel number   */

    int		serial_channel; /* Overall channel number */

    /* these are added for debugging purposes */
    int         rx_overflow;

    /*-----------------05/10/2000 12:04PM---------------
     * added for xon and xoff
     * --------------------------------------------------*/
    BOOLEAN     hw_flowc;           /*  h/w flow ctrl (en/dis)able  */
    BOOLEAN     sw_flowc;           /*  s/w flow ctrl (en/dis)able (xon/xoff) */
    UINT8       xoff_ch;            /*  char to use for xoff */
    UINT8       xon_ch;             /*  char to use for xon */

    /* Replacing SINGLE flow_controled with individual statuses */
    //BOOLEAN     flow_controled;     /*  flow ctrl active */
    BOOLEAN	rx_hw_flow;
    BOOLEAN	tx_sw_flow;
    BOOLEAN	rx_sw_flow;
    BOOLEAN	tx_user_flow;
    BOOLEAN	rx_user_flow;
    BOOLEAN	rx_alarm_flow; /* Serial alarms code flow control */
    BOOLEAN     tx_rtstoggle_flow;  /* Flow control during predelay */


    /* Pre and post delay times when RTS toggle is enabled */ 
    unsigned long predelay_timeout;
    unsigned long postdelay_timeout;

    /* Timers for rtstoggle pre and post delay */
    TX_TIMER predelay_timer;
    TX_TIMER postdelay_timer;

#if NETWORK_STACK == TRECK
    /* Select handling. */
    uint32	    select_context;
    int		    checkMask;
#endif
    
    /* TX break data */
    enum brk_states breakstatus;
    int breaktime;
    ulong lastbreak;
}
ONCC_CHAN;

#ifdef __cplusplus
}
#endif


#endif   /* netos_serl.h */
