/*
 *            Copyright (c) 1998-2003 by NETsilicon Inc.
 *
 *  This software is copyrighted by and is the sole property of 
 *  NETsilicon.  All rights, title, ownership, or other interests
 *  in the software remain the property of NETsilicon.  This
 *  software may only be used in accordance with the corresponding
 *  license agreement.  Any unauthorized use, duplication, transmission,  
 *  distribution, or disclosure of this software is expressly forbidden.
 *
 *  This Copyright notice may not be removed or modified without prior
 *  written consent of NETsilicon.
 *
 *  NETsilicon, reserves the right to modify this software
 *  without notice.
 *
 *  NETsilicon
 *  411 Waverley Oaks Road                  USA 781.647.1234
 *  Suite 304                               http://www.netsilicon.com
 *  Waltham, MA 02452                       AmericaSales@netsilicon.com
 *
 *************************************************************************
 *  Copyright (c) 2003 Digi International Inc., All Rights Reserved
 * 
 *  This software contains proprietary and confidential information of Digi
 *  International Inc.  By accepting transfer of this copy, Recipient agrees
 *  to retain this software in confidence, to prevent disclosure to others,
 *  and to make no use of this software other than that for which it was
 *  delivered.  This is an unpublished copyrighted work of Digi International
 *  Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
 *  prohibited.
 * 
 *  Restricted Rights Legend
 *
 *  Use, duplication, or disclosure by the Government is subject to
 *  restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
 *  Technical Data and Computer Software clause at DFARS 252.227-7031 or
 *  subparagraphs (c)(1) and (2) of the Commercial Computer Software -
 *  Restricted Rights at 48 CFR 52.227-19, as applicable.
 *
 *  Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
 *
 *****************************************************************************
 *
 *  Description.
 *
 *  This file contains the code that sets up the GPIO ports for the 9750.
 *  This file should not be updated and instead updates should be made to 
 *  gpio.h in the platofrm directory.
 *
 *
 * Edit Date/Ver   Edit Description
 * ==============  =======================================================
 *
 *
 */

/*
 * Do not attempt to trace registers before the C library is
 * setup.  The global variables used by the trace library will
 * be in an undefined state until after the C library has
 * initilized the data segement.
 */
#ifdef NA_REG_TRACE_ENABLED
    #ifdef _REG_DEF_H
        #error Trace code compiled into init code.
    #endif
    #undef NA_REG_TRACE_ENABLED
#endif

#include <stdlib.h>
#include <Npttypes.h>
#include "bsp.h"
#include <narmled.h>
#include <nareset.h>
#include "init.h"
#include "bootldr.h"
#include "reg_def.h"

#include "gpio_def.h"
#include <gpio.h>
#include <bum_reg.h>
#include "NA_ISR.h"


static unsigned long volatile * const gpioConfigReg[] =
{
    NA_GPIO_CONFIGREG1,
    NA_GPIO_CONFIGREG2,
    NA_GPIO_CONFIGREG3,
    NA_GPIO_CONFIGREG4,
    NA_GPIO_CONFIGREG5,
    NA_GPIO_CONFIGREG6,
    NA_GPIO_CONFIGREG7,
    NA_GPIO_CONFIGREG8,
    NA_GPIO_CONFIGREG9,
    NA_GPIO_CONFIGREG10
};

static unsigned long volatile * const gpioStatusReg[] =
{
    NA_GPIO_STATUSREG1,
    NA_GPIO_STATUSREG2,
    NA_GPIO_STATUSREG3
};

static unsigned long volatile * const gpioControlReg[] =
{
    NA_GPIO_CONTRLREG1,
    NA_GPIO_CONTRLREG2,
    NA_GPIO_CONTRLREG3
};


/* NS9360 supports 18-bit no. of colors for lower power disipation */
#if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_24BIT_APP)
    #error GPIO Mux LCD does not support LCD 24-bit no. of colors
#endif

/******************** GPIO 0 Configuration **********************/
static void naSetupPort0(void)
{
#if (BSP_GPIO_MUX_SERIAL_B != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 0 between Serial B and DMA 0
    #endif

    #if (BSP_GPIO_MUX_TIMER_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 0 between Serial B and Timer 1
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p0, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    #if (BSP_GPIO_MUX_TIMER_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 0 between DMA 0 and Timer 1
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p0, NA_GPIO_SELECT_MUX_FUNC1);
    
#elif (BSP_GPIO_MUX_TIMER_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p0, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN0 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p0, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN0 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p0, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p0, NA_GPIO_OUTPUT_STATE);

    #else

        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);
    #endif
#endif
}

/******************** GPIO 1 Configuration **********************/
static void naSetupPort1(void)
{
#if (BSP_GPIO_MUX_SERIAL_B != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 1 between Serial B and DMA 0
    #endif

    #if (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 1 between Serial B and IRQ 0
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p1, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    #if (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 1 between DMA 0 and IRQ 0
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p1, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p1, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN1 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p1, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN1 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p1, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);
    #endif
#endif
}
/******************** GPIO 2 Configuration **********************/
static void naSetupPort2(void)
{
#if (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)

    #if (BSP_GPIO_MUX_TIMER_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 2 between Serial B and Timer 0
    #endif

    #if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 2 between Serial B and DMA 1
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p2, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p2, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_TIMER_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 2 between Timer 0 and DMA 1
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p2, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p2, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN2 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p2, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN2 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p2, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p2, NA_GPIO_OUTPUT_STATE);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 3 Configuration **********************/
static void naSetupPort3(void)
{
#if (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 3 between Serial B and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 3 between Serial B and DMA 0
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p3, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p3, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 3 between IEEE 1284 and DMA 0
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p3, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p3, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN3 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p3, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN3 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p3, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p3, NA_GPIO_OUTPUT_STATE);

    #else

        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 4 Configuration **********************/
static void naSetupPort4(void)
{
#if (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 4 between Serial B and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 4 between Serial B and DMA 0
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p4, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p4, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 4 between IEEE 1284 and DMA 0
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p4, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p4, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN4 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p4, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN4 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p4, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p4, NA_GPIO_OUTPUT_STATE);

    #else

        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);
    #endif
#endif
}
/******************** GPIO 5 Configuration **********************/
static void naSetupPort5(void)
{
#if (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 5 between Serial B and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 5 between Serial B and DMA 0
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p5, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p5, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 5 between IEEE 1284 and DMA 0
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p5, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p5, NA_GPIO_SELECT_MUX_FUNC2);

#else

    #if (BSP_GPIO_INITIAL_STATE_PIN5 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p5, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN5 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p5, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 6 Configuration **********************/
static void naSetupPort6(void)
{
#if (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_SPI_APP) || \
    (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 6 between Serial B and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_TIMER_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 6 between Serial B and Timer 7
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE) && \
        (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p6, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p6, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_TIMER_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 6 between IEEE 1284 and Timer 7
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p6, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_TIMER_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p6, NA_GPIO_SELECT_MUX_FUNC2);

#else

    #if (BSP_GPIO_INITIAL_STATE_PIN6 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p6, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN6 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p6, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 7 Configuration **********************/
static void naSetupPort7(void)
{
#if (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_SPI_APP) || \
    (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 7 between Serial B and DMA 0 alternate
    #endif

    #if (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 7 between Serial B and IRQ 1
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE) && \
        (BSP_GPIO_MUX_SERIAL_B == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p7, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p7, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    #if (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 7 between DMA 0 alternate and IRQ 1
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p7, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p7, NA_GPIO_SELECT_MUX_FUNC2);

#else

    #if (BSP_GPIO_INITIAL_STATE_PIN7 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p7, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN7 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p7, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG1, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 8 Configuration **********************/
static void naSetupPort8(void)
{

#if (BSP_GPIO_MUX_SERIAL_A != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p0, NA_GPIO_SELECT_MUX_FUNC0);

#else

    #if (BSP_GPIO_INITIAL_STATE_PIN8 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p8, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN8 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p8, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p0, NA_GPIO_OUTPUT_STATE);

    #else

        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 9 Configuration **********************/
static void naSetupPort9(void)
{

#if (BSP_GPIO_MUX_SERIAL_A != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    #if (BSP_GPIO_MUX_TIMER_8 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 9 between Serial A and Timer 8 alternate
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p1, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_TIMER_8 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p1, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN9 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p9, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN9 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p9, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 10 Configuration **********************/
static void naSetupPort10(void)
{

#if (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_PWM0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 10 between Serial A and Pulse Width Modulator 0
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p2, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p2, NA_GPIO_SELECT_MUX_FUNC0);
    #endif
    
#elif (BSP_GPIO_MUX_PWM0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p2, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN10 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p10, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN10 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p10, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p2, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 11 Configuration **********************/
static void naSetupPort11(void)
{
#if (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 11 between Serial A and IRQ 2 alternate
    #endif

    #if (BSP_GPIO_MUX_TIMER_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 11 between Serial A and Timer 0 alternate
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p3, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p3, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    #if (BSP_GPIO_MUX_TIMER_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 11 between IRQ 2 alternate and Timer 0 alternate
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p3, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_TIMER_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p3, NA_GPIO_SELECT_MUX_FUNC2);

#else

    #if (BSP_GPIO_INITIAL_STATE_PIN11 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p11, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN11 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p11, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p3, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);
    #endif
#endif
}
/******************** GPIO 12 Configuration **********************/
static void naSetupPort12(void)
{
#if (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_PWM1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 12 between Serial A and Pulse Width Modulator 1
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p4, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p4, NA_GPIO_SELECT_MUX_FUNC0);
    #endif
    
#elif (BSP_GPIO_MUX_PWM1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p4, NA_GPIO_SELECT_MUX_FUNC2);

#else

    #if (BSP_GPIO_INITIAL_STATE_PIN12 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p12, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN12 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p12, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p4, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 13 Configuration **********************/
static void naSetupPort13(void)
{
#if (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)

    #if (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 13 between Serial A and IRQ 0 alternate
    #endif
    
    #if (BSP_GPIO_MUX_PWM2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 13 between Serial A and Pulse Width Modulator 2
    #endif
    
    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p5, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p5, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    #if (BSP_GPIO_MUX_PWM2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 13 between IRQ 0 alternate and Pulse Width Modulator 2
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p5, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_PWM2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p5, NA_GPIO_SELECT_MUX_FUNC2);

#else

    #if (BSP_GPIO_INITIAL_STATE_PIN13 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p13, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN13 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p13, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);
    #endif
#endif
}
/******************** GPIO 14 Configuration **********************/
static void naSetupPort14(void)
{
#if (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_SPI_APP) || \
    (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_TIMER_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 14 between Serial A and Timer 1
    #endif

    #if (BSP_GPIO_MUX_PWM3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 14 between Serial A and Pulse Width Modulator 3
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE) && \
        (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p6, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p6, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_TIMER_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_PWM3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 14 between Timer 1 and Pulse Width Modulator 3
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p6, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_PWM3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p6, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN14 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p14, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN14 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p14, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 15 Configuration **********************/
static void naSetupPort15(void)
{
#if (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_SPI_APP) || \
    (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_TIMER_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 15 between Serial A and Timer 2
    #endif

    #if (BSP_GPIO_MUX_LCD_EXTERNAL_CLOCK_IN != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 15 between Serial A and LCD
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE) && \
        (BSP_GPIO_MUX_SERIAL_A == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p7, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p7, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_TIMER_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD_EXTERNAL_CLOCK_IN != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 15 between Timer 2 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p7, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD_EXTERNAL_CLOCK_IN != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p7, NA_GPIO_SELECT_MUX_FUNC2);
#else

    #if (BSP_GPIO_INITIAL_STATE_PIN15 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p15, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN15 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p15, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG2, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 16 Configuration **********************/
static void naSetupPort16(void)
{
#if (BSP_GPIO_MUX_USB_OVERCURRENT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 16 between USB Overcurrent and IEEE 1284 Paper Jam alternate
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p0, NA_GPIO_SELECT_MUX_FUNC0);

/* #elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) */
#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    /* This pin is used to enable the tranceiver and not used as an alternative function to paper jam */
#ifdef PCM_AUTO_MODE
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p0, NA_GPIO_SELECT_MUX_FUNC1);
#else 
    /* Defect in ns9750 requires Manual Mode override */
    /* Logic requires HIGH signal to enable tranceiver */
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p0, NA_GPIO_SELECT_MUX_GPIO_OUTPUT);
#endif

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN16 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p16, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN16 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p16, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p0, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 17 Configuration **********************/
static void naSetupPort17(void)
{
#if (BSP_GPIO_MUX_USB_OVERCURRENT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p1, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN17 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p17, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN17 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p17, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 18 Configuration **********************/
static void naSetupPort18(void)
{
#if (BSP_GPIO_MUX_ETHERNET_CAM == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 18 between Ethernet CAM and LCD
    #endif

    #if (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 18 between Ethernet CAM and IRQ3 alternate
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p2, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    #if (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 18 between LCD and IRQ 3 alternate
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p2, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p2, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN18 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p18, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN18 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p18, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p2, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 19 Configuration **********************/
static void naSetupPort19(void)
{
#if (BSP_GPIO_MUX_ETHERNET_CAM == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 19 between Ethernet CAM and LCD
    #endif

    #if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 19 between Ethernet CAM and DMA 1 alternate
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p3, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    #if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 19 between LCD and DMA 1 alternate
    #endif
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p3, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p3, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN19 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p19, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN19 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p19, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p3, NA_GPIO_OUTPUT_STATE);
 
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 20 Configuration **********************/
static void naSetupPort20(void)
{
#if (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 20 between Serial C and LCD
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p4, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p4, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p4, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN20 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p20, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN20 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p20, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p4, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 21 Configuration **********************/
static void naSetupPort21(void)
{
#if (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 21 between Serial C and LCD
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p5, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p5, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p5, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN21 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p21, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN21 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p21, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 22 Configuration **********************/
static void naSetupPort22(void)
{
#if (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_SPI_APP) || \
    (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 22 between Serial C and LCD
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE) && \
        (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p6, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p6, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p6, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN22 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p22, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN22 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p22, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 23 Configuration **********************/
static void naSetupPort23(void)
{
#if (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_SPI_APP) || \
    (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_LCD_LINE_END == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 23 between Serial C and LCD Line End
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE) && \
        (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p7, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p7, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_LCD_LINE_END == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p7, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN23 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p23, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN23 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p23, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG3, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 24 Configuration **********************/
static void naSetupPort24(void)
{
#if (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 24 between Serial D and LCD
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p0, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p0, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p0, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN24 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p24, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN24 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p24, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p0, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 25 Configuration **********************/
static void naSetupPort25(void)
{
#if (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 25 between Serial D and LCD
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p1, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p1, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p1, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN25 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p25, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN25 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p25, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 26 Configuration **********************/
static void naSetupPort26(void)
{
#if (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_SPI_APP) || \
    (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 26 between Serial D and LCD
    #endif

    #if (BSP_GPIO_MUX_TIMER_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 26 between Serial D and Timer 3
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE) && \
        (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p2, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p2, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    #if (BSP_GPIO_MUX_TIMER_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 26 between LCD and Timer 3
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p2, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_TIMER_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p2, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN26 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p26, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN26 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p26, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p2, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 27 Configuration **********************/
static void naSetupPort27(void)
{
#if (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_SPI_APP) || \
    (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 27 between Serial D and LCD
    #endif

    #if (BSP_GPIO_MUX_TIMER_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 27 between Serial D and Timer 4
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE) && \
        (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p3, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p3, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_LCD != BSP_GPIO_MUX_INTERNAL_USE_ONLY)

    #if (BSP_GPIO_MUX_TIMER_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 27 between LCD and Timer 4
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p3, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_TIMER_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p3, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN27 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p27, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN27 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p27, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p3, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 28 Configuration **********************/
static void naSetupPort28(void)
{
#if (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    
    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_ALTERNATE_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_8BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 28 between IRQ 1 alternate and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p4, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_8BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p4, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_ALTERNATE_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p4, NA_GPIO_SELECT_MUX_FUNC2);

#else

    #if (BSP_GPIO_INITIAL_STATE_PIN28 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p28, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN28 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p28, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p4, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 29 Configuration **********************/
static void naSetupPort29(void)
{
#if (BSP_GPIO_MUX_TIMER_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_ALTERNATE_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_8BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 29 between Timer 5 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p5, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_8BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p5, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_ALTERNATE_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p5, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN29 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p29, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN29 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p29, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 30 Configuration **********************/
static void naSetupPort30(void)
{
#if (BSP_GPIO_MUX_TIMER_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_ALTERNATE_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_8BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 30 between Timer 6 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p6, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_8BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p6, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_ALTERNATE_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p6, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN30 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p30, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN30 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p30, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 31 Configuration **********************/
static void naSetupPort31(void)
{
#if (BSP_GPIO_MUX_TIMER_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_ALTERNATE_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_8BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 31 between Timer 7 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p0, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_8BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p7, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_4BIT_ALTERNATE_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p7, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN31 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p31, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN31 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG1, p31, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG4, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 32 Configuration **********************/
static void naSetupPort32(void)
{
#if (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 32 between IRQ 2 and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 32 between IRQ 2 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p0, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 32 between IEEE 1284 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p0, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p0, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN32 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p0, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN32 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p0, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p0, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 33 Configuration **********************/
static void naSetupPort33(void)
{
#if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 33 between IEEE 1284 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p1, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p1, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN33 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p1, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN33 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p1, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 34 Configuration **********************/
static void naSetupPort34(void)
{
#if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 34 between I2C and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 34 between I2C and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p2, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 34 between IEEE 1284 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p2, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p2, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN34 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p2, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN34 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p2, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p2, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 35 Configuration **********************/
static void naSetupPort35(void)
{
#if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 35 between I2C and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 35 between Timer 10 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p3, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 35 between IEEE 1284 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p3, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p3, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN35 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p3, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN35 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p3, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p3, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 36 Configuration **********************/
static void naSetupPort36(void)
{
#if (BSP_GPIO_MUX_PWM0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 36 between PWM 0 and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 36 between PWM 0 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p4, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 36 between IEEE 1284 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p4, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p4, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN36 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p4, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN36 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p4, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p4, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 37 Configuration **********************/
static void naSetupPort37(void)
{
#if (BSP_GPIO_MUX_PWM1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 37 between PWM 1 and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 37 between PWM 1 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p5, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 37 between IEEE 1284 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p5, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p5, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN37 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p5, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN37 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p5, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);


    #endif
#endif
}
/******************** GPIO 38 Configuration **********************/
static void naSetupPort38(void)
{
#if (BSP_GPIO_MUX_PWM2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 38 between PWM 2 and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 38 between IEEE 1284 and PWM 2
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p6, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 38 between IEEE 1284 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p6, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p6, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN38 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p6, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN38 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p6, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 39 Configuration **********************/
static void naSetupPort39(void)
{
#if (BSP_GPIO_MUX_PWM3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 39 between PWM 3 and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 39 between IEEE 1284 and PWM 3
    #endif
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p7, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
        (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)
    #error GPIO Mux conflict on GPIO port 39 between IEEE 1284 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p7, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) || \
    (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_DUAL_8BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p7, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN39 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p7, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN39 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p7, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG5, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 40 Configuration **********************/
static void naSetupPort40(void)
{
#if (BSP_GPIO_MUX_SERIAL_C != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    #if (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 40 between Serial C and IRQ 3
    #endif

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP)
    #error GPIO Mux conflict on GPIO port 40 between Serial C and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p0, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP)
    #error GPIO Mux conflict on GPIO port 40 between IRQ 3 and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p0, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p0, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN40 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p8, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN40 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p8, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p0, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 41 Configuration **********************/
static void naSetupPort41(void)
{
#if (BSP_GPIO_MUX_SERIAL_C != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    #if (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP) 
    #error GPIO Mux conflict on GPIO port 41 between Serial C and LCD
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p1, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_LCD == BSP_GPIO_MUX_LCD_18BIT_APP)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p1, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN41 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p9, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN41 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p9, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 42 Configuration **********************/
static void naSetupPort42(void)
{
#if (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 42 between Serial C and USB phy data
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p2, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p2, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
      (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p2, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN42 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p10, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN42 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p10, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p2, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 43 Configuration **********************/
static void naSetupPort43(void)
{
#if (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_C == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 43 between Serial C and 1284
    #endif

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 43 between Serial C and USB phy data
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p3, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p3, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 43 between 1284 and USB phy data
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p3, NA_GPIO_SELECT_MUX_FUNC1 | NA_GPIO_SELECT_MUX_OUTPUT);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p3, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN43 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p11, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN43 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p11, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p3, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 44 Configuration **********************/
static void naSetupPort44(void)
{
#if (BSP_GPIO_MUX_SERIAL_D != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 44 between Serial D and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 44 between Serial D and USB phy tx output enable
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p4, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 44 between IEEE 1284 and USB phy tx output enable
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p4, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p4, NA_GPIO_SELECT_MUX_FUNC2_AND_INVERT);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN44 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p12, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN44 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p12, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p4, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 45 Configuration **********************/
static void naSetupPort45(void)
{
#if (BSP_GPIO_MUX_SERIAL_D != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 45 between Serial D and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 45 between Serial D and USB phy rx data
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p5, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 45 between IEEE 1284 and USB phy rx data
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p5, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p5, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN45 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p13, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN45 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p13, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 46 Configuration **********************/
static void naSetupPort46(void)
{
#if (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 46 between Serial D and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 46 between Serial D and USB phy rx + data (unidirectional phy only)
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p6, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p6, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 46 between IEEE 1284 and USB phy rx + data (unidirectional phy only)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p6, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p6, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN46 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p14, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN46 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p14, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 47 Configuration **********************/
static void naSetupPort47(void)
{
#if (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || \
    (BSP_GPIO_MUX_SERIAL_D == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 47 between Serial D and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 47 between Serial D and USB phy rx - data (unidirectional phy only)
    #endif

    #if (BSP_GPIO_INVERT_SERIAL_CONTROL == TRUE)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p7, NA_GPIO_SELECT_MUX_FUNC0_AND_INVERT);
    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p7, NA_GPIO_SELECT_MUX_FUNC0);
    #endif

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 47 between IEEE 1284 and USB phy rx - data (unidirectional phy only)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p7, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) 

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p7, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN47 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p15, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN47 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p15, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG6, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 48 Configuration **********************/
static void naSetupPort48(void)
{
#if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 48 between USB phy suspend and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 48 between USB phy suspend and DMA 1
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p0, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 48 between IEEE 1284 and DMA 1
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p0, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p0, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN48 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p16, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN48 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p16, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p0, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 49 Configuration **********************/
static void naSetupPort49(void)
{
#if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_BIDIRECTIONAL_PHY)
    
    #if (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 49 between USB phy speed and IEEE 1284
    #endif

    #if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 49 between USB phy speed and DMA 1
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p1, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_1284 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #error GPIO Mux conflict on GPIO port 49 between IEEE 1284 and DMA 1
    #endif

/* This pin is activiated after all p1284 registers set in the Parallel Driver */
/*    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p1, NA_GPIO_SELECT_MUX_FUNC1); */
/*    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p1, NA_GPIO_SELECT_MUX_FUNC1); */
      narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p17, 0);
      narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p1, NA_GPIO_SELECT_MUX_GPIO_OUTPUT); 

#elif (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p1, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN49 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p17, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN49 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p17, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 50 Configuration **********************/
static void naSetupPort50(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 50 between MII and USB phy data - (duplicate)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p2, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p2, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN50 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p18, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN50 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p18, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p2, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 51 Configuration **********************/
static void naSetupPort51(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    
    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 51 between MII and USB phy data + (duplicate)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p3, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p3, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN51 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p19, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN51 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p19, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p3, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 52 Configuration **********************/
static void naSetupPort52(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_RXER == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 52 between PHY RXER and USB phy tx output enable (duplicate)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p4, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p4, NA_GPIO_SELECT_MUX_FUNC1_AND_INVERT);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN52 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p20, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN52 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p20, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p4, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 53 Configuration **********************/
static void naSetupPort53(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 53 between MII and USB phy rx data (duplicated)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p5, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p5, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN53 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p21, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN53 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p21, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 54 Configuration **********************/
static void naSetupPort54(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 54 between MII and USB phy suspend (duplicated)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p6, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p6, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN54 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p22, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN54 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p22, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 55 Configuration **********************/
static void naSetupPort55(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 55 between MII and USB phy speed (duplicated)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p7, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) || \
        (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p7, NA_GPIO_SELECT_MUX_FUNC1);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN55 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p23, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN55 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p23, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG7, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 56 Configuration **********************/
static void naSetupPort56(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY)
    #error GPIO Mux conflict on GPIO port 56 between MII and USB phy rx data + (unidirectional phy only, duplicated)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p0, NA_GPIO_SELECT_MUX_FUNC0);

#elif  (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY) 
           narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p24, 0);
           narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p0, NA_GPIO_OUTPUT_STATE);

#elif  (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY)
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p0, NA_GPIO_SELECT_MUX_FUNC1); 
 
#else
    #if (BSP_GPIO_INITIAL_STATE_PIN56 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p24, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN56 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p24, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p0, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 57 Configuration **********************/
static void naSetupPort57(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    

    #if (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY) 
    #error GPIO Mux conflict on GPIO port 57 between MII and USB phy rx data - (unidirectional phy only, duplicated)
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p1, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_BIDIRECTIONAL_PHY)
        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p25, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p1, NA_GPIO_OUTPUT_STATE);
 
#elif  (BSP_GPIO_MUX_USB_DEVICE ==  BSP_GPIO_MUX_USB_ALT_UNIDIRECTIONAL_PHY)   
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p1, NA_GPIO_SELECT_MUX_FUNC1); 

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN57 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p25, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN57 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p25, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 58 Configuration **********************/
static void naSetupPort58(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_TXER == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p2, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN58 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p26, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN58 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p26, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p2, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 59 Configuration **********************/
static void naSetupPort59(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p3, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN59 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p27, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN59 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p27, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p3, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 60 Configuration **********************/
static void naSetupPort60(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p4, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN60 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p28, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN60 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p28, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p4, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 61 Configuration **********************/
static void naSetupPort61(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p5, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN61 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p29, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN61 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p29, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);


    #endif
#endif
}
/******************** GPIO 62 Configuration **********************/
static void naSetupPort62(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p6, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN62 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p30, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN62 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p30, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 63 Configuration **********************/
static void naSetupPort63(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p7, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN63 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p31, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN63 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG2, p31, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG8, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 64 Configuration **********************/
static void naSetupPort64(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p0, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN64 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p0, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN64 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p0, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p0, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 65 Configuration **********************/
static void naSetupPort65(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERRUPT != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p1, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN65 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p1, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p1, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN65 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p1, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p1, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p1, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 66 Configuration **********************/
static void naSetupPort66(void)
{
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p2, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN66 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p2, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p2, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN66 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p2, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p2, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p2, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 67 Configuration **********************/
static void naSetupPort67(void)
{
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING != BSP_GPIO_MUX_INTERNAL_USE_ONLY) && \
    (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING != BSP_GPIO_MUX_USE_ADDR_BIT_22)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p3, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN67 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p3, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p3, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN67 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p3, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p3, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p3, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 68 Configuration **********************/
static void naSetupPort68(void)
{
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_24_AND_LOWER) || \
    (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_25_AND_LOWER) || \
    (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_26_AND_LOWER) || \
    (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)
    
    #if (BSP_GPIO_MUX_DRAM_CLOCK_ENABLE_CS0 != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 68 between Memory Address 24 and Memory clock enable
    #endif

    #if (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 68 between Memory Address 24 and IRQ 0 alternate
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p4, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_DRAM_CLOCK_ENABLE_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p4, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p4, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN68 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p4, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p4, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN68 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p4, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p4, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p4, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 69 Configuration **********************/
static void naSetupPort69(void)
{
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_25_AND_LOWER) || \
    (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_26_AND_LOWER) || \
    (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)
    
    #if (BSP_GPIO_MUX_DRAM_CLOCK_ENABLE_CS1 != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 69 between Memory Address 25 and Memory clock enable
    #endif

    #if (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 69 between Memory Address 25 and IRQ 1 alternate
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p5, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_DRAM_CLOCK_ENABLE_CS1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p5, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p5, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN69 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p5, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p5, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN69 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p5, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p5, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p5, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/******************** GPIO 70 Configuration **********************/
static void naSetupPort70(void)
{
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_26_AND_LOWER) || \
    (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)
    
    #if (BSP_GPIO_MUX_DRAM_CLOCK_ENABLE_CS2 != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 70 between Memory Address 26 and Memory clock enable
    #endif

    #if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 70 between Memory Address 26 and i2c duplicate
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p6, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_DRAM_CLOCK_ENABLE_CS2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p6, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p6, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN70 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p6, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p6, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN70 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p6, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p6, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p6, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 71 Configuration **********************/
static void naSetupPort71(void)
{
#if (BSP_GPIO_MUX_MEM_ADDR == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)
    
    #if (BSP_GPIO_MUX_DRAM_CLOCK_ENABLE_CS3 != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    #error GPIO Mux conflict on GPIO port 71 between Memory Address 27 and Memory clock enable
    #endif

    #if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #error GPIO Mux conflict on GPIO port 71 between Memory Address 27 and i2c duplicate
    #endif

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p7, NA_GPIO_SELECT_MUX_FUNC0);

#elif (BSP_GPIO_MUX_DRAM_CLOCK_ENABLE_CS3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p7, NA_GPIO_SELECT_MUX_FUNC1);

#elif (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH)

    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p7, NA_GPIO_SELECT_MUX_FUNC2);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN71 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p7, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p7, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN71 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p7, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p7, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG9, p7, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}
/******************** GPIO 72 Configuration **********************/
static void naSetupPort72(void)
{
#if (BSP_GPIO_MUX_SLOW_MEMORY_PERIPHERAL != BSP_GPIO_MUX_INTERNAL_USE_ONLY)
    
    narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG10, p0, NA_GPIO_SELECT_MUX_FUNC0);

#else
    #if (BSP_GPIO_INITIAL_STATE_PIN72 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p8, 0);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG10, p0, NA_GPIO_OUTPUT_STATE);

    #elif (BSP_GPIO_INITIAL_STATE_PIN72 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        narm_write_reg(NA_GPIO_CSBIT, NA_GPIO_CONTRLREG3, p8, 1);
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG10, p0, NA_GPIO_OUTPUT_STATE);

    #else
        narm_write_reg(NA_GPIO_CONFIG, NA_GPIO_CONFIGREG10, p0, NA_GPIO_SELECT_MUX_GPIO_INPUT);

    #endif
#endif
}

/*
 *
 *  Function: int NAsetGPIOpin(unsigned int channel, unsigned int state)
 *
 *  Description:
 *
 *      This function sets the output level of a output GPIO pin.  The function 
 *      requires the pin to be configured as a GPIO output.
 *
 *  Parameters:
 *
 *      channel         The GPIO channel.  Should be 0 through 72.  The pin
 *                      must also be setup as a GPIO output.
 *      state           Logic level to set the pit.  Non-zero value sets the
 *                      pin to a logic one, otherwise the pin is set to zero.
 *
 *  Return Values:
 *
 *      0                       SUCCESS
 *      NA_GPIO_INVALID_PIN     Invalid channel
 *      NA_GPIO_INVALID_SETUP   Valid channel, but not setup as a GPIO output
 *
 */
int NAsetGPIOpin(unsigned int channel, unsigned int state)
{
    unsigned int index, field, regValue;
    int irq_state;

    if (channel > NA_GPIO_LARGEST_VALID_GPIO_PIN)
    {
        return NA_GPIO_INVALID_PIN;
    }

    index    = channel >> 3;
    field    = channel % 8;
    regValue = (*gpioConfigReg[index] >> (field << 2)) & NA_GPIO_SELECT_MUX_GPIO_OUTPUT;
    if (regValue != NA_GPIO_SELECT_MUX_GPIO_OUTPUT)
    {
        return NA_GPIO_INVALID_SETUP;
    }

    index    = channel >> 5;
    field    = channel % 32;
    if (state)
    {
        /*  disable interrupts/thread switches around this read-modify-write  */
        irq_state = NASetIRQ (IRQ_DISABLED);
        *gpioControlReg[index] |= (1 << field);
        NASetIRQ (irq_state);
    }
    else
    {
        /*  disable interrupts/thread switches around this read-modify-write  */
        irq_state = NASetIRQ (IRQ_DISABLED);
        *gpioControlReg[index] &= ~(1 << field);
        NASetIRQ (irq_state);
    }

    return 0;
}




/*
 *
 *  Function: int NAgetGPIOpin(unsigned int channel, unsigned int *state)
 *
 *  Description:
 *
 *      This function gets the present level of a GPIO input pin.  The function 
 *      requires the pin to be configured as a GPIO input.
 *
 *  Parameters:
 *
 *      channel         The GPIO channel.  Should be 0 through 49.  The pin
 *                      must also be setup as a GPIO input.
 *      state           pointer to an integer that can be loaded with the 
 *                      logic level of the pin.  Non-zero value represents a 
 *                      logic one, otherwise the pin is zero.
 *
 *  Return Values:
 *
 *      0                       SUCCESS
 *      NA_GPIO_INVALID_PIN     Invalid channel
 *      NA_GPIO_INVALID_SETUP   Valid channel, but not setup as a GPIO output
 *      NA_GPIO_INVALID_ARG     Null state pointer
 *
 */
int NAgetGPIOpin(unsigned int channel, unsigned int *state)
{
    unsigned int index, field, regValue;

    if (state == NULL)
    {
        return NA_GPIO_INVALID_ARG;
    }

    if (channel > NA_GPIO_LARGEST_VALID_GPIO_PIN)
    {
        return NA_GPIO_INVALID_PIN;
    }

    index    = channel >> 3;
    field    = channel % 8;
    regValue = (*gpioConfigReg[index] >> (field << 2)) & NA_GPIO_SELECT_MUX_GPIO_OUTPUT;
    if (regValue != NA_GPIO_SELECT_MUX_GPIO_INPUT)
    {
        return NA_GPIO_INVALID_SETUP;
    }

    index    = channel >> 5;
    field    = channel % 32;
    *state   = *gpioStatusReg[index] & (1 << field);

    return 0;
}

/*ad*
 * Configures a GPIO multiplexer pin to a GPIO or multiplexer function.
 *
 * @param channel               The GPIO multiplexer channel to be configured
 * @param state                 The configuration for the GPIO multiplexer channel
 * @param initialValue          The initial value for the GPIO pin
 *
 * @return SUCCESS              The GPIO MUX pin has configured successfully.
 * @return NA_GPIO_INVALID_PIN  Invalid channel argument.
 * @return NA_GPIO_INVALID_ARG  Invalid state argument.
 *
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Pin_States 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Return_Values 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAgetGPIOpin
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAsetGPIOpin
 *
 * @external
 * @include "gpiomux_def.h"
 * @category BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360
 * @since 6.2
*ad*/

int NAconfigureGPIOpin(unsigned int channel, unsigned int state, unsigned int initialValue)
{
    unsigned int index, field, value;
    int irq_state;
    int retcode = SUCCESS;

    if (channel > NA_GPIO_LARGEST_VALID_GPIO_PIN)
        retcode = NA_GPIO_INVALID_PIN;
        
    switch (state)
    {
    case NA_GPIO_OUTPUT_STATE:
        index    = channel >> 5;
        field    = channel % 32;

        irq_state = NASetIRQ (IRQ_DISABLED);
        if (initialValue)
            *gpioControlReg[index] |= (1 << field);
        else
            *gpioControlReg[index] &= ~(1 << field);
        NASetIRQ (irq_state);
        break;

    case NA_GPIO_INPUT_STATE:
    case NA_GPIO_FUNC0_STATE:
    case NA_GPIO_FUNC1_STATE:
    case NA_GPIO_FUNC2_STATE:
        break;
    default:
        retcode = NA_GPIO_INVALID_ARG;
    }

    if (retcode == SUCCESS)
    {    
        index    = channel >> 3;
        field    = channel % 8;
        
        /*  disable interrupts/thread switches around this read-modify-write  */
        irq_state = NASetIRQ (IRQ_DISABLED);
        value = *gpioConfigReg[index] & ~(0xf << (field << 2));
        *gpioConfigReg[index] = value | (state << (field << 2));
        NASetIRQ (irq_state);
    }
    
    return retcode;    
}

/*
 * To get the a GPIO multiplexer configuration value of the specified pin.
 *
 * @param channel         The GPIO multiplexer pin number
 * @param state           To store the GPIO multiplexer pin configuration value 
 *
 * @return SUCCESS              The GPIO MUX pin has configured successfully.
 * @return NA_GPIO_INVALID_PIN  Invalid channel argument.
 *
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Pin_States 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Return_Values 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAgetGPIOpin
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAsetGPIOpin
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAconfigureGPIOpin
 *
 * @external
 * @include "gpiomux_def.h"
 * @category BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360
 * @since 7.1
 */

int NAgetGPIOpinConfig(unsigned int channel, unsigned int *state)
{
    unsigned int index, field, value;
    int retcode = SUCCESS;

    if (channel > NA_GPIO_LARGEST_VALID_GPIO_PIN)
        retcode = NA_GPIO_INVALID_PIN;
    else        
    {    
        index    = channel >> 3;
        field    = channel % 8;
        
        value  = *gpioConfigReg[index] & (0xf << (field << 2));
        *state = value >> (field << 2);  
    }
    
    return retcode;    
}


/*ad*
 * Inverts a GPIO multiplexer pin.
 *
 * @param channel    The GPIO multiplexer channel to be configured
 * @param enable     Inverts if this value is TRUE; otherwise, sets to normal
 *
 * @return SUCCESS              GPIO MUX pin has configured successfully.
 * @return NA_GPIO_INVALID_PIN  Invalid channel argument.
 *
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Pin_States 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Return_Values 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360:NAconfigureGPIOpin
 *
 * @external
 * @include "gpiomux_def.h"
 * @category BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360
 * @since 6.2
*ad*/
int NAInvertGPIOpin(unsigned int channel, BOOLEAN enable)
{
    unsigned int index, field;
    int retcode = SUCCESS;

    if (channel > NA_GPIO_LARGEST_VALID_GPIO_PIN)
        retcode = NA_GPIO_INVALID_PIN;      
    else
    {    
        index    = channel >> 3;
        field    = channel % 8;
        
        if (enable)
            *gpioConfigReg[index] |= (4 << (field << 2));
        else
            *gpioConfigReg[index] &= ~(4 << (field << 2));
    }
    
    return retcode;    
}

#ifdef BSP_GPIO_BOOT_MUX_PIN
static void NAToggleBootMux(unsigned int channel)
{
    if (NAconfigureGPIOpin(channel, NA_GPIO_OUTPUT_STATE, 0) != SUCCESS)
        return;      
    NAsetGPIOpin(channel, 1);
    NAsetGPIOpin(channel, 0);     
}
#endif

/*
 *
 *  Function: void customizeSetupGPIO(int chipType, int softRestart)
 *
 *  Description:
 *
 *      This function sets up all GPIO pins.  This version of the function 
 *      sets up the ports to support a NET+OS development board.  The port 
 *      is disabled (left configured for GPIO input) if the port is not used.
 *
 *  Notes:
 *
 *      THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *      is for NET+OS development boards only.
 *
 *      This function will be called by ncc_init() before RAM has been
 *      configured.  The program stack is thus particularly small, so 
 *      limit the amount of local scope.   Also do not use global or static 
 *      variables.
 *
 *  Parameters:
 *
 *      chipType        set to indicate the base chip type of the processor.
 *                      The subrevision information has been masked off.
 *      softRestart     set if a software restart has taken place.
 *
 *
 *  Return Values:
 *
 *      none
 *
 */
void customizeSetupGPIO(int chipType, int softRestart)
{
    NA_UNUSED_PARAM(chipType);
    NA_UNUSED_PARAM(softRestart);

    #ifdef BSP_GPIO_BOOT_MUX_PIN
    NAToggleBootMux(BSP_GPIO_BOOT_MUX_PIN);
    #endif

    naSetupPort0();
    naSetupPort1();
    naSetupPort2();
    naSetupPort3();
    naSetupPort4();
    naSetupPort5();
    naSetupPort6();
    naSetupPort7();
    naSetupPort8();
    naSetupPort9();
    naSetupPort10();
    naSetupPort11();
    naSetupPort12();
    naSetupPort13();
    naSetupPort14();
    naSetupPort15();
    naSetupPort16();
    naSetupPort17();
    naSetupPort18();
    naSetupPort19();
    naSetupPort20();
    naSetupPort21();
    naSetupPort22();
    naSetupPort23();
    naSetupPort24();
    naSetupPort25();
    naSetupPort26();
    naSetupPort27();
    naSetupPort28();
    naSetupPort29();
    naSetupPort30();
    naSetupPort31();
    naSetupPort32();
    naSetupPort33();
    naSetupPort35();
    naSetupPort34();
    naSetupPort36();
    naSetupPort37();
    naSetupPort38();
    naSetupPort39();
    naSetupPort40();
    naSetupPort41();
    naSetupPort42();
    naSetupPort43();
    naSetupPort45();
    naSetupPort44();
    naSetupPort46();
    naSetupPort47();
    naSetupPort48();
    naSetupPort49();
    naSetupPort50();
    naSetupPort51();
    naSetupPort52();
    naSetupPort53();
    naSetupPort55();
    naSetupPort54();
    naSetupPort56();
    naSetupPort57();
    naSetupPort58();
    naSetupPort59();
    naSetupPort60();
    naSetupPort61();
    naSetupPort62();
    naSetupPort63();
    naSetupPort64();
    naSetupPort65();
    naSetupPort66();
    naSetupPort67();
    naSetupPort68();
    naSetupPort69();
    naSetupPort70();
    naSetupPort71();
    naSetupPort72();
}



