/*
 *
 *     Module Name: bsp.h
 *         Version: 2.00
 *   Original Date: 08/18/03
 *          Author: Contact NetSilicon technical support if you have questions on this file.
 *        Language: Ansi C
 * Compile Options:
 * Compile defines:
 *       Libraries:
 *    Link Options:
 *
 *    Entry Points:
 *
 *  Copyright (c) 1996-2004 Digi International Inc., All Rights Reserved
 *
 *  This software contains proprietary and confidential information of Digi
 *  International Inc.  By accepting transfer of this copy, Recipient agrees
 *  to retain this software in confidence, to prevent disclosure to others,
 *  and to make no use of this software other than that for which it was
 *  delivered.  This is an unpublished copyrighted work of Digi International
 *  Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
 *  prohibited.
 *
 *  Restricted Rights Legend
 *
 *  Use, duplication, or disclosure by the Government is subject to
 *  restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
 *  Technical Data and Computer Software clause at DFARS 252.227-7031 or
 *  subparagraphs (c)(1) and (2) of the Commercial Computer Software -
 *  Restricted Rights at 48 CFR 52.227-19, as applicable.
 *
 *  Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
 *
 * bsp.h This file defines BSP wide constants, data structures and prototypes.
 * Please refer to the BSP porting guide for a description of this file.
 *
 */
/*
 * Edit History
 *
 * Date      INitials       Change Description
 * 02/13/04  JZW            removed definition of BSP_PRTENG_THREAD_PRIORITY
 *                          Changed BSP_SIMPLE_SERIAL_PORT's definition
 *                          from ....PORTB to ....PORTA
 * 07/01/05	 JZW            vantive 16441 - Remove restrictive code that requires PORT A to
 *                          be enabled if PORTB is enabled.
 */

#ifndef BSP_H
#define BSP_H

#include <Npttypes.h>
#include <bsp_api.h>
#include <tx_api.h>
#include <init_settings.h>
#include <spi_master_api.h>
#include <fs.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*ad*
 * Set this constant to TRUE to enable IP packet forwarding.  When this feature is
 * enabled, the TCP/IP stack will retransmit packets received on one network interface
 * out another if the packets are addressed to a device that is reachable
 * from the other interface.  You must enable this feature to allow the TCP/IP
 * stack to route packets from one network to another.  Disable this feature if it 
 * is not needed to improve security.
 *
 * @since 7.0
 * @external
 * @include  <bsp.h>
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_ENABLE_IP_FORWARDING        FALSE




/*ad*
 * Set this constant to TRUE to include the GDB debug.
 * This enables software debugging. Once this is enabled,
 * Bootloader starts TFTP to load image.bin for debugging 
 * and bsp starts GDB by waiting for gdb debugger.
 * 
 *
 * @since 7.0
 * @external
 * @include  <bsp.h>
 * @category BSP:BSP_Configuration
 *
 * @note gdb server library must be linked in your application.
 * @see @link gdbserver
 * @see @link NA_GDB_TFTP_SERVER
 * @see @link NA_GDB_COMPORT
 *
*ad*/
#define BSP_GDB_SERVER_ENABLE          FALSE

/*ad*
 * Bootloader default TFTP server ip address. 
 * This is used to specify default TFTP server ip address
 * to load debug image.
 * This BSP_GDB_SERVER_ENABLE must be enabled to include GDB.
 *
 * @since 7.0
 * @include     <bsp.h>
 * @external
 * @category BSP:BSP_Configuration
 * @see @link NA_GDB_COMPORT
 * @see @link BSP_GDB_SERVER_ENABLE
 *
 *ad*/
#define NA_GDB_TFTP_SERVER     "10.52.32.193" 

/*ad*
 * GDB communication port. 
 *
 * This is used for gdb debug. This serial port is used to communicate
 * between the board and the gdb debugger.
 * See BSP_GDB_SERVER_ENABLE to enable GDB debug.
 *
 * @since 7.0
 * @include     <bsp.h>
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_GDB_SERVER_ENABLE
 * @see @link NA_GDB_TFTP_SERVER
 *
 *ad*/
#define NA_GDB_COMPORT         "/com/0"

 

/*ad*
 * BSP_TICKS_PER_SECOND determines the system clock rate.  
 *
 * 
 * Use the constant NABspTicksPerSecond to access this value
 * in application code.
 *
 * @since 1.0
 * @external
 * @see @link NABspTicksPerSecond
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_TICKS_PER_SECOND        100                /* 10ms tick */

/*ad*
 * This constant defines the chip type and rev used in the BSP,
 * the values for this variable are defined in the header
 * file mcreg_def.h.
 *
 * @since 6.1
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_CHIP_TYPE NCC_NS9360_A


/*ad*
 * Set BSP_SERIAL_PORT_X  to one of these values to
 * indicate what type of serial driver should be built for the port.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @name SerialPortConfigurations   "Serial Port Configurations"
 * @see @link BSP_SERIAL_PORT_1
 * @see @link BSP_SERIAL_PORT_2
 * @see @link BSP_SERIAL_PORT_3
 * @see @link BSP_SERIAL_PORT_4
 *
 * @param BSP_SERIAL_NO_DRIVER      Serial port will not be used
 * @param BSP_SERIAL_UART_DRIVER    Serial port will be used in standard UART mode
 * @param BSP_SERIAL_SPI_DRIVER     Serial port will be used in SPI Master mode
 * @param BSP_SERIAL_SPI_SLAVE_DRIVER    Serial port will be used in SPI Slave mode
 *
 * @note The SPI Slave drive is ONLY supported in NET+OS 6.2 for NS9750 and NS9360 chips,
 *       
*ad*/
#define BSP_SERIAL_NO_DRIVER        0
#define BSP_SERIAL_UART_DRIVER      1
#define BSP_SERIAL_SPI_DRIVER       2
#define BSP_SERIAL_SPI_SLAVE_DRIVER      4

/*ad*
 * This constant controls which serial driver is built in
 * the BSP for serial port 1.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link SerialPortConfigurations
*ad*/
#define BSP_SERIAL_PORT_1       BSP_SERIAL_UART_DRIVER

/*ad*
 * This constant controls which serial driver is built in
 * the BSP for serial port 2.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link SerialPortConfigurations
*ad*/
#define BSP_SERIAL_PORT_2       BSP_SERIAL_UART_DRIVER
// #define BSP_SERIAL_PORT_2       BSP_SERIAL_SPI_DRIVER

/*ad*
 * This constant controls which serial driver is built in
 * the BSP for serial port 3.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link SerialPortConfigurations
*ad*/
#define BSP_SERIAL_PORT_3       BSP_SERIAL_UART_DRIVER

/*ad*
 * This constant controls which serial driver is built in
 * the BSP for serial port 4.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link SerialPortConfigurations
*ad*/
#define BSP_SERIAL_PORT_4       BSP_SERIAL_UART_DRIVER

/*ad*
 * This constant determines which port is used to output Simple Serial data.
 * This data is used early in initialization (when the C library and OS are
 * not yet available).  The baud rate used is based on your NVRAM or appconf.h
 * setting for APP_BSP_BAUD_RATE.
 *
 * @name Simple_Serial_Port_Defines "Definitions for serial ports used fo simple serial communication."
 *
 * @since 6.1
 * @external
 * @category BSP:BSP_Configuration
 * @param BSP_SIMPLE_SERIAL_ON_PORTA    route simple serial to serial port A
 * @param BSP_SIMPLE_SERIAL_ON_PORTB    route simple serial to serial port B
 * @param BSP_SIMPLE_SERIAL_ON_PORTC    route simple serial to serial port C
 * @param BSP_SIMPLE_SERIAL_ON_PORTD    route simple serial to serial port D
*ad*/
#define BSP_SIMPLE_SERIAL_ON_PORTA      1
#define BSP_SIMPLE_SERIAL_ON_PORTB      2
#define BSP_SIMPLE_SERIAL_ON_PORTC      3
#define BSP_SIMPLE_SERIAL_ON_PORTD      4



/*ad*
 * This constant controls which serial port the simple serial driver
 * is connected too.
 *
 * @since 6.1
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_SIMPLE_SERIAL_PORT          BSP_SIMPLE_SERIAL_ON_PORTA

/*ad*
 * BSP_BAUD_RATE determines the baud rate that will be used for the configuration
 * dialog if APP_USE_NVRAM is 0, or if NVRAM is uninitialized.
 *
 * @external
 * @category general_purpose 
 * @since 7.1
*ad*/
#define BSP_BAUD_RATE	       38400 

/*ad*
 * BSP_STDIO_PORT determines which serial port stdin, stdout, and stderr should be redirected to.
 * It should be set to the device name of the port or NULL.
 *
 * @external
 * @category general_purpose 
 * @since 7.1 
*ad*/
#define BSP_STDIO_PORT              "/com/0"

/*ad*
 * BSP_DIALOG_PORT determines whether or not the initialization code will prompt for configuration
 * parameters.  It should be set to a string that contains the serial port to prompt with.  If it
 * is not set, or if it contains an invalid port, then the initialization code will not prompt for
 * configuration parameters.
 *
 * @external
 * @category general_purpose 
 * @since 7.1 
*ad*/
#define BSP_DIALOG_PORT             "/com/0"

/*ad*
 * Set this constant to TRUE to build the IEEE-1284 parallel port driver.
 *
 * @since 1.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_INCLUDE_PARALLEL_DRIVER     FALSE

/*ad*
 * Set this constant to TRUE to include the NUL driver.
 *
 * @since 1.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_INCLUDE_NUL_DRIVER          FALSE

/*ad*
 * Set this constant to TRUE to include the loopback driver.
 *
 * @since 1.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_INCLUDE_LOOPBACK_DRIVER     FALSE

/*ad*
 * Set this constant to TRUE to include the UPD debug driver.
 *
 * @since 1.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_INCLUDE_UDP_DEBUG_DRIVER    FALSE

/*ad*
 * Set this constant to TRUE to add the PPP devices into the TCP/IP
 * stack's table of devices.  Set it to FALSE if you do not intend
 * to use PPP.
 *
 * @category BSP:BSP_Configuration
 * @since 6.0
 *
 * @external
 * @see @link PPP
*ad*/
#define BSP_WANT_PPP                        FALSE

/*ad*
 * Set this constant to TRUE to add the ethernet device into the TCP/IP
 * stack's table of devices.  Set it to FALSE if you do not intend
 * to use ethernet.
 *
 * @category general_purpose
 * @since 6.0
 *
 * @external
*ad*/
#define BSP_WANT_ETHERNET        TRUE 


/*ad*
 * Set this constant to TRUE to add the 802.11 wireless device into the TCP/IP
 * stack's table of devices.  Set it to FALSE if you do not intend
 * to use wireless. 
 *
 * @category wireless
 * @since 6.3
 * @note For cardbus cards, make sure SW10-5 is on, for PCI cards, SW10-5 is off.
 *
 * @note Only one wireless interface is supported.
 *
 * @external
*ad*/
#define BSP_WANT_WIRELESS    FALSE 


/*ad*
 * Set this constant to TRUE to add the start the Wireless Protect Access (WPA)
 * application from the BSP.  Set it to FALSE if you do not intend to use WPA, 
 * or will start it later.  BSP_WANT_WIRELESS must be TRUE to use WPA.
 *
 * @category wireless
 * @since 6.3
 *
 * @external
*ad*/
#define BSP_WANT_WPA	       FALSE 



/*ad*
 * Set this constant to TRUE to include the LCD driver.
 *
 * @since 6.1
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_INCLUDE_LCD_DRIVER		     TRUE


/*ad*
 * Set BSP_USB_DEVICE_VERSION to one of these values to indicate which 
 * USB device driver will be used . 
 *
 * @since 7.1
 * @external
 * @category BSP:BSP_Configuration
 * @name UsbDeviceDriverVersion        "USB Device Driver Version"
 *
 * @param BSP_USB_DRIVER_VERSION_NA    No Usb Device Drive
 * @param BSP_USB_DRIVER_VERSION_1     Include USB device driver under 
 *                                     @link BSP:Device_Drivers:USB_DEVICE
 *
*ad*/
#define BSP_USB_DRIVER_VERSION_NA       0
#define BSP_USB_DRIVER_VERSION_1        1

/*ad*
 * This constant determines the USB device driver API used in the system.
 * Set this constant to one of the values in @link UsbDeviceDriverVersion
 *
 * @since 7.1
 * @external
 * @category BSP:BSP_Configuration
 *
*ad*/
#define BSP_USB_DEVICE_VERSION          BSP_USB_DRIVER_VERSION_1

/*ad*
 *
 * Set to TRUE to enable the USB mouse class driver.
 * 
 * @since 7.2
 * @external
 * @category BSP:BSP_Configuration
 *
*ad*/
#define BSP_ENABLE_USB_MOUSE FALSE

/*ad*
 *
 * Set to TRUE to enable the USB keyboard class driver.
 * 
 * @since 7.2
 * @external
 * @category BSP:BSP_Configuration
 *
*ad*/
#define BSP_ENABLE_USB_KEYBOARD FALSE

/*ad*
 *
 * Set to TRUE to enable the USB mass storage class driver, this includes
 * USB memory sticks.
 * 
 * @since 7.2
 * @external
 * @category BSP:BSP_Configuration
 *
*ad*/
#define BSP_ENABLE_USB_MASS_STORAGE FALSE

/*ad*
 *
 * Set to TRUE to enable the USB hub class driver, this is also
 * required for on-board USB hubs.
 * 
 * @since 7.2
 * @external
 * @category BSP:BSP_Configuration
 *
*ad*/
#define BSP_ENABLE_USB_HUB FALSE

/*ad*
 *
 * Set this constant to TRUE to enabale the USB host on ARM9
 * platforms. If your platform has an on-board hub, you must also
 * enable BSP_ENABLE_USB_HUB.
 *
 * @since 7.2
 * @external
 * @category BSP:BSP_Configuration
 *
*ad*/
#define BSP_ENABLE_USB_HOST  FALSE 

/*ad*
 * Turn this define on if you want to enable the Connect WiWave 802.11 b/g
 * USB Wireless networking PCI Express (PCIe) Mini Card.
 * By enabling the Connect WiWave, BSP_WANT_WIRELESS is also turned on.
 * 
 * @category wireless
 * @since 7.2
 * 
 * @note Only one wireless interface is supported, if the Connect WiWave is enabled
 * that the memory-mapped FPGA wireless card is not use.
 *
 * @external
*ad*/
#define BSP_ENABLE_WIWAVE    FALSE 




/*ad*
 * Sets this constant to TRUE to include the I2C_EXPANDER APIs in
 * the functional name based GPIO API functions.
 *
 * @since 6.3
 * @external
 * @category BSP:BSP_Configuration
*ad*/

#define BSP_INCLUDE_I2C_EXPANDER        TRUE




/*ad*
 * These manifest constants determine the ThreadX priority levels for
 * various server processes created by the API libraries.  ThreadX
 * priority levels range from 0 to 31 with 0 being the highest.  Threads
 * with greater priority can preempt threads
 * with lower priority.
 *
 * @since 1.0
 * @external
 * @category BSP:BSP_Configuration
 * @name ThreadPriorityLevels           "Thread Priority Levels"
 *
 * @param BSP_HIGH_PRIORITY             priority level for threads that need to be able to preempt
 *                                      most other threads
 * @param BSP_MEDIUM_PRIORITY           normal priority level for threads
 * @param BSP_LOW_PRIORITY              priority level for threads that should run at a low
 *                                      priority level (and be preemptable by most other threads)
 * @param BSP_DEFAULT_PRIORITY          default priority level for threads
 * @param BSP_FTP_SERVER_PRIORITY       priority level for FTP server thread
 * @param BSP_SNMP_AGENT_PRIORITY       priority level for SNMP agent thread
 * @param BSP_TELNET_SERVER_PRIORITY    priority level for Telnet server thread
 * @param BSP_SNTP_THREAD_PRIORITY      priority level for SNTP client thread
 * @param BSP_SERLMON_THREAD_PRIORITY   priority level for the serial monitor thread
*ad*/
#define BSP_HIGH_PRIORITY               8
#define BSP_MEDIUM_PRIORITY             16
#define BSP_LOW_PRIORITY                24
#define BSP_DEFAULT_PRIORITY            BSP_MEDIUM_PRIORITY
#define BSP_FTP_SERVER_PRIORITY         BSP_MEDIUM_PRIORITY
#define BSP_SNMP_AGENT_PRIORITY         BSP_MEDIUM_PRIORITY
#define BSP_TELNET_SERVER_PRIORITY      BSP_MEDIUM_PRIORITY
#define BSP_SNTP_THREAD_PRIORITY        BSP_MEDIUM_PRIORITY
#define BSP_SERLMON_THREAD_PRIORITY     15

/*ad*
 * This constant sets the UDP port on the UDP debug device.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_UDP_DEBUG_PORT              12345


/*ad*
 * Set BSP_NVRAM_DRIVER to this value if no NVRAM driver is to be
 * built.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_NVRAM_NONE                      0

/*ad*
 * Set BSP_NVRAM_DRIVER to this value if the EEPROM driver is to be
 * built.
 *
 * @since 6.0
 * @external
 * @deprecated 6.1
 * @note The EEPROM drives is not supported in NET+OS 6.1
 * @category BSP:BSP_Configuration
 * @see @link BSP_NVRAM_DRIVER
*ad*/
#define BSP_NVRAM_EEPROM                    1

/*ad*
 * Set BSP_NVRAM_DRIVER to this value if the last sector of flash
 * is to be used for NVRAM.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_NVRAM_DRIVER
*ad*/
#define BSP_NVRAM_LAST_FLASH_SECTOR         2

/*ad*
 * Set BSP_NVRAM_DRIVER to this value to build the serial EEPROM
 * driver.
 *
 * @since 6.0
 * @external
 * @deprecated 6.1
 * @category BSP:BSP_Configuration
 * @see @link BSP_NVRAM_DRIVER
*ad*/
#define BSP_NVRAM_SEEPROM                   3

/*ad*
 * Set BSP_NVRAM_DRIVER to this value to build the serial EEPROM
 * driver with semaphore protection.
 *
 * @since 6.0
 * @external
 * @deprecated 6.1
 * @category BSP:BSP_Configuration
 * @see @link BSP_NVRAM_DRIVER
*ad*/
#define BSP_NVRAM_SEEPROM_WITH_SEMAPHORES   4

/*ad*
 * Set BSP_NVRAM_DRIVER to this value if the last sector of serial flash
 * is to be used for NVRAM.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_NVRAM_DRIVER
*ad*/
#define BSP_NVRAM_LAST_SFLASH_SECTOR         5

/*ad*
 * Set BSP_NVRAM_DRIVER to this value if the I2C EEPROM is used for NVRAM.
 *
 * @since 7.1
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_NVRAM_DRIVER
*ad*/
#define BSP_NVRAM_EEPROM_OVER_I2C            6

/*ad*
 * Set BSP_NVRAM_DRIVER to this value if the NAND FLASH is used for NVRAM.
 *
 * @since 7.1
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_NVRAM_DRIVER
*ad*/
#define BSP_NVRAM_NAND            7

/*ad*
 * Set BSP_NVRAM_DRIVER to this value if the NVRAM functionality
 * will be provided by platform specific functions in "bsp.c".
 * All the custom functions are described in the BSP porting guide.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_NVRAM_DRIVER
*ad*/
#define BSP_NVRAM_CUSTOM                    255



/*ad*
 * This constant determines which NVRAM driver will be built.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
// #define BSP_NVRAM_DRIVER     BSP_NVRAM_EEPROM_OVER_I2C
#define BSP_NVRAM_DRIVER     BSP_NVRAM_NAND
#if (BSP_NVRAM_DRIVER ==  BSP_NVRAM_LAST_SFLASH_SECTOR)
#if BSP_SERIAL_PORT_2  != BSP_SERIAL_SPI_DRIVER
	#error serial port 2 should be used in simple spi driver
#endif
#endif

/*ad*
 * The BSP_UBOOT_ENV_SHARE parameter is applied only if BSP_NVRAM_DRIVER
 * is set to BSP_NVRAM_NAND.
 *
 * Set BSP_UBOOT_ENV_SHARE to TRUE to locate the dialog
 * environment variables in the U-Boot NVPT_NVRAM partition. 
 * NetOS will then store its environment parameters in an OS section of
 * type NVOS_NETOS. 
 *
 * The U-Boot nand layout can be displayed by typing "intnvram printall"
 * in the U-Boot console.
 *
 * When set to TRUE, MAC addresses, IP addressess, and baudrate
 * configurations are shared between U-Boot and NetOS.
 * These parameters are read from U-Boot at NetOS initialization
 * so as to be consistent, and if these parameters are changed in the
 * dialog, they are propagated to the U-Boot area when they are saved
 * from NetOS.
 * If BSP_UBOOT_ENV_SHARE is FALSE or missing, the dialog environment
 * variables are located in a separate and independent NAND block whose
 * location is defined in the nand_parts.h header file.
 *
 * @since 7.2
 * @external
 * @category BSP:BSP_Configuration
 *ad*/

#define BSP_UBOOT_ENV_SHARE TRUE

/*
 * Set this constant to the I2C bus address of the EEPROM.
 *
 * This constant is only used if BSP_NVRAM_DRIVER is set to BSP_NVRAM_EEPROM_OVER_I2C.
 *
 * @since 7.1
 * @external
 * @category BSP:BSP_Configuration:ARM7
 * @index BSP_NVRAM_DRIVER
 * @see @link BSP:BSP_Configuration:ARM7:NvramDriverConfigurations
 */
#define BSP_NVRAM_I2C_EEPROM_ADDRESS    0xA0

/*ad*
 * This constant determines the name of default serial flash device name.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_DEFAULT_FLASH_DEVICE		"defaultSFlash"


/*ad*
 * If BSP_NVRAM_DRIVER is set to BSP_NVRAM_EEPROM.  This constant
 * determines where the part should be mapped to in memory.
 *
 * @since 3.0
 * @external
 * @deprecated 6.1
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_EEPROM_BASE     0x3000000

/*ad*
 * If BSP_NVRAM_DRIVER is set to BSP_NVRAM_EEPROM.  This constant
 * determines the size of the part.
 *
 * @since 3.0
 * @external
 * @deprecated 6.1
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_EEPROM_SIZE     0x2000

/*ad*
 * BSP_FLASH_BANKS sets the number of flash memory banks.  The number of
 * flash banks is the number of flash chips located in memory where one
 * chip follows another in the address map.
 * For example, 2 16-bit chips wired up as 16-bit flash would count as 2
 * Flash banks.  2 16-bit chips wired up as 32-bit flash (in parallel)
 * would count as 1 bank.  4 16-bit chips wired up as 32-bit flash would
 * count as 2 banks (2 pairs of chips where the pairs follow each other in
 * the address map).
 *
 * @since 5.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_FLASH_BANKS  1


/*
 * Offset from the beginning of RAM which we are using for
 * the top of the stack for calling ncc_init, this memory is tested
 * in init.s.
 */
#define BSP_RAM_OFFSET 0x800


/*ad*
 *  BSP_STARTUP_DELAY defines number of additional seconds the system
 *  will wait, before starting the application. Used on NS9750 chip to
 *  let some network activity be received to use as a ramdom seed.
 *
 * @since 5.1
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_STARTUP_DELAY       2


/*ad*
 * If BSP_AUTOMATICALLY_ENABLE_INSTRUCTION_CACHE is set, then the BSP startup code will
 * enable instruction cache if it is present on the processor.  This constant is useful
 * if you will (or may in the future) run your firmware on different chips
 * that have or don't have cache.  Setting it TRUE will cause the BSP to
 * turn cache on if it detects it in the chip.  Setting it FALSE will cause
 * it leave cache off whether the chip supports it or not.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_AUTOMATICALLY_ENABLE_INSTRUCTION_CACHE  TRUE


/*ad*
 * Set this constant to TRUE to include the Real Time Clock driver.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_INCLUDE_RTC_DRIVER          FALSE

/*ad*
 * Set this constant to TRUE to include the native file system in C library.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_INCLUDE_FILESYSTEM_FOR_CLIBRARY          TRUE

/*ad*
 * When the BSP creates a native file system volume, this constant specifies the
 * percentage of the maximum number of inode blocks that can be allocated to
 * store inodes for a volume.  This constant allows specifying the upper limit of
 * the number of blocks reserved to store inodes.  Valid values are from 1 to 100.
 * Refer to NAFSinit_volume_cb for more information.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
 * @see @link NAFSinit_volume_cb
*ad*/
#define BSP_NATIVE_FS_INODE_BLOCK_LIMIT              100

/*ad*
 * When the BSP creates a native file system volume, this constant specifies the
 * maximum number of open directories that the file system will track.  A directory
 * is considered open if there are open files in the directory.  Valid values are
 * from 1 to 64.  Refer to NAFSinit_volume_cb for more information.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
 * @see @link NAFSinit_volume_cb
*ad*/
#define BSP_NATIVE_FS_MAX_OPEN_DIRS                  20

/*ad*
 * When the BSP creates a native file system volume, this constant specifies the
 * the maximum number of open files per directory that the file system will track.  
 * Valid values are from 1 to 64.  Refer to NAFSinit_volume_cb for more information.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
 * @see @link NAFSinit_volume_cb
*ad*/
#define BSP_NATIVE_FS_MAX_OPEN_FILES_PER_DIR         20

/*ad*
 * When the BSP creates a native file system volume, this constant specifies the
 * block size used for the volume.  Valid values are:
 *
 * <list type = "unordered">
 *      <item> NAFS_BLOCK_SIZE_512 </item>
 *      <item> NAFS_BLOCK_SIZE_1K </item>
 *      <item> NAFS_BLOCK_SIZE_2K </item>
 *      <item> NAFS_BLOCK_SIZE_4K </item>
 * </list>
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_NATIVE_FS_BLOCK_SIZE                     NAFS_BLOCK_SIZE_512

/*ad*
 * When the BSP creates the native file system RAM volume, this constant specifies the
 * size of the RAM volume in bytes.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_NATIVE_FS_RAM0_VOLUME_SIZE               2 * 1024 * 1024

/*ad*
 * When the BSP creates the native file system Flash volume, this constant specifies the
 * advanced options to use.  Valid values are the bitwise ORing of the following:
 *
 * <list type = "unordered">
 *      <item> NAFS_MOST_DIRTY_SECTOR </item>
 *      <item> NAFS_RANDOM_DIRTY_SECTOR </item>
 *      <item> NAFS_TRACK_SECTOR_ERASES </item>
 *      <item> NAFS_BACKGROUND_COMPACTING </item>
 * </list>
 *
 * Refer to NAFSinit_volume_cb for more information.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
 * @see @link NAFSinit_volume_cb
*ad*/
#define BSP_NATIVE_FS_FLASH0_OPTIONS                 NAFS_MOST_DIRTY_SECTOR | NAFS_TRACK_SECTOR_ERASES

/*ad*
 * If the BSP_NATIVE_FS_FLASH0_OPTIONS constant includes NAFS_BACKGROUND_COMPACTING, then this
 * constant specifies the percentage of erased blocks in a flash sector to gain in order
 * to trigger the sector compacting process.  Valid values are from 1 to 100.  Refer to
 * NAFSinit_volume_cb for more information.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
 * @see @link NAFSinit_volume_cb
*ad*/
#define BSP_NATIVE_FS_FLASH0_COMPACTING_THRESHOLD    100

/*ad*
 * Set this constant to TRUE to include the FAT file system in C library.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_INCLUDE_FAT_FILESYSTEM_FOR_CLIBRARY          FALSE

    
/*ad*
 * Set BSP_HANDLE_FATAL_ERROR to this value to trigger a reset when fatal
 * errors are encountered.
 *
 * Set BSP_HANDLE_UNEXPECTED_EXCEPTION to this value to trigger a reset
 * when an unexpected exception occurs.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_HANDLE_FATAL_ERROR
 * @see @link BSP_HANDLE_UNEXPECTED_EXCEPTION
*ad*/
#define BSP_ERROR_RESET_UNIT        FALSE

/*ad*
 * Set BSP_HANDLE_UNEXPECTED_EXCEPTION to this value to cause the error
 * handler to report the error.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_HANDLE_FATAL_ERROR
 * @see @link BSP_HANDLE_UNEXPECTED_EXCEPTION
*ad*/
#define BSP_ERROR_REPORT_ERROR      1

/*ad*
 * Set BSP_HANDLE_UNEXPECTED_EXCEPTION to this value to cause the exception
 * handler to attempt to continue execution when an unexpected exception
 * is encountered.  This is not recommended.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
 * @see @link BSP_HANDLE_FATAL_ERROR
 * @see @link BSP_HANDLE_UNEXPECTED_EXCEPTION
*ad*/
#define BSP_ERROR_CONTINUE          2

/*ad*
 * This constant determines how fatal errors are handled.  The possible values for
 * BSP_HANDLE_FATAL_ERROR are:
 *
 *      BSP_ERROR_RESET_UNIT    Unit is reset unless a debugger ICE was detected during
 *                              initialization.  If an ICE was detected, then the LEDs
 *                              are blinked in a pattern that indicates the error
 *                              condition, and it is possible to continue execution in
 *                              the debugger.
 *      BSP_ERROR_REPORT_ERROR  BSP blinks LEDs in a pattern that indicates the type of
 *                              error that occurred.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_HANDLE_FATAL_ERROR              BSP_ERROR_REPORT_ERROR

/*ad*
 * This constant determines how unexpected exceptions are handled.  The possible
 * values for BSP_HANDLE_UNEXPECTED_EXCEPTION are:
 *
 *      BSP_ERROR_RESET_UNIT    Unit is reset unless a debugger ICE was detected during
 *                              initialization.  If an ICE was detected, then the LEDs
 *                              are blinked in a pattern that indicates the error
 *                              condition, and it is possible to continue execution in
 *                              the debugger.
 *      BSP_ERROR_REPORT_ERROR  BSP blinks LEDs in a pattern that indicates the type of
 *                              error that occurred.
 *      BSP_ERROR_CONTINUE      the exception handler attempts to continue execution.  The
 *                              recovery procedure is to adjust the return address of the
 *                              exception to restart execution on the next instruction
 *                              after the one that causes the exception.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_HANDLE_UNEXPECTED_EXCEPTION     BSP_ERROR_REPORT_ERROR

/*ad*
 * Set this constant TRUE to inform NETOS startup process to setup SNTP for retreiving time. 
 * 
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
 *
 * @note This constant must be TRUE and APP_USE_NETWORK_TIME_PROTOCOL must be defined in the application to enable SNTP.
*ad*/
#define BSP_INCLUDE_SNTP    FALSE

/*ad*
 * This is used to define a callback function for SNTP. BSP initializes
 * SNTP with this callback definition.
 *
 * @since 6.0
 * @external
 * @category SNTP
 *
 * @note BSP_INCLUDE_SNTP must be TRUE and APP_USE_NETWORK_TIME_PROTOCOL must be defined in the application to enable SNTP.
*ad*/
#define BSP_SNTP_CALLBACK   NULL

/*ad*
 * This constant determines whether the BSP waits for the stack to be configured before
 * starting the application by calling the applicationStart() function.  Previous versions
 * of NET+OS always waited for the stack to be configured.
 *
 * Your application should not use any network resources until the stack has been
 * configured by setting an IP address on at least one interface.  You can use the
 * customizeAceGetInterfaceAddrInfo() function to determine if an IP address has
 * been assigned to an interface.
 *
 * Set BSP_WAIT_FOR_IP_CONFIG to TRUE cause the BSP to wait for an IP address to be
 * configured on at least one interface before calling applicationStart.
 *
 * Set BSP_WAIT_FOR_IP_CONFIG to FALSE to call applicationStart without waiting for
 * an IP address to be assigned.
 *
 * @since 6.1
 * @external
 * @category BSP:BSP_Configuration
 * @see @link customizeAceGetInterfaceAddrInfo
*ad*/
#define BSP_WAIT_FOR_IP_CONFIG  TRUE



/*ad*
 * Set this constant to the number of seconds the BSP should delay calling 
 * applicationStart when the only IP version 4 address available was derived from Auto-IP.
 *
 * NET+OS uses two methods for acquiring IP version 4 addresses from the network:  
 * DHCP and Auto-IP.  Addresses acquired with DHCP are better than addresses acquired
 * with Auto-IP because packets with Auto-IP addresses will be discarded by routers.
 * Auto-IP is used mainly on very simple networks with no routers, or with Ethernet 
 * cross cable connections between two devices.
 *
 * NET+OS will use the DHCP address by default when it has both a DHCP address and an 
 * Auto-IP address.  However, if your application opens a socket when there is only 
 * an Auto-IP address, then that socket will be bound to the Auto-IP address, 
 * even if a DHCP address is acquired later.  The application will not be able to use 
 * the socket to communicate with a device located across a router.
 *
 * You can disable Auto-IP by updating the IAM configuration settings stored in NVRAM.
 *
 * If you use both DHCP and Auto-IP, then you may want wait for a DHCP address before
 * letting your application start.  Some DHCP servers are slow to respond.  Set
 * BSP_AUTOIP_DELAY to the number of seconds NET+OS should wait for DHCP to acquire
 * an IP address after Auto-IP has an address.  Set this constant to 0 if NET+OS
 * should immediately call applicationStart as soon as any IP version 4 address is
 * available.  Set the constant to a large value if a DHCP address is prefered.  
 * Disable Auto-IP entirely if you do not want to use Auto-IP addresses.
 *
 * If both Ethernet and wireless interfaces are present, then NET+OS will wait for
 * at least one IPv4 address, which was not acquired via Auto-IP, to be available on 
 * one of them. 
 *
 * @since 7.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_AUTOIP_DELAY        (30)







/*ad*
 * This constant determines how the Ethernet link is monitored.
 * Set BSP_USE_PHY_INTERRUPT to TRUE to use PHY interupt to monitor the Ethernet link.
 * Set BSP_USE_PHY_INTERRUPT to FALSE to use 500ms ThreadX timer to monitor the link.
 *
 * BSP_USE_PHY_INTERRUPT should be set to TRUE only on platforms that support PHY interupt.
 *
 * @since 6.0
 * @external
 * @category BSP:BSP_Configuration:EthernetDriver
*ad*/
#define BSP_USE_PHY_INTERRUPT   TRUE

/*ad*
 * This constant sets increased MII drive strength for LXT971A PHY.
 *
 * A higher Media Independent Interface (MII) drive strength may be desired in some designs 
 * to drive signals over longer PCB trace lengths, or over high-capacitive loads, through 
 * multiple vias, or through a connector. 
 *
 * BSP_INCREASED_MII_DRIVE_STRENGTH must be set to TRUE for NS9750 development board.
 *
 * @since 6.1
 * @external
 * @category BSP:BSP_Configuration:EthernetDriver
*ad*/
#define BSP_INCREASED_MII_DRIVE_STRENGTH   TRUE


/*ad*
 * This constant enables IAM's initial IPv4 address conflict detection(ACD) for static addresses.
 * Initial ACD is performed by sending BSP_DEFAULT_ETH_NUM_ADDR_PROBES ARP probes at  
 * BSP_DEFAULT_ETH_PROBE_INTERVAL_MS intervals. An ARP probe is an ARP request with source protocol
 * address equal to zero. IAM always performs initial address conflict detection
 * for DHCP, Auto IP. Stateless auto-configured IPv6 addresses are tested for initial conflicts using
 * duplicate address detection(DAD), which is always enabled as well. 
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_ENABLE_ADDR_CONFLICT_DETECTION          TRUE

/*ad*
 * This constant enables IAM's ongoing IPv4 address conflict detection(ACD) after an interface has been
 * configured with a static or DHCP IPv4 address. Ongoing ACD is performed by checking for
 * for ARP requests, or replies coming from any interface, not originated by this host, 
 * whose ARP sender addresses are the same as the configured IPv4 address. IAM's ongoing ACD does
 * not send ARP probes to check for conflicts. An ARP probe is an ARP request with source protocol
 * address equal to zero. Ongoing ACD is always performed for Auto IP addresses. Stateless auto-configured
 * IPv6 addresses are tested for ongoing conflicts using duplicate address detection(DAD), which is
 * always enabled as well. 
 *
 * @since 7.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_ENABLE_ONGOING_ADDR_CONFLICT_DETECTION  FALSE


/*ad*
 * This constant specifies the default number of ARP probes IAM sends to detect IP address 
 * conflict on Ethernet and wireless interfaces. 
 * An ARP probe is an ARP request with source protocol address equal to zero.
 * This constant is used for static, DCHP, and Auto IP
 * IPv4 addresses, however, BSP_ENABLE_ADDR_CONFLICT_DETECTION must be set to TRUE for static 
 * addresses to use it. When this constant is set to zero, four ARP probes are sent by default.  
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_DEFAULT_ETH_NUM_ADDR_PROBES             4

/*ad*
 * This constant specifies the interval, in milliseconds, between ARP probes, sent by IAM to detect
 * initial IPv4 address conflicts on Ethernet and wireless interfaces.
 * An ARP probe is an ARP request with source protocol address equal to zero.
 * This constant is used for static, DCHP, and Auto IP
 * IPv4 addresses, however, BSP_ENABLE_ADDR_CONFLICT_DETECTION must be set to TRUE for static 
 * addresses to use it. When this constant is set to zero, ARP probes are sent at 200 ms 
 * intervals by default.  
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_DEFAULT_ETH_PROBE_INTERVAL_MS           200

/*ad*
 * This constant specifies the default number of ARP address announcements sent by IAM after 
 * an IPv4 address has been configured on an interface. This constant is used for static,
 * DHCP, and Auto IP addresses. IAM always sends one ARP announcement after configuring an IPv4 address,
 * therefore setting this constant to one forces IAM to send two announcements.
 *
 * An ARP announcement is an ARP request with both source and destination protocol address 
 * equal the interface IP address.
 * 
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_DEFAULT_ETH_NUM_ADDR_ANNOUNCEMENTS      4

/*ad*
 * This constant specifies the interval, in milliseconds, between ARP address announcements, sent 
 * by IAM after an IPv4 address has been configured. This constant is used for static, DHCP, and Auto IP
 * addresses.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_DEFAULT_ETH_ANNOUNCE_INTERVAL_MS        2000

/*ad*
 * This constant has been obsoleted in NET+OS 7.0
 *
 * @obsolete 7.0
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_MAX_ADDR_CONFLICTS                      3


/*ad*
 * This constant determines how long the watchdog timer waits before resetting the unit.
 * The unit of time is in seconds.
 *
 * @since 6.1
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_WATCHDOG_TIMEOUT_IN_SECONDS 30

/*ad*
 * Set this to TRUE to run the system POST tests during startup.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_POST_TEST          FALSE

/*ad*
 * Set this to TRUE to run the dialog code during startup.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_ENABLE_DIALOG          TRUE


/*ad*
 * Set this constant to TRUE to let the bootloader to download application
 * image from network only.
 *
 * @note     After the bootloader successfully downloads
 *           the application image, it reads the header of the image to determine if
 *           the image needs to be written to flash (or sflash) or stays in RAM. User
 *           needs to customize the field "WriteToFlash" in bootldr.dat in the platform 
 *           directory to suit their application.
 * 
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration
 * @see @link blFlags
*ad*/
#define BSP_BOOTLOADER_BOOT_FROM_NETWORK_ONLY        FALSE


/*ad*
 * Set this constant to TRUE to enable support for other protocols besides
 * TCP/IP.
 *
 * If this constant is set to FALSE, then the Ethernet driver will pass up
 * all received packets to the TCP/IP stack.  This is the recommended setting
 * for all TCP/IP applications.
 *
 * Set this constant to TRUE if your application needs to support other 
 * protocols.  This will enable code in the Ethernet driver that examines the
 * protocol ID of each received packet to determine which protocol it is 
 * addressed to.  The Ethernet driver will send packets to the protocol stack
 * that has been registered to receive them with @link NAEthAddPacketType.
 * TCP/IP is automatically registered to receive IP, ARP, and RARP packets.
 *
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration:EthernetDriver
 * @see @link NAEthAddPacketType
 * @see @link BSP_WANT_2ND_ETHERNET_CHECKSUM
*ad*/
#define BSP_ENABLE_ETHERNET_PACKET_TYPES        FALSE

/*ad*
 * Set this constant to TRUE to force the Ethernet driver to perform a second
 * CRC checksum check of incoming packets.  
 *
 * Under very rare conditions, the NET50 and NS7520 chips will corrupt data in
 * Ethernet receive packets.  This error is not detected by the Ethernet H/W.
 * See the errata on these parts for more information.  Set BSP_WANT_2ND_ETHERNET_CHECKSUM
 * to TRUE to have the Ethernet driver check received packets by performing a second 
 * CRC check.  The driver will discard packets that fail the CRC check.
 *
 * Note that UDP and TCP both perform their own checksums on received data, so this
 * check is NOT necessary for TCP/IP based applications.
 *
 * This constant is only valid for NET50 and NS7520 platforms.  It is ignored on all
 * others.
 *
 * If @link BSP_ENABLE_ETHERNET_PACKET_TYPES and BSP_WANT_2ND_ETHERNET_CHECKSUM are 
 * both set, the Ethernet driver will determine whether it should checksum packets 
 * by determining which protocol the packet is addressed to and then examining the 
 * want2ndCrcCheck field of the @link ethPacketType structure passed to @link 
 * NAEthAddPacketType when the packet type and protocol were registered.  
 * If @link BSP_ENABLE_ETHERNET_PACKET_TYPES is not set, but BSP_WANT_2ND_ETHERNET_CHECKSUM
 * is, then the Ethernet driver will perform a CRC checksum on all received packets.
 *
 * @note    There is a performance penalty when this feature is enabled.
 *          You do not need to enable this feature if your application only uses
 *          TCP/IP since TCP and UDP both perform checksums of incoming packets.
 * 
 * @since 6.2
 * @external
 * @category BSP:BSP_Configuration:EthernetDriver
 * @see @link NAEthAddPacketType
 * @see @link BSP_ENABLE_ETHERNET_PACKET_TYPES
*ad*/
#define BSP_WANT_2ND_ETHERNET_CHECKSUM       FALSE

/*ad*
 * Set this constant to TRUE to queue received Ethernet TCP/IP packets and 
 * pass them up to the TCP/IP stack in a lower priority thread. The priority of
 * the receive queue thread is determined by the BSP_ETH_RECV_QUEUE_THREAD_PRIORITY.
 *
 * This mode is recommended when another protocol stack is running in parallel 
 * with TCP/IP and the bypass packets need higher priority over TCP/IP packets.
 *
 * @since 6.3
 * @external
 * @category BSP:BSP_Configuration:EthernetDriver:EthernetBypass
 * @see @link NAEthRegisterCustomPacketFn
 * @see @link NAEthAddPacketType
*ad*/
#define BSP_USE_ETHERNET_RECV_QUEUE  FALSE

/*ad*
 * This constant defines the priority of the Ethernet receive queue thread,
 * if BSP_USE_ETHERNET_RECV_QUEUE is set to TRUE. 
 *
 * @since 6.3
 * @external
 * @category BSP:BSP_Configuration:EthernetDriver:EthernetBypass
*ad*/
#define BSP_ETHERNET_RECV_QUEUE_THREAD_PRIORITY  9

/*ad*
 * Set this constant to TRUE to enable the Ethernet transmit bypass of the TCP/IP stack. 
 * Bypass packets are sent using the NAEthBypassTransmit routine. 
 *
 * To avoid dependancy on TCP/IP stack recources for bypass packets, different queues are used
 * for bypass and TCP/IP packets. Total number of transmit DMA buffers is split to be used by
 * the Ethernet bypass and the TCP/IP stack. 
 *
 * The BSP_ETHERNET_BYPASS_SEND_BUFFERS number of DMA buffers is used for bypass. 
 * The rest of the DMA buffers - 1 are used for TCP/IP. To calculate the actual number of packets 
 * in DMA used for bypass, or TCP/IP divide the number buffers by the maximum number of chained 
 * packets (see eth.h).
 *
 * All bypass packets, submitted to the driver, above the once that fit into BSP_ETHERNET_BYPASS_SEND_BUFFERS,
 * are queued.
 *
 * @since 6.3
 * @external
 * @category BSP:BSP_Configuration:EthernetDriver:EthernetBypass
*ad*/
#define BSP_ENABLE_ETHERNET_BYPASS            FALSE

/*ad*
 * The number of Ethernet transmit DMA buffer descriptors, allocated for bypass packets,
 * if BSP_ENABLE_ETHERNET_BYPASS is enabled.
 *
 * @since 6.3
 * @external
 * @category BSP:BSP_Configuration:EthernetDriver:EthernetBypass
 * @see @link NAEthBypassTransmit
*ad*/
#define BSP_ETHERNET_BYPASS_SEND_BUFFERS       54


/* These settings are defined in init_settings.h */
#define     mem_power_level_normal	MEM_POWER_LEVEL_NORMAL
#define     mem_power_level_low		MEM_POWER_LEVEL_LOW


/* These settings are defined in init_settings.h */
#define    mem_width_16				MEM_WIDTH_16
#define    mem_width_32				MEM_WIDTH_32


/* These settings are defined in init_settings.h */
#define    x8						MEM_TYPE_x8
#define    x16						MEM_TYPE_x16
#define    x32						MEM_TYPE_x32


/*ad*
 * This structure is used to define the PrimeCell Memory Controller PL172
 * register settings for static memory. The regsiters are described in detail
 * in the PL172 Technical Reference Manual in section 3.2.
 *
 * @param enabled 1=Static Memory or periperhal on this chip select,
 * 0=Chip Select Not Connected.
 *
 * @param StaticConfig Static memory configuration register
 *
 * @param StaticWaitWen Static Memory Write Enable Delay Register
 *
 * @param StaticWaitOen Static Memory Output Enable Delay Register
 *
 * @param StaticWaitRd  Static Memory Read Delay Registers
 *
 * @param StaticWaitPage Static Memory Page Mode Read Delay Register
 *
 * @param StaticWaitWr Static Memory Write Delay Registers
 *
 * @param StaticWaitTurn Static Memory Turn Round Delay Register
 *
 * @param StaticBaseAddess For chip selects other than cs1, we need to know
 * the base address and the mask.  We get the base address of cs1 from the
 * linker option file.
 *
 * @param StaticMask Mask for the chip select.
 *
 * @external
 * @category BSP
 * @since 6.1
 * @include <bsp.h>
 * @see @link NAStaticMemoryType
*ad*/
typedef struct
{
    unsigned int const enabled;

    unsigned int const StaticConfig;

    unsigned int const StaticWaitWen;

    unsigned int const StaticWaitOen;

    unsigned int const StaticWaitRd;

    unsigned int const StaticWaitPage;

    unsigned int const StaticWaitWr;

    unsigned int const StaticWaitTurn;

    unsigned int const StaticBaseAddess;

    unsigned int const StaticMask;

} NAStaticMemoryType;

/* Index into static memory table */
#define CS_0_INDEX 0
#define CS_1_INDEX 1
#define CS_2_INDEX 2
#define CS_3_INDEX 3

/* Index into dynamic memory table */
#define CS_4_INDEX 0
#define CS_5_INDEX 1
#define CS_6_INDEX 2
#define CS_7_INDEX 3

/*ad*
 * Maximum number of PWM channels supported
 *
 * @since 6.2
 * @include 	<bsp.h>
 * @external
 * @category BSP:BSP_Configuration
 *
*ad*/
#define BSP_PWM_MAXIMUM_CHANNELS     4     

/*ad*
 * Bootloader flash recovery using TFTP mode (default value)
 *
 * @since 6.3
 * @include 	<bsp.h>
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_BOOTLOADER_TFTP_RECOVERY	0x0001

/*ad*
 * Bootloader flash recovery using serial mode. The default serial recovery uses
 * XModem communication protocol
 *
 * @since 6.3
 * @include 	<bsp.h>
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_BOOTLOADER_SERIAL_RECOVERY 	0x0002

/*ad*
 * Try the all flash recovery modes in the following order
 *
 * <list type="unordered"> 
 *  <item> TFTP </item>
 *  <item> Serial </item>
 * </list>
 *
 * @since 6.3
 * @include 	<bsp.h>
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_BOOTLOADER_TRY_ALL   	(BSP_BOOTLOADER_TFTP_RECOVERY | BSP_BOOTLOADER_SERIAL_RECOVERY)

/*ad*
 * Bootloader flash recovery mode. The default is set to BSP_BOOTLOADER_TRY_ALL
 *
 * @since 6.3
 * @include 	<bsp.h>
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_BOOTLOADER_RECOVERY_METHOD  BSP_BOOTLOADER_TRY_ALL

/*ad*
 * ENABLE_ADDP_SERVER is used to activate the ADDP server.
 * 
 * Set ENABLE_ADDP_SERVER to FALSE to disable the ADDP server.
 *
 * @external
 * @category ADDPConfiguration
 * @since 6.2
 *ad*/
#define ENABLE_ADDP_SERVER TRUE

#if (ENABLE_ADDP_SERVER == TRUE) && (BSP_WANT_WIRELESS == FALSE) && (BSP_WANT_ETHERNET == FALSE)
#error ADDP Server can be used only when Ethernet or wireless is enabled
#endif

/*ad*
 * This structure defines the internals of the GUID data structure.
 *
 * @since 6.2
 * @external
 * @category ADDPConfiguration
 * @param Data1	Specifies the first 8 hexadecimal digits of the GUID. 
 * @param Data2	Specifies the first group of 4 hexadecimal digits of the GUID. 
 * @param Data3	Specifies the second group of 4 hexadecimal digits of the GUID. 
 * @param Data4	Array of eight elements. The first two elements contain the third group of 4 hexadecimal digits of the GUID. The remaining six elements contain the final 12 hexadecimal digits of the GUID. 
 *
*ad*/
typedef struct _GUID{
   unsigned long	Data1;
   unsigned short	Data2;
   unsigned short	Data3;
   unsigned char	Data4[8];
} GUID, UUID;

/*ad*
 * ADDP_OEM_GUID Is the unique GUID assigned to this user.  This value is used
 * by ADDP to communicate with devices made by this user.  Currently set to 
 * Netsilicon GUID.
 * 
 * This value should be modified by the user to reflect their own GUID.
 * The GUID value is broken up based on the GUID structure defined above.
 *
 * @external
 * @category ADDPConfiguration
 * @since 6.2
 *ad*/

#define ADDP_OEM_GUID  {0xbf6db409, 0xc83d, 0x44a3, {0xa3, 0x6d, 0x21, 0x79, 0x7d, 0x2f, 0x73, 0xf9}}

/*ad*
 * ADDP_COOKIE Is the magic cookie.  This value is used by ADDP to communicate
 * with devices made by this user.  Currently set to DVKT.
 * 
 * This value should match the value defined in the ADDP client library.
 *
 * @external
 * @category ADDPConfiguration
 * @since 6.2
 *ad*/
#define ADDP_COOKIE	0x44564B54

/*ad*
 * ADDP_HARDWARE_NAME Is the name for the hardware.  
 * 
 * This value should be modified by the user to reflect their hardware name.
 *
 * @external
 * @category ADDPConfiguration
 * @since 6.2
 *ad*/

#define ADDP_HARDWARE_NAME	"ConnectCore 9P 9360"

/*ad*
 * ADDP_FEP_REV Is the FEP revision value.
 * 
 * The value should be modified by the user to represent the FEP revision for 
 * this application.
 *
 * @external
 * @category ADDPConfiguration
 * @since 6.2
 *ad*/

#define ADDP_FEP_REV		"Unknown"

/*ad*
 * ADDP_HARDWARE_REV Is the hardware revision of the device.
 * 
 * The value should be modified by the user to represent the hardware revision
 * for this device.
 *
 * @external
 * @category ADDPConfiguration
 * @since 6.2
 *ad*/

#define ADDP_HARDWARE_REV	1


/*ad*
 * This enables the deep sleep mode. Before setting this to true make sure one 
 * GPIO pin is configured to trigger external reset and another GPIO pin is
 * configured to hold the status and pin 68 is configured to function mode
 * 1 so that the SDRAM CKE signal is under direct control of the memory 
 * controller.
 * 
 * @since 7.0
 * @include 	<bsp.h>
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_ENABLE_SLEEP_EXTERNAL_RESET    FALSE


/*ad*
 * Set this to TRUE if you want to bypass the auto-negotiation during sleep 
 * wakeup mode. This will sped up the initialization by around 2 seconds. 
 * Ethernet may not configure correctly if you switch the device in sleep mode 
 * to the network whose duplex mode is fixed and is different from the 
 * device's previously configured duplex mode. 
 *
 * @since 7.0
 * @include 	<bsp.h>
 * @external
 * @category BSP:BSP_Configuration
*ad*/
#define BSP_WAKEUP_DISABLE_AUTO_NEGOTIATION   FALSE

extern const NAStaticMemoryType  NAStaticMemoryTable[];

/*
 * Offset into RAM which we write/read to determine if RAM is
 * present.
 */
#define MEMC_READ_LOCATION 4096

/* Number of 4-byte words in the memory controller cache */
#define MEMC_CTRL_BUF_LEN  64

#define BSP_RAM_BASE 0x00000000

/*
 * flag to control nccInit chip select logic
 */
#define BSP_SEEK_SDRAM  TRUE

extern unsigned int _NABspFlashSize;
extern unsigned int _NABspFlashStart;
extern unsigned int _NABspRamSize;
extern unsigned int _NARomBuild;

extern void bsp_printf(char *outString);    
extern int mprintf(const char *f, ...);

extern int NAFastMemoryTest(WORD32 from,  WORD32 to);

extern void postIsrHandler (void);

/*default serial flash configuration*/
extern NASpiDeviceConfigType NADefaultSflashDevice;

/*ad*
 * Builds the CLI as part of the BSP library when set to TRUE.
 *
 * @category CLI
 * @since 7.0
 * @include "cliapi.h"
 * @platform all
 * @library  libbsp
 * @external
 *
*ad*/
#define BSP_CLI_INCLUDE_IN_BSP  TRUE

/*ad*
 * Builds Telnet capability as part of the CLI when set to TRUE.
 * libtelnsvr.a must be linked in with this option. 
 *
 * @category CLI
 * @since 7.0
 * @include "cliapi.h"
 * @platform all
 * @library  libbsp
 * @external
 *
 * @see @link BSP_CLI_INCLUDE_IN_BSP
 * @see @link BSP_CLI_ENABLE
*ad*/
#define BSP_CLI_TELNET_ENABLE   TRUE

/*ad*
 * Builds serial capability as part of the CLI when set to TRUE.
 *
 * @category CLI
 * @since 7.0
 * @include "cliapi.h"
 * @platform all
 * @library  libbsp
 * @external
 *
 * @see @link BSP_CLI_INCLUDE_IN_BSP
 * @see @link BSP_CLI_ENABLE
*ad*/
#define BSP_CLI_SERIAL_ENABLE   FALSE


/*ad*
 * Starts the CLI before applicationStart() is called from the ROOT task.
 * The CLI api function @link naCliOpen cannot be called from 
 * the application if set to TRUE. A maximum of five Telnet sessions is permitted
 * if the Telnet server is enabled.
 *
 *
 * @category CLI
 * @since 7.0
 * @include "cliapi.h"
 * @platform all
 * @library  libbsp
 * @external
 *
 * @see @link BSP_CLI_TELNET_ENABLE
 * @see @link BSP_CLI_SERIAL_ENABLE
*ad*/
#define BSP_CLI_ENABLE  TRUE

/*ad*
 * When BSP_CLI_ENABLE is set to TRUE, this macros determines the
 * Telnet server port.
 *
 * @category CLI
 * @since 7.0
 * @include "cliapi.h"
 * @platform all
 * @library  libbsp
 * @external
 *
 * @see @link BSP_CLI_TELNET_ENABLE
*ad*/
#define BSP_CLI_TELNET_PORT     23 

/*ad*
 * When BSP_CLI_ENABLE is set to TRUE, this macros determines the
 * com port on which the CLI will be accessed. Make sure 
 * this port is not in conflict with printf's, which is usually
 * "/com/0".  
 *
 * @category CLI
 * @since 7.0
 * @include "cliapi.h"
 * @platform all
 * @library  libbsp
 * @external
 *
 * @see @link BSP_CLI_SERIAL_ENABLE
*ad*/
#define BSP_CLI_COM_PORT        "/com/1"

/*
 * When BSP_NOR_FLASH_DRIVER_ENABLE is set to TRUE, the nor flash
 * file system driver is built into the bsp. The flash partitioning is
 * applied in the startfilesystem.c file.
 */

#define	BSP_NOR_FLASH_DRIVER_ENABLE	FALSE

/*ad*
 * When BSP_YAFFS_DRIVER_ENABLE is set to TRUE, the yaffs nand flash
 * file system driver is built into the bsp. The flash partitioning is
 * applied in the yaffscfg2k.c file.
 *
 * @since 7.2
 * @include     <bsp.h>
 * @external
 * @category BSP:BSP_Configuration
 *ad*/
 
#define	BSP_YAFFS_DRIVER_ENABLE  TRUE

/*ad*
 * BSP_START_OF_DEV_BOARD_PARAMETERS is used to set the offset into
 * EEPROM space for the start of the board parameters structures. THis may
 * be done to share the EEPROM with another element on the board.
 * The value set is the byte offset into the EEPROM where NetOS will
 * write its parameters.
 * This replaces START_OF_DEV_BOARD_PARAMETERS previously defined
 * in boardParams.h.
 *
 * @since 7.2
 * @include     <bsp.h>
 * @external
 * @category BSP:BSP_Configuration
 *ad*/

#define	BSP_START_OF_DEV_BOARD_PARAMETERS   0
// #define	BSP_START_OF_DEV_BOARD_PARAMETERS   0xD00


#ifdef __cplusplus
}
#endif

#endif


